# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "UTCDateTime",
    "UTCDateTimeAlias",
    ]

import pytz

from storm.expr import Alias
from storm.properties import DateTime
from storm.variables import DateTimeVariable


class UTCDateTimeVariable(DateTimeVariable):

    def __init__(self, *args, **kwargs):
        kwargs["tzinfo"] = pytz.timezone("UTC")
        super(UTCDateTimeVariable, self).__init__(*args, **kwargs)


class UTCDateTime(DateTime):
    """Represent a datetime in the UTC timezone."""

    variable_class = UTCDateTimeVariable


# Expr statements in Storm do not check their arguments to load objects
# based on the variable class, so an explicit workaround is necessary.
#
# See: https://bugs.launchpad.net/storm/+bug/798938
class UTCDateTimeAlias(Alias):
    """Represent an expression as a UTC datetime."""

    variable_factory = UTCDateTimeVariable
