# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ensure_schemas_configured",
    "get_schema",
    ]

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from lpresults.database.schema import global_schema_manager


def configure_schemas(settings):
    if not hasattr(settings, "STORM_SCHEMAS"):
        raise ImproperlyConfigured(
            "You need to specify STORM_SCHEMAS in your Django settings file.")

    for name, package in settings.STORM_SCHEMAS.iteritems():
        global_schema_manager.setDefaultPackage(name, package)


have_configured_schemas = False


def ensure_schemas_configured():
    global have_configured_schemas
    if not have_configured_schemas:
        configure_schemas(settings)
        have_configured_schemas = True


def get_schema(name):
    # Make sure that schemas have been configured.
    ensure_schemas_configured()

    return global_schema_manager.get(name)
