# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DBEnum",
    ]

from storm.properties import SimpleProperty
from storm.variables import Variable

from lazr.enum import (
    DBEnumeratedType,
    DBItem,
    )


class DBEnumVariable(Variable):
    """A Storm variable class representing a DBEnumeratedType."""
    __slots__ = ("_enum",)

    def __init__(self, *args, **kwargs):
        self._enum = kwargs.pop("enum")
        if not issubclass(self._enum, DBEnumeratedType):
            raise TypeError(
                "%r must be a DBEnumeratedType: %r"
                % (self._enum, type(self._enum)))
        super(DBEnumVariable, self).__init__(*args, **kwargs)

    def parse_set(self, value, from_db):
        if from_db:
            return self._enum.items[value]
        else:
            if not isinstance(value, DBItem):
                raise TypeError("Not a DBItem: %r" % (value,))
            if self._enum != value.enum:
                raise TypeError("DBItem from wrong type, %r != %r" % (
                        self._enum.name, value.enum.name))
            return value

    def parse_get(self, value, to_db):
        if to_db:
            return value.value
        else:
            return value


class DBEnum(SimpleProperty):

    variable_class = DBEnumVariable
