# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import absolute_import

__metaclass__ = type

__all__ = [
    "get_store",
    "get_stores",
    "get_store_names",
    ]

from django.conf import settings

from storm.zope.interfaces import (
    IZStorm,
    ZStormError,
    )

from zope.component import getUtility


def get_store(name, autocommit=False):
    """Get the store object to the given name.

    :param name: Store name.
    :param autocommit: Optionally, autocommit isolation level.
    """
    if autocommit:
        name += "-autocommit"

    try:
        return getUtility(IZStorm).get(name)
    except ZStormError:
        aliases = getattr(settings, "STORM_ALIASES", {})
        if name not in aliases:
            raise

        return getUtility(IZStorm).get(aliases[name])


def get_stores(name):
    """Get the store name and object tuples for each store matching
    the given name.

    :param name: Store name.
    """
    zstorm = getUtility(IZStorm)
    for store_name in get_store_names():
        parts = store_name.split("-")
        if parts[-1].isdigit():
            parts.pop()

        part_name = "-".join(parts)
        if part_name == name:
            yield (store_name, zstorm.get(store_name))


def get_store_names():
    """Get all store names."""
    zstorm = getUtility(IZStorm)
    return sorted([name for name in zstorm.get_default_uris().keys()
        if not name.endswith("-autocommit")])
