# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from storm.exceptions import ExprError
from storm.expr import compile
from storm.properties import (
    Int,
    List,
    )

from lpresults.database.array import (
    ArrayConcat,
    ArrayContains,
    ArrayIsContainedBy,
    ArrayOverlaps,
    )


class Foo:
    __storm_table__ = "foo"

    id = Int(primary=True)
    ints = List(type=Int())


class TestArray(TestCase):

    def test_concat(self):
        expr = ArrayConcat(Foo.ints, [1])
        statement = compile(expr)
        self.assertEquals(statement, u"foo.ints || ?")

    def test_contains(self):
        expr = ArrayContains(Foo.ints, [1])
        statement = compile(expr)
        self.assertEquals(statement, u"foo.ints @> ?")

    def test_isContainedBy(self):
        expr = ArrayIsContainedBy(Foo.ints, [1])
        statement = compile(expr)
        self.assertEquals(statement, u"foo.ints <@ ?")

    def test_overlaps(self):
        expr = ArrayOverlaps(Foo.ints, [1])
        statement = compile(expr)
        self.assertEquals(statement, u"foo.ints && ?")

    def test_compile(self):
        expr = ArrayOverlaps([0], [1])
        self.assertRaises(ExprError, compile, expr)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
