# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.database.textgrams import (
    get_word_grams,
    get_text_grams,
    )


class TestWordGrams(TestCase):

    def test_words(self):
        results = get_word_grams("")
        self.assertEquals(results, [])

        results = get_word_grams("A")
        self.assertEquals(results, ["A"])

        results = get_word_grams("ab")
        self.assertEquals(results, ["ab"])

        results = get_word_grams("abc")
        self.assertEquals(len(results), 1)
        self.assertEquals(results, ["abc"])

        results = get_word_grams("abcd")
        self.assertEquals(len(results), 2)
        self.assertEquals(results, ["abc", "bcd"])

        results = get_word_grams("abcde")
        self.assertEquals(len(results), 3)
        self.assertEquals(results, ["abc", "bcd", "cde"])

    def test_long(self):
        results = get_word_grams("supercalifragilistic")
        self.assertEquals(
            len(results), len("supercalifragilistic") - 2)

    def test_unique(self):
        results = get_word_grams("abababababababa")
        self.assertEquals(len(results), 2)
        results = get_word_grams("abababababababa", unique=False)
        self.assertEquals(len(results), 13)

    def test_unicode(self):
        unicode_text = u"t\N{LATIN SMALL LETTER E WITH ACUTE}est"
        results = get_word_grams(unicode_text)
        self.assertEquals(len(results), 3)


class TestTextGrams(TestCase):

    def test_sentence(self):
        result = get_text_grams("This is a test.")
        self.assertEquals(result, "Thi his is a tes est st.".split())

    def test_newlines(self):
        result = get_text_grams("This\nis\na\ntest.")
        self.assertEquals(result, "Thi his is a tes est st.".split())

    def test_unicode(self):
        unicode_text = u"This is a t\N{LATIN SMALL LETTER E WITH ACUTE}st."
        result = get_text_grams(unicode_text)
        self.assertEquals(result, u"Thi his is a t\xe9s \xe9st st.".split())


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
