# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IDeviceProduct",
    "IHasDeviceProduct",
    "IDeviceProductTarget"
    ]

from zope.interface import (
    Attribute,
    Interface,
    )

from lpresults.hardware.interfaces.devicevendor import IHasDeviceVendor


class IDeviceProduct(Interface):

    name = Attribute("Device product name")


class IHasDeviceProduct(IHasDeviceVendor):

    product_name = Attribute("Device product name")
    subvendor_name = Attribute("Subdevice vendor name")
    subproduct_name = Attribute("Subdevice product name")


class IDeviceProductTarget(Interface):
    """A target for which device products can be created."""

    def getBusId():
        """Get the bus identifier for this target."""

    def getProductId(
        name, product_id, vendor_id, subproduct_id=None, subvendor_id=None):
        """Get the product identifier under this target.

        :param name: Product name.
        :param product_id: Product identifier.
        :param vendor_id: Vendor identifier.
        :param subproduct_id: Optionally, subproduct identifier.
        :param subvendor_id: Optionally, subvendor identifier.
        """

    def getProduct(
        name, product_id, vendor_id, subproduct_id=None, subvendor_id=None):
        """Get the product object under this target.

        :param name: Product name.
        :param product_id: Product identifier.
        :param vendor_id: Vendor identifier.
        :param subproduct_id: Optionally, subproduct identifier.
        :param subvendor_id: Optionally, subvendor identifier.
        """
