# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IProcessor",
    "IHasProcessor",
    "IProcessorTarget",
    ]

from zope.interface import Interface
from zope.schema import (
    Int,
    Text,
    TextLine,
    )

from lazr.restful.declarations import (
    export_as_webservice_entry,
    export_write_operation,
    exported,
    operation_parameters,
    )
from lazr.restful.fields import Reference
from lazr.restful.interface import copy_field

from lpresults.hardware.interfaces.processorplatform import IProcessorPlatform


class IProcessor(Interface):

    export_as_webservice_entry()

    platform = exported(
        Reference(
            schema=IProcessorPlatform,
            title=u"Platform",
            required=True,
            readonly=True,
            description=u"The platform of the processor."))

    vendor = exported(
        TextLine(
            title=u"Vendor",
            required=True,
            readonly=True,
            description=u"The vendor for this processor."))

    model = exported(
        TextLine(
            title=u"Model",
            required=True,
            readonly=True,
            description=u"The model for this processor."))

    model_number = exported(
        TextLine(
            title=u"Model Number",
            required=True,
            readonly=True,
            description=u"Model number for this processor."))

    model_version = exported(
        TextLine(
            title=u"Model Version",
            required=True,
            readonly=True,
            description=u"Model version for this processor."))

    model_revision = exported(
        TextLine(
            title=u"Model Revision",
            required=True,
            readonly=True,
            description=u"Model revision for this processor."))

    bogomips = exported(
        Int(
            title=u"Bogomips",
            required=True,
            readonly=True,
            description=u"The number of bogomips for this processor."))

    cache = exported(
        Int(
            title=u"Cache",
            required=True,
            readonly=True,
            description=u"The amount of cache for this processor."))

    speed = exported(
        Int(
            title=u"Speed",
            required=True,
            readonly=True,
            description=u"The speed in megahertz for this processor."))

    other = exported(
        Text(
            title=u"Other",
            description=u"Other information for this processor."))


class IHasProcessor(Interface):

    processor = exported(
        Reference(
            schema=IProcessor,
            title=u"Processor",
            required=True,
            readonly=True,
            description=u"The processor for this target."))

    processor_count = exported(
        Int(
            title=u"Number of processors",
            required=True,
            readonly=True,
            description=u"Numberof processor instances in the target."))


class IProcessorTarget(IHasProcessor):

    @operation_parameters(
        vendor=copy_field(IProcessor["vendor"]),
        model=copy_field(IProcessor["model"]),
        model_number=copy_field(IProcessor["model_number"]),
        model_version=copy_field(IProcessor["model_version"]),
        model_revision=copy_field(IProcessor["model_revision"]),
        bogomips=copy_field(IProcessor["bogomips"]),
        cache=copy_field(IProcessor["cache"]),
        speed=copy_field(IProcessor["speed"]),
        other=copy_field(IProcessor["other"]),
        )
    @export_write_operation()
    def setProcessor(
        platform, vendor, model, model_number, model_version,
        model_revision, cache, bogomips, speed, other):
        """Set the processor for this target."""
