# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ISystem",
    "ISystemSet",
    "IHasSystem",
    "SystemSearchParams",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )
from zope.schema import TextLine

from lazr.restful.declarations import exported
from lazr.restful.fields import Reference
from lazr.restful.frameworks.django import IDjangoLocation
from lazr.restful.interfaces import ITopLevelEntryLink
from lazr.restful.simple import IMultiplePathPartLocation

from lpresults.registry.interfaces.search import (
    ISearchSet,
    SearchParams,
    )

from lpresults.hardware.interfaces.unit import IUnitTarget
from lpresults.hardware.interfaces.systemunit import ISystemUnit


class ISystem(IMultiplePathPartLocation, IUnitTarget):

    id = Attribute("A unique identifier for the system")

    name = exported(
        TextLine(
            title=u"Name",
            required=True,
            readonly=True,
            description=u"The name consisting of the make and model."))

    make = exported(
        TextLine(
            title=u"Make",
            required=True,
            readonly=True,
            description=u"The make for this system."))

    model = exported(
        TextLine(
            title=u"Model",
            required=True,
            readonly=True,
            description=u"The model for this system."))


class ISystemSet(ITopLevelEntryLink, IDjangoLocation, ISearchSet):

    def get(id):
        """Get a system by its id.

        Return None if there is no such system.
        """

    def retrieve(make, model):
        """Return the system for the given make and model, or a new
        system if not found.

        :param make: The make of the system.
        :param model: The model of the system.
        """


class IHasSystem(Interface):

    system = exported(
        Reference(
            schema=ISystem,
            title=u"System",
            required=True,
            readonly=True,
            description=u"The system associated with this object."))


class SystemSearchParams(SearchParams):
    """Encapsulates search parameters for ISystemSet.search()"""

    def __init__(self, make=None, model=None, **kwargs):
        super(SystemSearchParams, self).__init__(**kwargs)
        self.make = make
        self.model = model


ISystem["units"].value_type.schema = ISystemUnit
ISystemUnit["system"].schema = ISystem
