# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IHasUnits",
    "IUnit",
    "IUnitSet",
    "IUnitTarget",
    "UnitSearchParams",
    ]

from zope.interface import Interface
from zope.schema import (
    Int,
    List,
    TextLine,
    )

from lazr.restful.declarations import exported
from lazr.restful.fields import (
    CollectionField,
    Reference,
    )
from lazr.restful.frameworks.django import IDjangoLocation

from lpresults.registry.interfaces.search import (
    ISearchSet,
    SearchParams,
    )


class IUnit(IDjangoLocation):
    """A unit of hardware."""

    identifiers = List(
        title=u"Identifiers",
        required=True,
        readonly=True,
        description=u"List of identifiers unique to the unit.",
        value_type=TextLine())

    sequence = exported(
        Int(
            title=u"Sequence",
            required=True,
            readonly=True,
            description=u"Sequence number identifying the hardware."))


class IUnitSet(ISearchSet):

    def new(identifiers):
        """Create a new unit.

        :param identifiers: The nique identifiers of the unit.
        """

    def get(sequence):
        """Get a unit by its sequence number.

        :param id: The identifier of the unit.
        """

    def getByIdentifiers(identifiers):
        """Get a unit by identifiers.

        :param identifiers: List of identifiers unique to the unit.
        """


class IHasUnits(Interface):
    """An entity which has a collection of units."""

    units = exported(
        CollectionField(
            title=u"List of units.",
            value_type=Reference(schema=IUnit),
            readonly=True))

    def getUnit(sequence):
        """Return the unit for the given sequence number, or None."""


class IUnitTarget(IHasUnits):
    """An entity on which a unit can be created."""

    def newUnit(identifiers):
        """Return a new unit.

        :param identifiers: List of identifiers unique to the unit.
        """

    def retrieveUnit(identifiers):
        """Return the unit for the given identifiers, or a new unit if
        not found.

        :param identifiers: List of identifiers of the existing or new unit.
        """


class UnitSearchParams(SearchParams):
    """Encapsulates search parameters for IUnitSet.search()"""

    def __init__(self, identifiers=None, sequence=None, **kwargs):
        super(UnitSearchParams, self).__init__(**kwargs)
        self.identifiers = identifiers
        self.sequence = sequence
