# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Device",
    "device_set",
    ]

from storm.locals import (
    AutoReload,
    Int,
    Reference,
    )

from zope.interface import implements

from lpresults.hardware.interfaces.device import (
    IDevice,
    IDeviceSet,
    )
from lpresults.hardware.models.devicebus import DeviceBus
from lpresults.hardware.models.devicecategory import DeviceCategory
from lpresults.hardware.models.devicevendor import DeviceVendor
from lpresults.hardware.models.store import get_hardware_store


class Device:

    __storm_table__ = "device"

    implements(IDevice)

    id = Int(primary=True, allow_none=False, default=AutoReload)
    bus_id = Int(name="bus", allow_none=False)
    product_id = Int(name="product", allow_none=False)
    vendor_id = Int(name="vendor")
    category_id = Int(name="category")
    parent_id = Int(name="parent")

    bus = Reference(bus_id, DeviceBus.id)
    vendor = Reference((bus_id, vendor_id),
        (DeviceVendor.bus_id, DeviceVendor.vendor_id))
    category = Reference(category_id, DeviceCategory.id)

    def __init__(
        self, bus, product_id, vendor_id=None, category=None, parent=None):

        self.bus = bus
        self.product_id = product_id
        self.vendor_id = vendor_id
        self.category = category
        self.parent = parent


class DeviceSet:

    implements(IDeviceSet)

    def getId(
        self, bus_name, category_name=None, product_name=None,
        vendor_name=None, product_id=None, vendor_id=None,
        subproduct_id=None, subvendor_id=None):
        """See `IDeviceSet`."""
        return get_hardware_store().execute(
            "SELECT get_device(?, ?, ?, ?, ?, ?, ?, ?)",
            (bus_name, category_name, product_name, vendor_name,
            product_id, vendor_id, subproduct_id, subvendor_id,)).get_one()[0]


device_set = DeviceSet()
