# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DeviceBus",
    "HasDeviceBusMixin",
    "device_bus_set",
    ]

from storm.locals import (
    AutoReload,
    Int,
    Reference,
    Unicode,
    )

from zope.cachedescriptors.property import Lazy
from zope.component import getUtility
from zope.interface import implements

from lpresults.database.datetime import UTCDateTime

from lpresults.hardware.interfaces.devicebus import (
    IDeviceBus,
    IDeviceBusSet,
    )
from lpresults.hardware.models.deviceproduct import (
    DeviceProduct,
    DeviceProductTargetMixin,
    )
from lpresults.hardware.models.devicevendor import (
    DeviceVendor,
    DeviceVendorTargetMixin,
    )
from lpresults.hardware.models.store import get_hardware_store


class DeviceBus(DeviceProductTargetMixin, DeviceVendorTargetMixin):

    __storm_table__ = "devicebus"

    implements(IDeviceBus)

    id = Int(primary=True, allow_none=False, default=AutoReload)
    name = Unicode(allow_none=False)
    date_modified = UTCDateTime()

    def __init__(self, name, date_modified=None):
        self.name = name
        self.date_modified = date_modified

    def getBusId(self):
        return self.id


class DeviceBusSet:

    implements(IDeviceBusSet)

    def get(self, id):
        """See `IDeviceBusSet`."""
        return get_hardware_store().get(DeviceBus, id)

    def getByName(self, name):
        store = get_hardware_store()
        bus = store.find(DeviceBus, name=name).one()
        if not bus:
            bus = DeviceBus(name)
            store.add(bus)

        return bus


class HasDeviceBusMixin:

    @property
    def bus_name(self):
        """See `IHasDeviceBus`."""
        raise NotImplementedError()

    @Lazy
    def bus(self):
        """See `IHasDeviceBus`."""
        return getUtility(IDeviceBusSet).getByName(self.bus_name)


DeviceProduct.bus = Reference(DeviceProduct.bus_id, DeviceBus.id)
DeviceVendor.bus = Reference(DeviceVendor.bus_id, DeviceBus.id)

device_bus_set = DeviceBusSet()
