# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DeviceState",
    ]

from storm.locals import (
    AutoReload,
    Int,
    Reference,
    )

from zope.interface import implements

from lpresults.database.constants import (
    INFINITY,
    UTC_NOW,
    )
from lazr.delegates import delegates

from lpresults.database.datetime import UTCDateTime

from lpresults.hardware.interfaces.device import IDevice
from lpresults.hardware.interfaces.devicestate import IDeviceState
from lpresults.hardware.models.device import Device
from lpresults.hardware.models.devicedriver import DeviceDriver
from lpresults.hardware.models.systemunithelper import SystemUnitHelper


class DeviceState(SystemUnitHelper):

    __storm_table__ = "devicestate"

    implements(IDeviceState)
    delegates(IDevice, context="device")

    id = Int(primary=True, allow_none=False, default=AutoReload)
    system_unit_id = Int(allow_none=False)
    device_id = Int(allow_none=False)
    subdevice_id = Int()
    driver_id = Int()
    date_created = UTCDateTime(allow_none=False, default=UTC_NOW)
    date_deleted = UTCDateTime(allow_none=False, default=INFINITY)

    device = Reference(device_id, Device.id)
    subdevice = Reference(subdevice_id, Device.id)
    driver = Reference(driver_id, DeviceDriver.id)

    def __init__(self, system_unit, device, subdevice=None, driver=None,
            date_created=None):
        super(DeviceState, self).__init__(system_unit)
        self.device = device
        self.subdevice = subdevice
        self.driver = driver

        if date_created is not None:
            self.date_created = date_created
