# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DeviceVendor",
    "DeviceVendorTargetMixin",
    ]

from storm.locals import (
    Int,
    RawStr,
    Unicode,
    )

from zope.interface import implements

from lpresults.database.textsearch import (
    TSVectorPart,
    TSVectorValidator,
    )

from lpresults.hardware.interfaces.devicevendor import (
    IDeviceVendor,
    IDeviceVendorTarget,
    )
from lpresults.hardware.models.store import get_hardware_store


DEVICE_VENDOR_VALIDATOR = TSVectorValidator(
    "text_search_vector", TSVectorPart(u"name", weight=u"A"))


class DeviceVendor:
    """A device vendor on a specific bus.

    A vendor can be given an optional identifier which must be less
    than or equal 0xFFFF otherwise the identifier will be automatically
    generated with a greater value. This makes it possible to mix vendors
    with known and unknown identifiers on the same bus.
    """
    __storm_table__ = "devicevendor"
    __storm_primary__ = ("bus_id", "vendor_id",)

    implements(IDeviceVendor)

    name = Unicode(allow_none=False, validator=DEVICE_VENDOR_VALIDATOR)
    bus_id = Int(name="bus", allow_none=False)
    vendor_id = Int(name="vendor", allow_none=False)

    text_search_vector = RawStr()

    def __init__(self, bus, name, vendor_id=None):
        self.bus = bus
        self.name = name
        if vendor_id is not None:
            self.vendor_id = vendor_id


class DeviceVendorTargetMixin:

    implements(IDeviceVendorTarget)

    def getBusId(self):
        """See `IDeviceVendorTarget`."""
        raise NotImplementedError()

    def setVendor(self, name, vendor_id=None):
        """See `IDeviceVendorTarget`."""
        bus_id = self.getBusId()
        return get_hardware_store().execute(
            "SELECT set_device_vendor(?, ?, ?)",
            (name, bus_id, vendor_id,), noresult=True)
