# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "SystemUnit",
    "SystemUnitSet",
    ]

from storm.locals import (
    AutoReload,
    Int,
    List,
    Reference,
    Unicode,
    )

from zope.interface import implements

from lpresults.database.array import ArrayOverlaps
from lpresults.database.search import search_to_expression

from lpresults.hardware.interfaces.system import ISystem
from lpresults.hardware.interfaces.systemunit import (
    ISystemUnit,
    ISystemUnitSet,
    )
from lpresults.hardware.interfaces.unit import UnitSearchParams
from lpresults.hardware.models.system import System
from lpresults.hardware.models.unit import (
    Unit,
    UnitSet,
    )


class SystemUnit(Unit):

    __storm_table__ = "systemunit"

    implements(ISystemUnit)

    id = Int(primary=True, allow_none=False, default=AutoReload)
    sequence = Int(allow_none=False)
    system_id = Int(name="system", allow_none=False)
    identifiers = List(type=Unicode())

    system = Reference(system_id, System.id)

    def __init__(self, system, identifiers):
        super(SystemUnit, self).__init__(identifiers)
        self.system = system

    @property
    def __parent__(self):
        return self.system

    @property
    def __url_path__(self):
        return str(self.sequence)


class SystemUnitSet(UnitSet):
    """See `ISystemUnitSet`."""

    implements(ISystemUnitSet)

    def new(self, identifiers):
        """See `IUnitSet`."""
        return SystemUnit(self.target, identifiers)

    def get(self, sequence):
        """See `IUnitSet`."""
        search_params = UnitSearchParams(sequence=sequence)
        return self.search(search_params).one()

    def buildSearchExpressions(self, search_params):
        """See SearchSet."""
        expressions = super(SystemUnitSet, self).buildSearchExpressions(
            search_params)

        if ISystem.providedBy(self.target):
            expressions.append(SystemUnit.system_id == self.target.id)

        if search_params.sequence is not None:
            expressions.append(
                search_to_expression(
                    SystemUnit.sequence, search_params.sequence))

        if search_params.identifiers is not None:
            expressions.append(ArrayOverlaps(
                SystemUnit.identifiers, search_params.identifiers))

        return expressions

    def getSearchResultTables(self):
        """See SearchSet."""
        return SystemUnit
