# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "SystemUnitHelper",
    "SystemUnitTwin",
    ]

from storm.locals import Int

from zope.interface import implements

from lpresults.hardware.interfaces.systemunit import (
    IHasSystemUnit,
    ISystemUnitSet,
    )


class SystemUnitHelper:
    """Helper for implementing L{IHasSystemUnit} classes."""

    implements(IHasSystemUnit)

    _system_unit = None
    system_unit_id = None

    def __init__(self, system_unit=None):
        if system_unit is not None:
            if isinstance(system_unit, (int, long)):
                self.system_unit_id = system_unit
            else:
                self.system_unit = system_unit

    def _getSystemUnit(self):
        _series = self._system_unit
        series_id = self.system_unit_id
        if ((series_id is not None and not _series) or
            (_series is not None and _series.id != series_id)):

            self._system_unit = ISystemUnitSet(self).get(series_id)

        return self._system_unit

    def _setSystemUnit(self, system_unit):
        if system_unit is None:
            self.system_unit_id = None
        else:
            self.system_unit_id = system_unit.id

        self._system_unit = system_unit

    system_unit = property(_getSystemUnit, _setSystemUnit)


class SystemUnitTwin(SystemUnitHelper):
    """The representation of a system unit in the hardware database.

    The primary purpose of this object is to serve as a point of relational
    stability with all system unit objects in the registry databases.
    """
    __storm_table__ = "systemunittwin"

    id = Int(primary=True, allow_none=False)
    system_id = Int(name="system", allow_none=False)

    def __init__(self, system_unit):
        super(SystemUnitTwin, self).__init__(system_unit)
        self.system_id = system_unit.system_id

    # This property makes SystemUnitHelper happy.  When SystemUnitHelper
    # sets system_unit_id this code causes the id of this object to be the
    # same as the SystemUnit in the registry database represented by this
    # one.
    def _getSystemUnitID(self):
        return self.id

    def _setSystemUnitID(self, id):
        self.id = id

    system_unit_id = property(_getSystemUnitID, _setSystemUnitID)
