# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.hardware.interfaces.system import (
    ISystemSet,
    SystemSearchParams,
    )
from lpresults.hardware.models.tests.test_unit import TestUnitTargetMixin
from lpresults.hardware.testing.cases import TestCaseWithFactory
from lpresults.hardware.testing.layers import FunctionalDatabaseLayer


class TestSystemSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_retrieve(self):
        make = self.factory.getUniqueString(u"system-make")
        model = self.factory.getUniqueString(u"system-model")
        system = getUtility(ISystemSet).retrieve(make, model)
        self.assertNotEquals(system, None)
        self.assertEquals(system.make, make)
        self.assertEquals(system.model, model)

    def test_searchEmpty(self):
        make = self.factory.getUniqueString(u"system-make")
        params = SystemSearchParams(make=make)
        other = getUtility(ISystemSet).search(params).one()
        self.assertEquals(other, None)

    def test_searchMake(self):
        make = self.factory.getUniqueString(u"system-make")
        system = self.factory.makeSystem(make=make)

        # Search by make
        params = SystemSearchParams(make=make)
        other = getUtility(ISystemSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, system)

    def test_searchModel(self):
        model = self.factory.getUniqueString(u"system-model")
        system = self.factory.makeSystem(model=model)

        # Search by model
        params = SystemSearchParams(model=model)
        other = getUtility(ISystemSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, system)


class TestUnitTarget(TestCaseWithFactory, TestUnitTargetMixin):

    def getUnitTarget(self):
        return self.factory.makeSystem()


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
