# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestUnitSetMixin",
    "TestUnitTargetMixin",
    ]

from unittest import defaultTestLoader

from lpresults.hardware.interfaces.unit import (
    IUnit,
    IUnitSet,
    )
from lpresults.hardware.testing.layers import FunctionalDatabaseLayer


class TestUnitSetMixin:

    layer = FunctionalDatabaseLayer

    def getUnitContext(self):
        raise NotImplementedError()

    def test_new(self):
        context = self.getUnitContext()
        identifiers = [self.factory.getUniqueString(u"unit-identifier")]
        unit = IUnitSet(context).new(identifiers)
        self.assertNotEquals(unit, None)
        self.assertTrue(IUnit.providedBy(unit))
        self.assertEquals(unit.identifiers, identifiers)

    def test_getByIdentifiers(self):
        context = self.getUnitContext()
        identifiers = [self.factory.getUniqueString(u"unit-identifier")]
        unit = IUnitSet(context).getByIdentifiers(identifiers)
        self.assertEquals(unit, None)

        IUnitSet(context).new(identifiers)
        unit = IUnitSet(context).getByIdentifiers(identifiers)
        self.assertNotEquals(unit, None)
        self.assertEquals(unit.identifiers, identifiers)


class TestUnitTargetMixin:

    layer = FunctionalDatabaseLayer

    def getUnitTarget(self):
        raise NotImplementedError()

    def test_units(self):
        target = self.getUnitTarget()
        self.assertEquals(target.units.count(), 0)

        identifiers = [self.factory.getUniqueString(u"unit-identifier")]
        unit = target.newUnit(identifiers)
        self.assertEquals(target.units.count(), 1)
        self.assertTrue(unit in target.units)

    def test_getUnit(self):
        target = self.getUnitTarget()
        unit = target.getUnit(1)
        self.assertEquals(unit, None)

        self.factory.makeSystemUnit(system=target)
        unit = target.getUnit(1)
        self.assertNotEquals(unit, None)

    def test_newUnit(self):
        target = self.getUnitTarget()
        identifiers = [self.factory.getUniqueString(u"unit-identifier")]
        unit = target.newUnit(identifiers)
        self.assertEquals(unit.identifiers, identifiers)

    def test_retrieveUnit(self):
        target = self.getUnitTarget()
        identifiers = [self.factory.getUniqueString(u"unit-identifier")]
        unit = target.retrieveUnit(identifiers)
        self.assertNotEquals(unit, None)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
