# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Unit",
    "UnitSet",
    "UnitTargetMixin",
    ]

from zope.event import notify
from zope.interface import implements

from lazr.lifecycle.event import ObjectCreatedEvent
from lazr.restful.simple import TraverseWithGet

from lpresults.registry.models.search import SearchSet

from lpresults.hardware.interfaces.unit import (
    IUnit,
    IUnitSet,
    IUnitTarget,
    UnitSearchParams,
    )
from lpresults.hardware.models.store import get_hardware_store


class Unit:

    implements(IUnit)

    def __init__(self, identifiers):
        self.identifiers = identifiers

    @property
    def __url_path__(self):
        return self.sequence


class UnitSet(SearchSet):
    """See `IUnitSet`."""

    implements(IUnitSet)

    search_params_factory = UnitSearchParams

    def __init__(self, target):
        self.target = target

    def new(self, identifiers):
        """See `IUnitSet`."""
        raise NotImplementedError()

    def get(self, sequence):
        """See `IUnitSet`."""
        raise NotImplementedError()

    def getByIdentifiers(self, identifiers):
        """See `IUnitSet`."""
        search_params = self.search_params_factory(identifiers=identifiers)
        return self.search(search_params).one()

    def getSearchStore(self, search_aprams):
        """See SearchSet."""
        return get_hardware_store()

    def search(self, *args):
        if not args:
            search_params = self.search_params_factory()
            args = (search_params,)

        return super(UnitSet, self).search(*args)


class UnitTargetMixin(TraverseWithGet):

    implements(IUnitTarget)

    @property
    def units(self):
        """See `IHasUnits`."""
        return IUnitSet(self).search()

    def get(self, request, name):
        """See `ITraverseWithGet`."""
        value = super(UnitTargetMixin, self).get(request, name)
        if value is None:
            sequence = int(name)
            value = self.getUnit(sequence)

        return value

    def getUnit(self, sequence):
        """See `IHasUnits`."""
        return IUnitSet(self).get(sequence)

    def newUnit(self, identifiers):
        """See `IUnitTarget`."""
        if not identifiers:
            raise TypeError("identifers must not be empty")
        unit = IUnitSet(self).new(identifiers)
        notify(ObjectCreatedEvent(unit))
        return unit

    def retrieveUnit(self, identifiers):
        """See `IUnitTarget`."""
        unit = IUnitSet(self).getByIdentifiers(identifiers)
        if unit is None:
            unit = self.newUnit(identifiers)

        return unit
