# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = [
    "create",
    ]

from pkg_resources import resource_string

from storm.zope.schema import ZSchema

from lpresults import database


def create():
    from lpresults.hardware import schema as patch_package

    return ZSchema(CREATES, DROPS, DELETES, patch_package)


CREATES = [
"""
GRANT SELECT
    ON TABLE patch
    TO lpresults
""",
"""
CREATE TEXT SEARCH CONFIGURATION public.lpresults_english
    (COPY = pg_catalog.english)
""",
"""
SET default_text_search_config = 'public.lpresults_english'
""",
"""
ALTER TEXT SEARCH CONFIGURATION public.lpresults_english
   ALTER MAPPING FOR asciiword WITH english_stem
""",
"""
CREATE TABLE system (
    id SERIAL NOT NULL PRIMARY KEY,
    make CHARACTER VARYING(80),
    model CHARACTER VARYING(80),
    counter INTEGER DEFAULT 0,
    text_search_vector tsvector)
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE system
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE system_id_seq
    TO lpresults
""",
"""
CREATE INDEX system__text_search_vector__key
    ON system USING GIN(text_search_vector)
""",
"""
CREATE TABLE systemunit (
    id SERIAL NOT NULL PRIMARY KEY,
    sequence INTEGER NOT NULL,
    system INTEGER NOT NULL,
    identifiers TEXT[] NOT NULL)
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE systemunit
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE systemunit_id_seq
    TO lpresults
""",
"""
CREATE INDEX systemunit__identifiers__key
    ON systemunit USING GIN(identifiers)
""",
"""
CREATE UNIQUE INDEX systemunit__sequence__system__key
    ON systemunit (sequence, system)
""",
"""
CREATE TABLE processorplatform (
    id SERIAL NOT NULL PRIMARY KEY,
    name CHARACTER VARYING(16) NOT NULL UNIQUE)
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE processorplatform
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE processorplatform_id_seq
    TO lpresults
""",
"""
CREATE TABLE processor (
    id SERIAL NOT NULL PRIMARY KEY,
    platform INTEGER NOT NULL,
    type CHARACTER VARYING(16) NOT NULL,
    model CHARACTER VARYING(48) NOT NULL,
    model_number CHARACTER VARYING(8) NOT NULL,
    model_version CHARACTER VARYING(8) NOT NULL,
    model_revision CHARACTER VARYING(8) NOT NULL,
    cache INTEGER NOT NULL,
    bogomips INTEGER NOT NULL,
    speed INTEGER NOT NULL,
    other TEXT NOT NULL,
    UNIQUE(platform, type, model, model_number, model_version,
        model_revision, cache, bogomips, speed, other))
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE processor
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE processor_id_seq
    TO lpresults
""",
"""
CREATE TABLE devicebus (
    id SERIAL NOT NULL PRIMARY KEY,
    name CHARACTER VARYING(16) NOT NULL UNIQUE,
    date_modified TIMESTAMP WITHOUT TIME ZONE)
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE devicebus
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE devicebus_id_seq
    TO lpresults
""",
"""
CREATE TABLE devicecategory (
    id SERIAL NOT NULL PRIMARY KEY,
    name CHARACTER VARYING(16) NOT NULL UNIQUE)
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE devicecategory
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE devicecategory_id_seq
    TO lpresults
""",
"""
CREATE TABLE devicedriver (
    id SERIAL NOT NULL PRIMARY KEY,
    name CHARACTER VARYING(80) NOT NULL UNIQUE)
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE devicedriver
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE devicedriver_id_seq
    TO lpresults
""",
"""
CREATE TABLE devicevendor (
    name CHARACTER VARYING(256) NOT NULL,
    bus INTEGER NOT NULL,
    vendor INTEGER NOT NULL,
    text_search_vector tsvector,
    PRIMARY KEY (bus, vendor))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE devicevendor
    TO lpresults
""",
"""
CREATE SEQUENCE devicevendor_vendor_seq MINVALUE 65536
""",
"""
GRANT SELECT, UPDATE
    ON TABLE devicevendor_vendor_seq
    TO lpresults
""",
"""
CREATE INDEX devicevendor__text_search_vector__key
    ON devicevendor USING GIN(text_search_vector)
""",
"""
CREATE TABLE deviceproduct (
    name CHARACTER VARYING(256) NOT NULL,
    bus INTEGER NOT NULL,
    product INTEGER NOT NULL,
    vendor INTEGER,
    subproduct INTEGER,
    subvendor INTEGER,
    text_search_vector tsvector)
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE deviceproduct
    TO lpresults
""",
"""
CREATE SEQUENCE deviceproduct_product_seq MINVALUE 65536
""",
"""
GRANT SELECT, UPDATE
    ON TABLE deviceproduct_product_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX deviceproduct__bus__product__key
    ON deviceproduct (bus, product)
    WHERE vendor IS NULL AND subproduct IS NULL AND subvendor IS NULL
""",
"""
CREATE UNIQUE INDEX deviceproduct__bus__product__vendor__key
    ON deviceproduct (bus, product, vendor)
    WHERE subproduct IS NULL AND subvendor IS NULL
""",
"""
CREATE UNIQUE INDEX
    deviceproduct__bus__product__vendor__subproduct__subvendor__key
    ON deviceproduct (bus, product, vendor, subproduct, subvendor)
""",
"""
CREATE INDEX deviceproduct__text_search_vector__key
    ON deviceproduct USING GIN(text_search_vector)
""",
"""
CREATE TABLE device (
    id SERIAL NOT NULL PRIMARY KEY,
    bus INTEGER NOT NULL,
    category INTEGER,
    product INTEGER NOT NULL,
    vendor INTEGER,
    parent INTEGER)
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE device
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE device_id_seq
    TO lpresults
""",
"""
CREATE INDEX device__bus__product__key
    ON device (bus, product)
    WHERE vendor IS NULL AND parent IS NULL
""",
"""
CREATE INDEX device__bus__product__vendor__key
    ON device (bus, product, vendor)
    WHERE parent IS NULL
""",
"""
CREATE UNIQUE INDEX device__bus__product__vendor__category__parent__key
    ON device (bus, product, (COALESCE(vendor, (-1))),
        (COALESCE(category, (-1))), (COALESCE(parent, (-1))))
""",
"""
CREATE TABLE devicestate (
    id SERIAL NOT NULL PRIMARY KEY,
    system_unit INTEGER NOT NULL,
    device INTEGER NOT NULL,
    subdevice INTEGER,
    driver INTEGER,
    count INTEGER NOT NULL DEFAULT 1,
    date_created TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
    date_deleted TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT '3000-01-01')
""",
"""
GRANT SELECT, INSERT, DELETE
    ON TABLE devicestate
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE devicestate_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX devicestate__id__key
    ON devicestate (id)
    WHERE date_deleted = 'infinity'
""",
"""
CREATE INDEX devicestate__system_unit__key
    ON devicestate (system_unit)
""",
"""
CREATE INDEX devicestate__device__key
    ON devicestate (device)
""",
"""
CREATE INDEX devicestate__date_created__date_deleted__key
    ON devicestate (date_created, date_deleted)
""",
"""
CREATE RULE devicestate_delete
    AS ON DELETE TO devicestate
    DO INSTEAD
       (UPDATE devicestate SET date_deleted=now()
       WHERE id=OLD.id AND date_deleted='infinity';)
""",
"""
CREATE LANGUAGE plpythonu
""",
"""
CREATE FUNCTION text_as_grams(i_txt TEXT) RETURNS TEXT
LANGUAGE plpythonu AS
$$
"""
+ resource_string(database.__name__, "textgrams.py")
+ """
return get_text_grams_as_text(i_txt)
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION text_as_grams(TEXT)
    TO lpresults
""",
"""
CREATE LANGUAGE plpgsql
""",
"""
CREATE FUNCTION next_system_sequence(
    system_id INTEGER)
    RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    next_sequence INTEGER;
BEGIN
    UPDATE system SET counter = counter + 1
        WHERE id = system_id;
    SELECT INTO next_sequence counter FROM system
        WHERE id = system_id;
    RETURN next_sequence;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION next_system_sequence(INTEGER)
    TO lpresults
""",
"""
CREATE FUNCTION systemunit_insert_before() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
   NEW.sequence = next_system_sequence(NEW.system);

   RETURN NEW;
END
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION systemunit_insert_before()
    TO lpresults
""",
"""
CREATE TRIGGER systemunit_insert_before_trigger
    BEFORE INSERT ON systemunit
    FOR EACH ROW
    EXECUTE PROCEDURE systemunit_insert_before()
""",
"""
CREATE FUNCTION get_device_bus(bus_name TEXT) RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    bus_id INTEGER;
BEGIN
    SELECT devicebus.id INTO bus_id FROM devicebus
        WHERE devicebus.name = bus_name;
    IF NOT FOUND THEN
        INSERT INTO devicebus (name)
            VALUES (bus_name)
            RETURNING id INTO bus_id;
    END IF;
    RETURN bus_id;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_device_bus(TEXT)
    TO lpresults
""",
"""
CREATE FUNCTION get_device_category(category_name TEXT) RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    category_id INTEGER;
BEGIN
    SELECT devicecategory.id INTO category_id FROM devicecategory
        WHERE devicecategory.name = category_name;
    IF NOT FOUND THEN
        INSERT INTO devicecategory (name)
            VALUES (category_name)
            RETURNING id INTO category_id;
    END IF;
    RETURN category_id;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_device_category(TEXT)
    TO lpresults
""",
"""
CREATE FUNCTION get_device_product(
    product_name TEXT, bus_id INTEGER,
    product_id INTEGER, vendor_id INTEGER,
    subproduct_id INTEGER, subvendor_id INTEGER)
    RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    return_id INTEGER;
BEGIN
    IF product_id IS NOT NULL THEN
        SELECT product INTO return_id FROM deviceproduct
            WHERE bus = bus_id
            AND product = product_id
            AND vendor IS NOT DISTINCT FROM vendor_id
            AND subproduct IS NOT DISTINCT FROM subproduct_id
            AND subvendor IS NOT DISTINCT FROM subvendor_id;
        IF NOT FOUND THEN
            IF product_name IS NOT NULL THEN
                INSERT INTO deviceproduct (
                        bus, product, vendor,
                        subproduct, subvendor, name)
                    VALUES (
                        bus_id, product_id, vendor_id,
                        subproduct_id, subvendor_id, product_name)
                    RETURNING product INTO return_id;
            ELSE
                return_id := NULL;
            END IF;
        END IF;
    ELSIF product_name IS NOT NULL THEN
        SELECT product INTO return_id FROM deviceproduct
            WHERE bus = bus_id
            AND product >= 65536
            AND vendor IS NOT DISTINCT FROM vendor_id
            AND subproduct IS NOT DISTINCT FROM subproduct_id
            AND subvendor IS NOT DISTINCT FROM subvendor_id
            AND name = product_name;
        IF NOT FOUND THEN
            INSERT INTO deviceproduct (
                    bus, vendor,
                    subproduct, subvendor, name)
                VALUES (
                    bus_id, vendor_id,
                    subproduct_id, subvendor_id, product_name)
                RETURNING product INTO return_id;
        END IF;
    ELSE
        return_id := NULL;
    END IF;
    RETURN return_id;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_device_product(
        TEXT, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER)
    TO lpresults
""",
"""
CREATE FUNCTION set_device_product(
    product_name TEXT, bus_id INTEGER,
    product_id INTEGER, vendor_id INTEGER,
    subproduct_id INTEGER, subvendor_id INTEGER)
    RETURNS VOID
LANGUAGE plpgsql AS
$$
BEGIN
    LOOP
        UPDATE deviceproduct
            SET name = product_name
            WHERE bus = bus_id
            AND product = product_id
            AND vendor = vendor_id
            AND subproduct IS NOT DISTINCT FROM subproduct_id
            AND subvendor IS NOT DISTINCT FROM subvendor_id;
        IF FOUND THEN
            RETURN;
        END IF;

        BEGIN
            INSERT INTO deviceproduct (
                    name, bus, product, vendor,
                    subproduct, subvendor)
                VALUES (
                    product_name, bus_id, product_id, vendor_id,
                    subproduct_id, subvendor_id);
            RETURN;
        EXCEPTION WHEN unique_violation THEN
            NULL;  -- do nothing
        END;
    END LOOP;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION set_device_product(
        TEXT, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER)
    TO lpresults
""",
"""
CREATE FUNCTION get_device_vendor(
    vendor_name TEXT, bus_id INTEGER, vendor_id INTEGER)
    RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    return_id INTEGER;
BEGIN
    IF vendor_id IS NOT NULL THEN
        SELECT vendor INTO return_id FROM devicevendor
            WHERE bus = bus_id
            AND vendor = vendor_id;
        IF NOT FOUND THEN
            IF vendor_name IS NOT NULL THEN
                INSERT INTO devicevendor (bus, vendor, name)
                    VALUES (bus_id, vendor_id, vendor_name)
                    RETURNING vendor INTO return_id;
            ELSE
                return_id := NULL;
            END IF;
        END IF;
    ELSIF vendor_name IS NOT NULL THEN
        SELECT vendor INTO return_id FROM devicevendor
            WHERE bus = bus_id
            AND vendor >= 65536
            AND name = vendor_name;
        IF NOT FOUND THEN
            INSERT INTO devicevendor (bus, name)
                VALUES (bus_id, vendor_name)
                RETURNING vendor INTO return_id;
        END IF;
    ELSE
        return_id := NULL;
    END IF;
    RETURN return_id;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_device_vendor(TEXT, INTEGER, INTEGER)
    TO lpresults
""",
"""
CREATE FUNCTION set_device_vendor(
    vendor_name TEXT, bus_id INTEGER, vendor_id INTEGER)
    RETURNS VOID
LANGUAGE plpgsql AS
$$
BEGIN
    LOOP
        UPDATE devicevendor
            SET name = vendor_name
            WHERE bus = bus_id
            AND vendor = vendor_id;
        IF FOUND THEN
            RETURN;
        END IF;

        BEGIN
            INSERT INTO devicevendor (name, bus, vendor)
                VALUES (vendor_name, bus_id, vendor_id);
            RETURN;
        EXCEPTION WHEN unique_violation THEN
            NULL;  -- do nothing
        END;
    END LOOP;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION set_device_vendor(TEXT, INTEGER, INTEGER)
    TO lpresults
""",
"""
CREATE FUNCTION get_device_by_id(
    bus_id INTEGER, category_id INTEGER,
    product_id INTEGER, vendor_id INTEGER,
    parent_id INTEGER)
    RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    device_id INTEGER;
BEGIN
    SELECT device.id INTO device_id FROM device
        WHERE bus = bus_id
        AND category IS NOT DISTINCT FROM category_id
        AND product = product_id
        AND vendor IS NOT DISTINCT FROM vendor_id
        AND parent IS NOT DISTINCT FROM parent_id;
    IF NOT FOUND THEN
        INSERT INTO device (bus, category, product, vendor, parent)
            VALUES (bus_id, category_id, product_id, vendor_id, parent_id)
            RETURNING id INTO device_id;
    END IF;
    RETURN device_id;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_device_by_id(INTEGER, INTEGER, INTEGER, INTEGER, INTEGER)
    TO lpresults
""",
"""
CREATE FUNCTION get_device(
    bus_name TEXT, category_name TEXT,
    product_name TEXT, vendor_name TEXT,
    product_id INTEGER, vendor_id INTEGER,
    subproduct_id INTEGER, subvendor_id INTEGER)
    RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    bus_id INTEGER;
    category_id INTEGER;
    device_id INTEGER;
    parent_id INTEGER;
    new_product_id INTEGER;
    new_vendor_id INTEGER;
BEGIN
    LOOP
        IF subproduct_id IS NOT NULL AND subvendor_id IS NOT NULL THEN
            SELECT d.id INTO device_id FROM device d
                JOIN device p ON p.id = d.parent
                JOIN devicebus db ON db.id = d.bus
                LEFT JOIN devicecategory dc ON dc.id = d.category
                WHERE db.name = bus_name
                AND d.product = subproduct_id
                AND d.vendor = subvendor_id
                AND p.product = product_id
                AND p.vendor = vendor_id
                AND dc.name IS NOT DISTINCT FROM category_name;
        ELSE
            SELECT d.id INTO device_id FROM device d
                JOIN devicebus db ON db.id = d.bus
                JOIN deviceproduct dp
                    ON dp.bus = d.bus
                    AND dp.product = d.product
                    AND dp.vendor IS NOT DISTINCT FROM d.vendor
                    AND dp.subproduct IS NULL
                    AND dp.subvendor IS NULL
                LEFT JOIN devicevendor dv
                    ON dv.bus = d.bus
                    AND dv.vendor = d.vendor
                LEFT JOIN devicecategory dc
                    ON dc.id = d.category
                WHERE db.name = bus_name
                AND CASE WHEN product_id IS NOT NULL THEN
                        d.product = product_id
                    ELSE
                        d.product >= 65536 AND dp.name = product_name
                    END
                AND CASE WHEN vendor_id IS NOT NULL THEN
                        d.vendor = vendor_id
                    ELSE
                        d.vendor >= 65536 AND dv.name = vendor_name
                    END
                AND dc.name IS NOT DISTINCT FROM category_name;
        END IF;
        IF FOUND THEN
            RETURN device_id;
        END IF;

        BEGIN
            bus_id := get_device_bus(bus_name);
            IF category_name IS NOT NULL THEN
                category_id := get_device_category(category_name);
            ELSE
                category_id := NULL;
            END IF;

            new_vendor_id := get_device_vendor(
                vendor_name, bus_id, vendor_id);
            new_product_id := get_device_product(
                product_name, bus_id,
                product_id, new_vendor_id,
                subproduct_id, subvendor_id);

            IF subproduct_id IS NOT NULL AND subvendor_id IS NOT NULL THEN
                parent_id := get_device_by_id(
                    bus_id, category_id, new_product_id, new_vendor_id, NULL);
                new_product_id := subproduct_id;
                new_vendor_id := subvendor_id;
            ELSE
                parent_id := NULL;
            END IF;

            device_id := get_device_by_id(
                bus_id, category_id, new_product_id, new_vendor_id,
                parent_id);
            RETURN device_id;
        EXCEPTION
            WHEN unique_violation THEN
                NULL;  -- do nothing
        END;
    END LOOP;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_device(
        TEXT, TEXT, TEXT, TEXT, INTEGER, INTEGER, INTEGER, INTEGER)
    TO lpresults
""",
"""
CREATE FUNCTION devicevendor_vendor() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF TG_OP = 'INSERT' THEN
        IF NEW.vendor IS NULL THEN
            NEW.vendor := nextval('devicevendor_vendor_seq');
        ELSIF NEW.vendor >= 65536 THEN
            RAISE EXCEPTION 'vendor must be less than 2^16: %s', NEW.vendor;
        END IF;

        IF NEW.text_search_vector IS NULL THEN
            NEW.text_search_vector :=
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.name), '')), 'A');
        END IF;
    ELSIF TG_OP = 'UPDATE' THEN
        IF NOT NEW.vendor = OLD.vendor THEN
            RAISE EXCEPTION 'vendor cannot be updated';
        END IF;

        IF NEW.text_search_vector = OLD.text_search_vector
            AND NOT NEW.name = OLD.name THEN
            NEW.text_search_vector :=
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.name), '')), 'A');
        END IF;
    END IF;
    RETURN NEW;
END
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION devicevendor_vendor()
    TO lpresults
""",
"""
CREATE TRIGGER devicevendor_vendor_trigger
    BEFORE INSERT OR UPDATE ON devicevendor
    FOR EACH ROW EXECUTE PROCEDURE devicevendor_vendor()
""",
"""
CREATE FUNCTION deviceproduct_product() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF TG_OP = 'INSERT' THEN
        IF NEW.product IS NULL THEN
            NEW.product := nextval('deviceproduct_product_seq');
        ELSIF NEW.product >= 65536 THEN
            RAISE EXCEPTION 'product must be less than 2^16: %s', NEW.product;
        END IF;

        IF NEW.text_search_vector IS NULL THEN
            NEW.text_search_vector :=
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.name), '')), 'A');
        END IF;
    ELSIF TG_OP = 'UPDATE' THEN
        IF NOT NEW.product = OLD.product THEN
            RAISE EXCEPTION 'product cannot be updated';
        END IF;

        IF NEW.text_search_vector = OLD.text_search_vector
            AND NOT NEW.name = OLD.name THEN
            NEW.text_search_vector :=
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.name), '')), 'A');
        END IF;
    END IF;
    RETURN NEW;
END
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION deviceproduct_product()
    TO lpresults
""",
"""
CREATE TRIGGER deviceproduct_product_trigger
    BEFORE INSERT OR UPDATE ON deviceproduct
    FOR EACH ROW EXECUTE PROCEDURE deviceproduct_product()
""",
"""
CREATE FUNCTION devicestate_update() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF NOT OLD.date_deleted = 'infinity' THEN  -- can be non-infinity
        RETURN NULL; -- no update
    END IF;
    IF NEW.date_deleted = 'infinity' THEN
        INSERT INTO devicestate VALUES (
            OLD.id,
            OLD.system,
            OLD.device,
            OLD.subdevice,
            OLD.driver,
            OLD.count,
            OLD.date_created, now());
        NEW.date_created = now();
    END IF;
    RETURN NEW;
END
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION devicestate_update()
    TO lpresults
""",
"""
CREATE TRIGGER devicestate_update BEFORE UPDATE ON devicestate
    FOR EACH ROW EXECUTE PROCEDURE devicestate_update()
""",
"""
CREATE FUNCTION system_make_model() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF TG_OP = 'INSERT' THEN
        IF NEW.text_search_vector IS NULL THEN
            NEW.text_search_vector :=
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.make), '')), 'A') ||
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.model), '')), 'B');
        END IF;
    ELSIF TG_OP = 'UPDATE' THEN
        IF NEW.text_search_vector = OLD.text_search_vector
            AND NOT (NEW.make = OLD.make AND NEW.model = OLD.model) THEN
            NEW.text_search_vector :=
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.make), '')), 'A') ||
                SETWEIGHT(TO_TSVECTOR(
                    'simple', COALESCE(text_as_grams(NEW.model), '')), 'B');
        END IF;
    END IF;
    RETURN NEW;
END
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION system_make_model()
    TO lpresults
""",
"""
CREATE TRIGGER system_make_model_trigger
    BEFORE INSERT OR UPDATE ON system
    FOR EACH ROW EXECUTE PROCEDURE system_make_model()
""",
"""
ALTER TABLE ONLY device
    ADD CONSTRAINT device__bus__fk
    FOREIGN KEY (bus)
    REFERENCES devicebus(id)
""",
"""
ALTER TABLE ONLY device
    ADD CONSTRAINT device__category__fk
    FOREIGN KEY (category)
    REFERENCES devicecategory(id)
""",
"""
ALTER TABLE ONLY device
    ADD CONSTRAINT device__parent__fk
    FOREIGN KEY (parent)
    REFERENCES device(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY deviceproduct
    ADD CONSTRAINT deviceproduct__bus__fk
    FOREIGN KEY (bus)
    REFERENCES devicebus(id)
""",
"""
ALTER TABLE ONLY devicevendor
    ADD CONSTRAINT devicevendor__bus__fk
    FOREIGN KEY (bus)
    REFERENCES devicebus(id)
""",
"""
ALTER TABLE ONLY devicestate
    ADD CONSTRAINT devicestate__device__fk
    FOREIGN KEY (device)
    REFERENCES device(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY devicestate
    ADD CONSTRAINT devicestate__driver__fk
    FOREIGN KEY (driver)
    REFERENCES devicedriver(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY devicestate
    ADD CONSTRAINT devicestate__subdevice__fk
    FOREIGN KEY (subdevice)
    REFERENCES device(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY devicestate
    ADD CONSTRAINT devicestate__system_unit__fk
    FOREIGN KEY (system_unit)
    REFERENCES systemunit(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY processor
    ADD CONSTRAINT processor__platform__fk
    FOREIGN KEY (platform)
    REFERENCES processorplatform(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY systemunit
    ADD CONSTRAINT systemunit__system__fk
    FOREIGN KEY (system)
    REFERENCES system(id) ON DELETE CASCADE
""",
]

DROPS = [
"DROP FUNCTION devicestate_update()",
"DROP FUNCTION deviceproduct_product()",
"DROP FUNCTION devicevendor_vendor()",
"DROP FUNCTION set_device_vendor(TEXT, INTEGER, INTEGER)",
"DROP FUNCTION get_device_vendor(TEXT, INTEGER, INTEGER)",
"""
DROP FUNCTION set_device_product(
    TEXT, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER)
""",
"""
DROP FUNCTION get_device_product(
    TEXT, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER)
""",
"DROP FUNCTION get_device_category(TEXT)",
"DROP FUNCTION get_device_bus(TEXT)",
"DROP LANGUAGE plpgsql",
"DROP TABLE devicestate",
"DROP TABLE device",
"DROP TABLE deviceproduct",
"DROP TABLE devicevendor",
"DROP TABLE devicedriver",
"DROP TABLE devicecategory",
"DROP TABLE devicebus",
"DROP TABLE processor",
"DROP TABLE processorplatform",
"DROP TABLE systemunit",
"DROP TABLE system",
]

DELETES = [
"DELETE FROM devicestate",
"DELETE FROM device",
"DELETE FROM deviceproduct",
"DELETE FROM devicevendor",
"DELETE FROM devicedriver",
"DELETE FROM devicecategory",
"DELETE FROM devicebus",
"DELETE FROM processor",
"DELETE FROM processorplatform",
"DELETE FROM systemunit",
"DELETE FROM system",
]
