# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IDistribution",
    "IDistributionSet",
    "IHasDistribution",
    "DistributionSearchParams",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )
from zope.schema import Object

from lazr.restful.declarations import exported
from lazr.restful.fields import Reference
from lazr.restful.frameworks.django import IDjangoLocation

from lpresults.registry.interfaces.distroseries import IDistroSeries
from lpresults.registry.interfaces.pillar import IPillar
from lpresults.registry.interfaces.search import (
    ISearchSet,
    SearchParams,
    )
from lpresults.registry.interfaces.series import ISeriesTarget
from lpresults.tracker.services.patchers import patch_entry_return_type


class IDistribution(IPillar, ISeriesTarget):
    """A Distribution.

    This maps to the concept of Distribution in the Launchpad Registry.
    """
    id = Attribute("A unique identifier for the distribution")

    current_series = exported(
        Reference(
            title=u"Current series",
            schema=IDistroSeries,
            required=True))


class IDistributionSet(IDjangoLocation, ISearchSet):

    def create(name):
        """Create and return a brand new Distribution if it already exists
        in Launchpad, otherwise return None.

        See `IDistribution` for a description of the parameters.
        """

    def get(id):
        """Get a distribution by its id.

        Return None if there is no such distribution.
        """

    def getByName(name):
        """Return the distribution with the given name.

        Return None if there is no such distribution.
        """

    def search():
        """Return the default set of distributions without filtering."""


class IHasDistribution(Interface):

    distribution = Object(
        schema=IDistribution,
        title=u"Distribution",
        required=True,
        readonly=True,
        description=u"Distribution associated with this object.")


class DistributionSearchParams(SearchParams):
    """Encapsulates search parameters for IDistributionSet.search()"""

    def __init__(self, name=None, **kwargs):
        super(DistributionSearchParams, self).__init__(**kwargs)
        self.name = name


IDistribution["series"].value_type.schema = IDistroSeries
IDistroSeries["distribution"].schema = IDistribution
patch_entry_return_type(
    IDistribution, "retrieveSeries", IDistroSeries)
