# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IDistroSeries",
    "IDistroSeriesSet",
    "IHasDistroSeries",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )
from zope.schema import Object

from lazr.restful.declarations import exported
from lazr.restful.fields import Reference

from lpresults.registry.interfaces.series import (
    ISeries,
    ISeriesSet,
    )


class IDistroSeries(ISeries):
    """This maps to the concept of DistroSeries in Launchpad."""

    id = Attribute("A unique identifier for the distribution series")

    distribution_id = Attribute("A unique identifier for the distribution")

    distribution = exported(
        Reference(
            title=u"Distribution",
            schema=Interface,  # IDistribution
            required=True))


class IDistroSeriesSet(ISeriesSet):
    """Interface representing the set of DistroSeries."""


class IHasDistroSeries(Interface):

    distro_series = Object(
        schema=IDistroSeries,
        title=u"Distribution series",
        required=True,
        readonly=True,
        description=u"Distribution series associated with this object.")
