# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IPillar",
    "IPillarName",
    "IPillarSet",
    "PillarSearchParams",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )
from zope.schema import TextLine

from lazr.restful.declarations import exported
from lazr.restful.frameworks.django import IDjangoLocation
from lazr.restful.interfaces import ITopLevelEntryLink

from lpresults.registry.interfaces.search import (
    ISearchSet,
    SearchParams,
    )
from lpresults.registry.validators import name_validator


class IPillar(IDjangoLocation):
    """An object that might be a project or a distribution.

    This is a polymorphic object served by the pillar set. Check the
    individual object to see what type it is.
    """
    name = exported(
        TextLine(
            title=u"Name",
            required=True,
            readonly=True,
            constraint=name_validator,
            description=u"A short name for the project"))


class IPillarName(Interface):
    """A data structure for identifying a pillar.

    This includes the pillar object, as well as information about whether
    it's a project or distribution.
    """
    name = TextLine(title=u"The name.")

    pillar = Attribute("The pillar object")


class IPillarSet(ITopLevelEntryLink, IDjangoLocation, ISearchSet):
    """An object for searching across projects and distributions.

    Projects and distributions are collectively known as "pillars". This
    object lets you do a combined search across all types of pillars.
    """
    def createDistribution(*args, **kwargs):
        """Create and return a Distribution pillar.

        See `IDistributionSet.create` for a description of the parameters.
        """

    def createProject(*args, **kwargs):
        """Create and return a Project pillar.

        See `IProjectSet.create` for a description of the parameters.
        """

    def getByName(name):
        """Return the pillar whose name matches the given name.

        If no pillar is found, return None.
        """

    def retrieveByName(name):
        """Return the pillar for the given name, or a new pillar
        retrieved from Launchpad.

        :param name: The name of the pillar to retrieve.
        """


class PillarSearchParams(SearchParams):
    """Encapsulates search parameters for IPillarSet.search()"""

    def __init__(self, name=None, **kwargs):
        super(PillarSearchParams, self).__init__(**kwargs)
        self.name = name
