# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IProject",
    "IProjectSet",
    "IHasProject",
    "ProjectSearchParams",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )
from zope.schema import Object

from lazr.restful.declarations import exported
from lazr.restful.fields import Reference
from lazr.restful.frameworks.django import IDjangoLocation

from lpresults.registry.interfaces.pillar import IPillar
from lpresults.registry.interfaces.projectseries import IProjectSeries
from lpresults.registry.interfaces.search import (
    ISearchSet,
    SearchParams,
    )
from lpresults.registry.interfaces.series import ISeriesTarget
from lpresults.tracker.services.patchers import patch_entry_return_type


class IProject(IPillar, ISeriesTarget):
    """A Project.

    This maps to the concept of Product in the Launchpad Registry,
    renamed to Project.

    See: https://bugs.launchpad.net/launchpad/+bug/109153
    """
    id = Attribute("A unique identifier for the project")

    development_focus = exported(
        Reference(
            title=u"Development focus",
            schema=IProjectSeries,
            required=True))


class IProjectSet(IDjangoLocation, ISearchSet):

    def create(name):
        """Create and return a brand new Project.

        See `IProject` for a description of the parameters.
        """

    def get(id):
        """Get a project by its id.

        Return None if there is no such project.
        """

    def getByName(name):
        """Return the project with the given name.

        Return None if there is no such project.
        """

    def search():
        """Return the default set of projects without filtering."""


class IHasProject(Interface):

    project = Object(
        schema=IProject,
        title=u"Project",
        required=True,
        readonly=True,
        description=u"Project associated with this object.")


class ProjectSearchParams(SearchParams):
    """Encapsulates search parameters for IProjectSet.search()"""

    def __init__(self, name=None, **kwargs):
        super(ProjectSearchParams, self).__init__(**kwargs)
        self.name = name


IProject["series"].value_type.schema = IProjectSeries
IProjectSeries["project"].schema = IProject
patch_entry_return_type(
    IProject, "retrieveSeries", IProjectSeries)
