# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IProjectSeries",
    "IProjectSeriesSet",
    "IHasProjectSeries",
    ]

from zope.interface import (
    Attribute,
    Interface,
    )
from zope.schema import Object

from lazr.restful.declarations import exported
from lazr.restful.fields import Reference

from lpresults.registry.interfaces.series import (
    ISeries,
    ISeriesSet,
    )


class IProjectSeries(ISeries):
    """This maps to the concept of ProductSeries in Launchpad."""

    id = Attribute("A unique identifier for the project series")

    project_id = Attribute("A unique identifier for the project")

    project = exported(
        Reference(
            title=u"Project",
            schema=Interface,  # IProject
            required=True))


class IProjectSeriesSet(ISeriesSet):
    """Interface representing the set of ProjectSeries."""


class IHasProjectSeries(Interface):

    project_series = Object(
        schema=IProjectSeries,
        title=u"Project series",
        required=True,
        readonly=True,
        description=u"Project series associated with this object.")
