# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Anonymous",
    ]

from zope.interface import implements

from lpresults.registry.interfaces.person import IPerson
from lpresults.registry.models.oauth import OAuthToken


class AnonymousAccessToken(OAuthToken):

    def __init__(self, consumer, person):
        self.consumer = consumer
        self.person = person
        self.key = ""
        self.secret = ""


class Anonymous:
    """Users within the Django authentication system are represented by
    this model.
    """

    implements(IPerson)

    def __init__(self, consumer=None):
        self.consumer = consumer

    @property
    def oauth_access_tokens(self):
        """See `IPerson`."""
        if self.consumer is not None:
            return [AnonymousAccessToken(self.consumer, self)]
        else:
            return []

    def isAnonymous(self):
        """Always returns True. This is a way of comparing Anonymous
        users to Person objects.
        """
        return True

    def isAuthenticated(self):
        """Always return False. This is a way to tell if the user has
        been authenticated in templates.
        """
        return False

    # Compatibility layer with Django
    is_anonymous = isAnonymous
    is_authenticated = isAuthenticated
