# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DistroSeries",
    "DistroSeriesSet",
    ]

from storm.locals import (
    AutoReload,
    Int,
    Unicode,
    )

from zope.interface import implements

from lpresults.database.search import search_to_expression

from lpresults.registry.interfaces.distribution import IDistribution
from lpresults.registry.interfaces.distroseries import (
    IDistroSeries,
    IDistroSeriesSet,
    )
from lpresults.registry.models.series import (
    Series,
    SeriesSet,
    )
from lpresults.registry.models.store import get_registry_store


class DistroSeries(Series):

    implements(IDistroSeries)

    __storm_table__ = "distroseries"

    id = Int(primary=True, allow_none=False, default=AutoReload)
    distribution_id = Int(name="distribution", allow_none=False)
    name = Unicode(allow_none=False)

    # distribution reference defined after Distribution

    def __init__(self, distribution, name):
        super(DistroSeries, self).__init__(name)
        self.distribution = distribution

    @property
    def __parent__(self):
        return self.distribution

    @property
    def is_current_series(self):
        return self.id == self.distribution.current_series_id


class DistroSeriesSet(SeriesSet):
    """See `IDistroSeriesSet`."""

    implements(IDistroSeriesSet)

    def new(self, name):
        """See `ISeriesSet`."""
        return DistroSeries(self.target, name)

    def get(self, id):
        """See `ISeriesSet`."""
        return get_registry_store().get(DistroSeries, id)

    def buildSearchExpressions(self, search_params):
        """See SearchSet."""
        expressions = super(DistroSeriesSet, self).buildSearchExpressions(
            search_params)

        if IDistribution.providedBy(self.target):
            expressions.append(
                DistroSeries.distribution_id == self.target.id)

        if search_params.name is not None:
            expressions.append(
                search_to_expression(DistroSeries.name, search_params.name))

        return expressions

    def getSearchResultTables(self):
        """See SearchSet."""
        return DistroSeries
