# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DistroSeriesHelper",
    "DistroSeriesTwin",
    ]

from storm.locals import Int

from zope.interface import implements

from lpresults.registry.interfaces.distroseries import (
    IHasDistroSeries,
    IDistroSeriesSet,
    )


class DistroSeriesHelper:
    """Helper for implementing L{IHasDistroSeries} classes."""

    implements(IHasDistroSeries)

    _distro_series = None
    distro_series_id = None

    def __init__(self, distro_series=None):
        if distro_series is not None:
            if isinstance(distro_series, (int, long)):
                self.distro_series_id = distro_series
            else:
                self.distro_series = distro_series

    def _getDistroSeries(self):
        _series = self._distro_series
        series_id = self.distro_series_id
        if ((series_id is not None and not _series) or
            (_series is not None and _series.id != series_id)):

            self._distro_series = IDistroSeriesSet(self).get(series_id)

        return self._distro_series

    def _setDistroSeries(self, distro_series):
        if distro_series is None:
            self.distro_series_id = None
        else:
            self.distro_series_id = distro_series.id
        self._distro_series = distro_series

    distro_series = property(_getDistroSeries, _setDistroSeries)


class DistroSeriesTwin(DistroSeriesHelper):
    """The representation of an distribution series in the registry database.

    The primary purpose of this object is to serve as a point of relational
    stability with all distribution series objects in the registry databases.
    """
    __storm_table__ = "distroseriestwin"

    id = Int(primary=True, allow_none=False)
    distribution_id = Int(name="distribution", allow_none=False)

    def __init__(self, distro_series):
        super(DistroSeriesTwin, self).__init__(distro_series)
        self.distribution_id = distro_series.distribution_id

    # This property makes DistroSeriesHelper happy.  When DistroSeriesHelper
    # sets distro_series_id this code causes the id of this object to be the
    # same as the DistroSeries in the registry database represented by this
    # one.
    def _getDistroSeriesID(self):
        return self.id

    def _setDistroSeriesID(self, id):
        self.id = id

    distro_series_id = property(_getDistroSeriesID, _setDistroSeriesID)
