# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "launchpad_utility",
    ]

import os
import httplib2
import simplejson

from django.conf import settings
from django.http import HttpResponse

from zope.interface import implements

from lpresults.registry.interfaces.launchpad import ILaunchpad


class Launchpad:

    implements(ILaunchpad)

    service_root = settings.LAUNCHPAD_SERVICE_ROOT
    service_version = settings.LAUNCHPAD_SERVICE_VERSION
    web_root = settings.LAUNCHPAD_WEB_ROOT

    def proxy(self, request):
        headers = {"Referer": self.web_root}
        if "HTTP_ACCEPT" in request.META:
            headers["Accept"] = request.META["HTTP_ACCEPT"]

        if request.method == "GET":
            url_ending = "%s?%s" % (request.path, request.GET.urlencode())
            url = "%s%s" % (self.web_root, url_ending)
            body = None

        elif request.method == "POST":
            url = "%s%s" % (self.web_root, request.path)
            body = request.POST.urlencode()

        conn = httplib2.Http()
        response, content = conn.request(url, request.method, body, headers)
        mimetype = response["content-type"]
        status = int(response["status"])

        return HttpResponse(content, mimetype, status)

    def getPath(self, path):
        conn = httplib2.Http()
        url = self.service_root + self.service_version + "/" + path
        response, content = conn.request(url, "GET")
        status = int(response["status"])
        if status == 404:
            return None

        return simplejson.loads(unicode(content))

    def getPillar(self, pillar_name):
        return self.getPath(pillar_name)

    def getPillarSeries(self, pillar_name, series_name):
        return self.getPath(os.path.join(pillar_name, series_name))


launchpad_utility = Launchpad()
