# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "PersonHelper",
    "PersonTwin",
    ]

from storm.locals import Int

from zope.component import getUtility
from zope.interface import implements

from lpresults.registry.interfaces.person import (
    IHasPerson,
    IPersonSet,
    )


class PersonHelper:
    """Helper for implementing L{IHasPerson} classes."""

    implements(IHasPerson)

    _person = None
    person_id = None

    def __init__(self, person=None):
        if person is not None:
            if isinstance(person, (int, long)):
                self.person_id = person
            else:
                self.person = person

    def _getPerson(self):
        _person = self._person
        person_id = self.person_id
        if ((person_id is not None and not _person) or
            (_person is not None and _person.id != person_id)):

            self._person = self.getPerson(person_id)

        return self._person

    def _setPerson(self, person):
        if person is None:
            self.person_id = None
        else:
            self.person_id = person.id
        self._person = person

    person = property(_getPerson, _setPerson)

    def getPerson(self, id):
        return getUtility(IPersonSet).get(id)


class PersonTwin(PersonHelper):
    """The representation of an person in the registry database.

    The primary purpose of this object is to serve as a point of relational
    stability with all person-related objects in the registry databases.
    """
    __storm_table__ = "persontwin"

    id = Int(primary=True, allow_none=False)

    def __init__(self, person):
        super(PersonTwin, self).__init__(person)

    # This property makes PersonHelper happy.  When PersonHelper
    # sets person_id this code causes the id of this object to be the
    # same as the Person in the registry database represented by this
    # one.
    def _getPersonID(self):
        return self.id

    def _setPersonID(self, id):
        self.id = id

    person_id = property(_getPersonID, _setPersonID)
