# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ProjectSeries",
    "ProjectSeriesSet",
    ]

from storm.locals import (
    AutoReload,
    Int,
    Unicode,
    )

from zope.interface import implements

from lpresults.database.search import search_to_expression

from lpresults.registry.interfaces.project import IProject
from lpresults.registry.interfaces.projectseries import (
    IProjectSeries,
    IProjectSeriesSet,
    )
from lpresults.registry.models.series import (
    Series,
    SeriesSet,
    )
from lpresults.registry.models.store import get_registry_store


class ProjectSeries(Series):

    implements(IProjectSeries)

    __storm_table__ = "projectseries"

    id = Int(primary=True, allow_none=False, default=AutoReload)
    project_id = Int(name="project", allow_none=False)
    name = Unicode(allow_none=False)

    # project reference defined after Project

    def __init__(self, project, name):
        super(ProjectSeries, self).__init__(name)
        self.project = project

    @property
    def __parent__(self):
        return self.project

    @property
    def is_development_focus(self):
        return self.id == self.project.development_focus_id


class ProjectSeriesSet(SeriesSet):
    """See `IProjectSeriesSet`."""

    implements(IProjectSeriesSet)

    def new(self, name):
        """See `ISeriesSet`."""
        return ProjectSeries(self.target, name)

    def get(self, id):
        """See `ISeriesSet`."""
        return get_registry_store().get(ProjectSeries, id)

    def buildSearchExpressions(self, search_params):
        """See SearchSet."""
        expressions = super(ProjectSeriesSet, self).buildSearchExpressions(
            search_params)

        if IProject.providedBy(self.target):
            expressions.append(ProjectSeries.project_id == self.target.id)

        if search_params.name is not None:
            expressions.append(
                search_to_expression(
                    ProjectSeries.name, search_params.name))

        return expressions

    def getSearchResultTables(self):
        """See SearchSet."""
        return ProjectSeries
