# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ProjectSeriesHelper",
    "ProjectSeriesTwin",
    ]

from storm.locals import Int

from zope.interface import implements

from lpresults.registry.interfaces.projectseries import (
    IHasProjectSeries,
    IProjectSeriesSet,
    )


class ProjectSeriesHelper:
    """Helper for implementing L{IHasProjectSeries} classes."""

    implements(IHasProjectSeries)

    _project_series = None
    project_series_id = None

    def __init__(self, project_series=None):
        if project_series is not None:
            if isinstance(project_series, (int, long)):
                self.project_series_id = project_series
            else:
                self.project_series = project_series

    def _getProjectSeries(self):
        _series = self._project_series
        series_id = self.project_series_id
        if ((series_id is not None and not _series) or
            (_series is not None and _series.id != series_id)):

            self._project_series = IProjectSeriesSet(self).get(series_id)

        return self._project_series

    def _setProjectSeries(self, project_series):
        if project_series is None:
            self.project_series_id = None
        else:
            self.project_series_id = project_series.id

        self._project_series = project_series

    project_series = property(_getProjectSeries, _setProjectSeries)


class ProjectSeriesTwin(ProjectSeriesHelper):
    """The representation of an project series in the registry database.

    The primary purpose of this object is to serve as a point of relational
    stability with all project series objects in the registry databases.
    """
    __storm_table__ = "projectseriestwin"

    id = Int(primary=True, allow_none=False)
    project_id = Int(name="project", allow_none=False)

    def __init__(self, project_series):
        super(ProjectSeriesTwin, self).__init__(project_series)
        self.project_id = project_series.project_id

    # This property makes ProjectSeriesHelper happy.  When ProjectSeriesHelper
    # sets project_series_id this code causes the id of this object to be the
    # same as the ProjectSeries in the registry database represented by this
    # one.
    def _getProjectSeriesID(self):
        return self.id

    def _setProjectSeriesID(self, id):
        self.id = id

    project_series_id = property(_getProjectSeriesID, _setProjectSeriesID)
