# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Series",
    "SeriesSet",
    "SeriesTargetMixin",
    ]

from urlparse import urlparse

from zope.component import getUtility
from zope.event import notify
from zope.interface import implements

from lazr.lifecycle.event import ObjectCreatedEvent
from lazr.restful.simple import TraverseWithGet

from lpresults.registry.models.search import SearchSet
from lpresults.registry.models.store import get_registry_store
from lpresults.registry.interfaces.launchpad import ILaunchpad
from lpresults.registry.interfaces.series import (
    ISeries,
    ISeriesSet,
    ISeriesTarget,
    SeriesSearchParams,
    )


class Series:

    implements(ISeries)

    def __init__(self, name):
        self.name = name

    @property
    def __url_path__(self):
        return self.name


class SeriesSet(SearchSet):
    """See `ISeriesSet`."""

    implements(ISeriesSet)

    search_params_factory = SeriesSearchParams

    def __init__(self, target):
        self.target = target

    def new(self, name):
        """See `ISeriesSet`."""
        raise NotImplementedError()

    def get(self, id):
        """See `ISeriesSet`."""
        raise NotImplementedError()

    def getByName(self, name):
        """See `ISeriesSet`."""
        search_params = self.search_params_factory(name=name)
        return self.search(search_params).one()

    def getSearchStore(self, search_aprams):
        """See SearchSet."""
        return get_registry_store()

    def search(self, *args):
        if not args:
            search_params = self.search_params_factory()
            args = (search_params,)

        return super(SeriesSet, self).search(*args)


class SeriesTargetMixin(TraverseWithGet):

    implements(ISeriesTarget)

    @property
    def series(self):
        """See `IHasSeries`."""
        return ISeriesSet(self).search()

    def get(self, request, name):
        """See `ITraverseWithGet`."""
        value = super(SeriesTargetMixin, self).get(request, name)
        if value is None:
            value = self.getSeries(name)

        return value

    def getSeries(self, name):
        """See `IHasSeries`."""
        return ISeriesSet(self).getByName(name)

    def newSeries(self, name):
        """See `ISeriesTarget`."""
        series = ISeriesSet(self).new(name)
        notify(ObjectCreatedEvent(series))
        return series

    def retrieveSeries(self, name):
        """See `ISeriesTarget`."""
        series = self.getSeries(name)
        if series is None:
            launchpad_series = getUtility(ILaunchpad).getPillarSeries(
                self.name, name)
            if launchpad_series is not None:
                resource_type_link = launchpad_series["resource_type_link"]
                resource_type = urlparse(resource_type_link)[5]
                if resource_type not in ("distro_series", "project_series",):
                    raise AssertionError("Unknown series type: %s" % name)

                series = self.newSeries(name)

        return series
