# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.registry.interfaces.distribution import (
    IDistributionSet,
    DistributionSearchParams,
    )
from lpresults.registry.models.tests.test_series import TestSeriesTargetMixin
from lpresults.registry.testing.cases import TestCaseWithFactory
from lpresults.registry.testing.layers import FunctionalDatabaseLayer


class TestDistributionSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_create(self):
        distro_name = self.factory.getUniqueString(u"distribution-name")
        series_name = self.factory.getUniqueString(u"series-name")
        distribution = getUtility(IDistributionSet).create(
            distro_name, series_name)
        self.assertNotEquals(distribution, None)
        self.assertEquals(distribution.name, distro_name)
        self.assertEquals(distribution.current_series.name, series_name)

    def test_get(self):
        distribution = self.factory.makeDistribution()
        other = getUtility(IDistributionSet).get(distribution.id)
        self.assertNotEquals(other, None)
        self.assertEquals(other, distribution)

    def test_getByName(self):
        name = self.factory.getUniqueString(u"distribution-name")
        other = getUtility(IDistributionSet).getByName(name)
        self.assertEquals(other, None)

        distribution = self.factory.makeDistribution(name=name)
        other = getUtility(IDistributionSet).getByName(name)
        self.assertNotEquals(other, None)
        self.assertEquals(other, distribution)

    def test_searchName(self):
        name = self.factory.getUniqueString(u"distribution-name")
        params = DistributionSearchParams(name=name)
        other = getUtility(IDistributionSet).search(params).one()
        self.assertEquals(other, None)

        distribution = self.factory.makeDistribution(name=name)
        other = getUtility(IDistributionSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, distribution)


class TestSeriesTarget(TestCaseWithFactory, TestSeriesTargetMixin):

    def getSeriesTarget(self):
        return self.factory.makeDistribution()


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
