# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import provideAdapter
from zope.interface.verify import verifyObject

from lpresults.registry.interfaces.distroseries import (
    IHasDistroSeries,
    IDistroSeriesSet,
    )
from lpresults.registry.models.distroseries import DistroSeriesSet
from lpresults.registry.models.distroserieshelper import DistroSeriesHelper
from lpresults.registry.testing.cases import TestCaseWithFactory
from lpresults.registry.testing.layers import FunctionalDatabaseLayer


class FakeDistroSeriesTarget(DistroSeriesHelper):

    def __init__(self, distro_series_id):
        self.distro_series_id = distro_series_id


class TestDistroSeriesHelper(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def setUp(self):
        super(TestDistroSeriesHelper, self).setUp()
        provideAdapter(
            DistroSeriesSet, (FakeDistroSeriesTarget,), IDistroSeriesSet)

    def test_hasDistroSeries(self):
        helper = DistroSeriesHelper()
        self.assertTrue(verifyObject(IHasDistroSeries, helper))

    def test_getDistroSeries(self):
        distro_series = self.factory.makeDistroSeries()
        target = FakeDistroSeriesTarget(distro_series.id)
        self.assertEquals(target.distro_series, distro_series)

    def test_withDistroSeries(self):
        distro_series = self.factory.makeDistroSeries()
        helper = DistroSeriesHelper(distro_series)
        self.assertEquals(helper.distro_series, distro_series)

    def test_withoutDistroSeries(self):
        helper = DistroSeriesHelper()
        self.assertEquals(helper.distro_series, None)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
