# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.registry.interfaces.person import (
    IPerson,
    IPersonSet,
    PersonSearchParams,
    )
from lpresults.registry.testing.cases import TestCaseWithFactory
from lpresults.registry.testing.layers import FunctionalDatabaseLayer


class TestPersonSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_createPerson(self):
        person = getUtility(IPersonSet).create(u"me", u"123")
        self.assertNotEquals(person, None)
        self.assertTrue(IPerson.providedBy(person))
        self.assertEquals(person.name, u"me")
        self.assertEquals(person.identity, u"123")

    def test_get(self):
        person = self.factory.makePerson()
        other = getUtility(IPersonSet).get(person.id)
        self.assertNotEquals(other, None)
        self.assertTrue(IPerson.providedBy(other))
        self.assertEquals(other, person)

    def test_getByIdentity(self):
        identity = self.factory.getUniqueString(u"person-identity")
        other = getUtility(IPersonSet).getByIdentity(identity)
        self.assertEquals(other, None)

        person = self.factory.makePerson(identity=identity)
        other = getUtility(IPersonSet).getByIdentity(identity)
        self.assertNotEquals(other, None)
        self.assertTrue(IPerson.providedBy(other))
        self.assertEquals(other, person)

    def test_getByName(self):
        name = self.factory.getUniqueString(u"person-name")
        other = getUtility(IPersonSet).getByName(name)
        self.assertEquals(other, None)

        person = self.factory.makePerson(name=name)
        other = getUtility(IPersonSet).getByName(name)
        self.assertNotEquals(other, None)
        self.assertTrue(IPerson.providedBy(other))
        self.assertEquals(other, person)

    def test_searchName(self):
        name = self.factory.getUniqueString(u"person-name")
        params = PersonSearchParams(name=name)
        other = getUtility(IPersonSet).search(params).one()
        self.assertEquals(other, None)

        person = self.factory.makePerson(name=name)
        other = getUtility(IPersonSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertTrue(IPerson.providedBy(other))
        self.assertEquals(other, person)

    def test_searchIdentity(self):
        identity = self.factory.getUniqueString(u"person-identity")
        params = PersonSearchParams(identity=identity)
        other = getUtility(IPersonSet).search(params).one()
        self.assertEquals(other, None)

        person = self.factory.makePerson(identity=identity)
        other = getUtility(IPersonSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertTrue(IPerson.providedBy(other))
        self.assertEquals(other, person)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
