# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.registry.interfaces.distribution import IDistribution
from lpresults.registry.interfaces.pillar import (
    IPillarSet,
    PillarSearchParams,
    )
from lpresults.registry.interfaces.project import IProject
from lpresults.registry.testing.cases import TestCaseWithFactory
from lpresults.registry.testing.layers import FunctionalDatabaseLayer


class TestPillarSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_getByName(self):
        name = self.factory.getUniqueString(u"project-name")
        other = getUtility(IPillarSet).getByName(name)
        self.assertEquals(other, None)

        # Get exact name
        project = self.factory.makeProject(name=name)
        other = getUtility(IPillarSet).getByName(name)
        self.assertNotEquals(other, None)
        self.assertEquals(other, project)

        # Get name prefix
        other = getUtility(IPillarSet).getByName(name[:-1])
        self.assertEquals(other, None)

        # Get name suffix
        other = getUtility(IPillarSet).getByName(name[1:])
        self.assertEquals(other, None)

    def test_retrieveUnknown(self):
        name = self.factory.getUniqueString(u"name")
        pillar = getUtility(IPillarSet).retrieveByName(name)
        self.assertEquals(pillar, None)

    def test_retrieveProject(self):
        name = self.factory.getUniqueString(u"project-name")
        project = getUtility(IPillarSet).retrieveByName(name)
        self.assertNotEquals(project, None)
        self.assertTrue(IProject.providedBy(project))
        self.assertEquals(project.name, name)

    def test_retrieveDistribution(self):
        name = self.factory.getUniqueString(u"distribution-name")
        distribution = getUtility(IPillarSet).retrieveByName(name)
        self.assertNotEquals(distribution, None)
        self.assertTrue(IDistribution.providedBy(distribution))
        self.assertEquals(distribution.name, name)

    def test_searchProjectName(self):
        name = self.factory.getUniqueString(u"project-name")
        params = PillarSearchParams(name=name)
        other = getUtility(IPillarSet).search(params).one()
        self.assertEquals(other, None)

        project = self.factory.makeProject(name=name)
        other = getUtility(IPillarSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, project)

    def test_searchDistributionName(self):
        name = self.factory.getUniqueString(u"distribution-name")
        params = PillarSearchParams(name=name)
        other = getUtility(IPillarSet).search(params).one()
        self.assertEquals(other, None)

        distribution = self.factory.makeDistribution(name=name)
        other = getUtility(IPillarSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, distribution)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
