# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.registry.interfaces.project import (
    IProjectSet,
    ProjectSearchParams,
    )
from lpresults.registry.models.tests.test_series import TestSeriesTargetMixin
from lpresults.registry.testing.cases import TestCaseWithFactory
from lpresults.registry.testing.layers import FunctionalDatabaseLayer


class TestProjectSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_create(self):
        project_name = self.factory.getUniqueString(u"project-name")
        series_name = self.factory.getUniqueString(u"series-name")
        project = getUtility(IProjectSet).create(project_name, series_name)
        self.assertNotEquals(project, None)
        self.assertEquals(project.name, project_name)
        self.assertEquals(project.development_focus.name, series_name)

    def test_get(self):
        project = self.factory.makeProject()
        other = getUtility(IProjectSet).get(project.id)
        self.assertNotEquals(other, None)
        self.assertEquals(other, project)

    def test_getByName(self):
        name = self.factory.getUniqueString(u"project-name")
        other = getUtility(IProjectSet).getByName(name)
        self.assertEquals(other, None)

        project = self.factory.makeProject(name=name)
        other = getUtility(IProjectSet).getByName(name)
        self.assertNotEquals(other, None)
        self.assertEquals(other, project)

    def test_searchEmpty(self):
        name = self.factory.getUniqueString(u"project-name")
        params = ProjectSearchParams(name=name)
        other = getUtility(IProjectSet).search(params).one()
        self.assertEquals(other, None)

    def test_searchName(self):
        name = self.factory.getUniqueString(u"project-name")
        project = self.factory.makeProject(name=name)

        # Search by name
        params = ProjectSearchParams(name=name)
        other = getUtility(IProjectSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, project)


class TestSeriesTarget(TestCaseWithFactory, TestSeriesTargetMixin):

    def getSeriesTarget(self):
        return self.factory.makeProject()


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
