# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import provideAdapter
from zope.interface.verify import verifyObject

from lpresults.registry.interfaces.projectseries import (
    IHasProjectSeries,
    IProjectSeriesSet,
    )
from lpresults.registry.models.projectseries import ProjectSeriesSet
from lpresults.registry.models.projectserieshelper import ProjectSeriesHelper
from lpresults.registry.testing.cases import TestCaseWithFactory
from lpresults.registry.testing.layers import FunctionalDatabaseLayer


class FakeProjectSeriesTarget(ProjectSeriesHelper):

    def __init__(self, project_series_id):
        self.project_series_id = project_series_id


class TestProjectSeriesHelper(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def setUp(self):
        super(TestProjectSeriesHelper, self).setUp()
        provideAdapter(
            ProjectSeriesSet, (FakeProjectSeriesTarget,), IProjectSeriesSet)

    def test_hasProjectSeries(self):
        helper = ProjectSeriesHelper()
        self.assertTrue(verifyObject(IHasProjectSeries, helper))

    def test_getProjectSeries(self):
        project_series = self.factory.makeProjectSeries()
        target = FakeProjectSeriesTarget(project_series.id)
        self.assertEquals(target.project_series, project_series)

    def test_withProjectSeries(self):
        project_series = self.factory.makeProjectSeries()
        helper = ProjectSeriesHelper(project_series)
        self.assertEquals(helper.project_series, project_series)

    def test_withoutProjectSeries(self):
        helper = ProjectSeriesHelper()
        self.assertEquals(helper.project_series, None)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
