# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestSeriesSetMixin",
    "TestSeriesTargetMixin",
    ]

from unittest import defaultTestLoader

from lpresults.registry.interfaces.series import (
    ISeries,
    ISeriesSet,
    )
from lpresults.registry.testing.layers import FunctionalDatabaseLayer


class TestSeriesSetMixin:

    layer = FunctionalDatabaseLayer

    def getSeriesContext(self):
        raise NotImplementedError()

    def test_new(self):
        context = self.getSeriesContext()
        name = self.factory.getUniqueString(u"series-name")
        series = ISeriesSet(context).new(name)
        self.assertNotEquals(series, None)
        self.assertTrue(ISeries.providedBy(series))
        self.assertEquals(series.name, name)

    def test_getByName(self):
        context = self.getSeriesContext()
        name = self.factory.getUniqueString(u"series-name")
        series = ISeriesSet(context).getByName(name)
        self.assertEquals(series, None)

        ISeriesSet(context).new(name)
        series = ISeriesSet(context).getByName(name)
        self.assertNotEquals(series, None)
        self.assertEquals(series.name, name)


class TestSeriesTargetMixin:

    layer = FunctionalDatabaseLayer

    def getSeriesTarget(self):
        raise NotImplementedError()

    def test_series(self):
        target = self.getSeriesTarget()
        # The target will at least have a development_focus or current_series
        self.assertEquals(target.series.count(), 1)

        name = self.factory.getUniqueString(u"series-name")
        series = target.newSeries(name)
        self.assertEquals(target.series.count(), 2)
        self.assertTrue(series in target.series)

    def test_getSeries(self):
        target = self.getSeriesTarget()
        name = self.factory.getUniqueString(u"series-name")
        series = target.getSeries(name)
        self.assertEquals(series, None)

        target.newSeries(name)
        series = target.getSeries(name)
        self.assertNotEquals(series, None)

    def test_newSeries(self):
        target = self.getSeriesTarget()
        name = self.factory.getUniqueString(u"series-name")
        series = target.newSeries(name)
        self.assertEquals(series.name, name)

    def test_retrieveSeries(self):
        target = self.getSeriesTarget()
        name = self.factory.getUniqueString(u"series-name")
        series = target.retrieveSeries(name)
        self.assertNotEquals(series, None)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
