# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = [
    "create",
    ]

from storm.zope.schema import ZSchema


def create():
    from lpresults.registry import schema as patch_package

    return ZSchema(CREATES, DROPS, DELETES, patch_package)


CREATES = [
"""
GRANT SELECT
    ON TABLE patch
    TO lpresults
""",
"""
CREATE LANGUAGE plpythonu
""",
"""
CREATE FUNCTION valid_name(TEXT) RETURNS BOOLEAN
LANGUAGE plpythonu IMMUTABLE RETURNS NULL ON NULL INPUT AS
$$
    import re
    name = args[0]
    pat = r"^[a-z0-9][a-z0-9\+\.\-]*\Z"
    if re.match(pat, name):
        return 1
    return 0
$$
""",
"""
CREATE TABLE distribution (
    id SERIAL NOT NULL PRIMARY KEY,
    name TEXT NOT NULL,
    current_series INTEGER,
    CONSTRAINT valid_name CHECK (valid_name(name)))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE distribution
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE distribution_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX distribution__name__key
    ON distribution (name)
""",
"""
CREATE TABLE distroseries (
    id SERIAL NOT NULL PRIMARY KEY,
    distribution INTEGER NOT NULL,
    name TEXT NOT NULL,
    CONSTRAINT valid_name CHECK (valid_name(name)))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE distroseries
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE distroseries_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX distroseries__distribution__name__key
    ON distroseries (distribution, name)
""",
"""
CREATE TABLE project (
    id SERIAL NOT NULL PRIMARY KEY,
    name TEXT NOT NULL,
    development_focus INTEGER,
    CONSTRAINT valid_name CHECK (valid_name(name)))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE project
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE project_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX project__name__key
    ON project (name)
""",
"""
CREATE TABLE projectseries (
    id SERIAL NOT NULL PRIMARY KEY,
    project INTEGER NOT NULL,
    name TEXT NOT NULL,
    CONSTRAINT valid_name CHECK (valid_name(name)))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE projectseries
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE projectseries_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX projectseries__project__name__key
    ON projectseries (project, name)
""",
"""
CREATE TABLE pillarname (
    id SERIAL NOT NULL PRIMARY KEY,
    name TEXT NOT NULL,
    distribution INTEGER,
    project INTEGER,
    CONSTRAINT valid_name CHECK (valid_name(name)),
    CONSTRAINT valid_target CHECK (
        (distribution IS NOT NULL) OR (project IS NOT NULL)))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE pillarname
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE pillarname_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX pillarname__name__key
    ON pillarname (name)
""",
"""
CREATE UNIQUE INDEX pillarname__distribution__key
    ON pillarname (distribution)
    WHERE distribution IS NOT NULL
""",
"""
CREATE UNIQUE INDEX pillarname__project__key
    ON pillarname (project)
    WHERE project IS NOT NULL
""",
"""
CREATE TABLE person (
    id SERIAL NOT NULL PRIMARY KEY,
    name TEXT NOT NULL,
    identity TEXT,
    CONSTRAINT valid_name CHECK (valid_name(name)))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE person
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE person_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX person__name__key
    ON person (name)
""",
"""
CREATE UNIQUE INDEX person__identity__key
    ON person (identity)
    WHERE identity IS NOT NULL
""",
"""
CREATE TABLE oauthconsumer (
    id SERIAL NOT NULL PRIMARY KEY,
    key TEXT NOT NULL,
    secret TEXT NOT NULL,
    date_created TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE oauthconsumer
    TO lpresults
""",
"""
GRANT SELECT, UPDATE
    ON TABLE oauthconsumer_id_seq
    TO lpresults
""",
"""
CREATE UNIQUE INDEX oauthconsumer__key__key
    ON oauthconsumer (key)
""",
"""
CREATE TABLE oauthaccesstoken (
    key TEXT NOT NULL PRIMARY KEY,
    secret TEXT NOT NULL,
    consumer INTEGER NOT NULL,
    person INTEGER NOT NULL,
    permission INTEGER NOT NULL,
    date_created TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE oauthaccesstoken
    TO lpresults
""",
"""
CREATE TABLE oauthrequesttoken (
    key TEXT NOT NULL PRIMARY KEY,
    secret TEXT NOT NULL,
    consumer INTEGER NOT NULL,
    person INTEGER,
    permission INTEGER,
    date_created TIMESTAMP WITHOUT TIME ZONE NOT NULL
        DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'),
    date_reviewed TIMESTAMP WITHOUT TIME ZONE,
    CONSTRAINT reviewed_request CHECK (
        ((date_reviewed IS NULL) = (person IS NULL))
        AND ((date_reviewed IS NULL) = (permission IS NULL))))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE oauthrequesttoken
    TO lpresults
""",
"""
CREATE LANGUAGE plpgsql
""",
"""
CREATE FUNCTION distribution_insert_or_update_after() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF TG_OP = 'INSERT' THEN
        INSERT INTO pillarname (
            name,
            distribution)
        VALUES (
            NEW.name,
            NEW.id);
    ELSIF NEW.name != OLD.name THEN
        UPDATE pillarname
            SET name = NEW.name
            WHERE distribution = NEW.id;
    END IF;
    RETURN NULL;  -- ignored
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION distribution_insert_or_update_after()
    TO lpresults
""",
"""
CREATE TRIGGER distribution_insert_or_update_after_trigger
    AFTER INSERT OR UPDATE ON distribution
    FOR EACH ROW
    EXECUTE PROCEDURE distribution_insert_or_update_after()
""",
"""
CREATE FUNCTION project_insert_or_update_after() RETURNS TRIGGER
LANGUAGE plpgsql AS
$$
BEGIN
    IF TG_OP = 'INSERT' THEN
        INSERT INTO pillarname (
            name,
            project)
        VALUES (
            NEW.name,
            NEW.id);
    ELSIF NEW.name != OLD.name THEN
        UPDATE pillarname
            SET name = NEW.name
            WHERE project = NEW.id;
    END IF;
    RETURN NULL;  -- ignored
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION project_insert_or_update_after()
    TO lpresults
""",
"""
CREATE TRIGGER project_insert_or_update_after_trigger
    AFTER INSERT OR UPDATE ON project
    FOR EACH ROW
    EXECUTE PROCEDURE project_insert_or_update_after()
""",
"""
ALTER TABLE ONLY distribution
    ADD CONSTRAINT distribution__current_series__fk
    FOREIGN KEY (current_series)
    REFERENCES distroseries(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY distroseries
    ADD CONSTRAINT distroseries__distribution__fk
    FOREIGN KEY (distribution)
    REFERENCES distribution(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY oauthaccesstoken
    ADD CONSTRAINT oauthaccesstoken__consumer__fk
    FOREIGN KEY (consumer)
    REFERENCES oauthconsumer(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY oauthaccesstoken
    ADD CONSTRAINT oauthaccesstoken__person__fk
    FOREIGN KEY (person)
    REFERENCES person(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY oauthrequesttoken
    ADD CONSTRAINT oauthrequesttoken__consumer__fk
    FOREIGN KEY (consumer)
    REFERENCES oauthconsumer(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY oauthrequesttoken
    ADD CONSTRAINT oauthrequesttoken__person__fk
    FOREIGN KEY (person)
    REFERENCES person(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY pillarname
    ADD CONSTRAINT pillarname__distribution__fk
    FOREIGN KEY (distribution)
    REFERENCES distribution(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY pillarname
    ADD CONSTRAINT pillarname__project__fk
    FOREIGN KEY (project)
    REFERENCES project(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY project
    ADD CONSTRAINT project__development_focus__fk
    FOREIGN KEY (development_focus)
    REFERENCES projectseries(id) ON DELETE CASCADE
""",
"""
ALTER TABLE ONLY projectseries
    ADD CONSTRAINT projectseries__project__fk
    FOREIGN KEY (project)
    REFERENCES project(id) ON DELETE CASCADE
""",
]

DROPS = [
"DROP FUNCTION project_insert_or_update_after",
"DROP FUNCTION distribution_insert_or_update_after",
"DROP LANGUAGE plpgsql",
"DROP TABLE oauthrequesttoken",
"DROP TABLE oauthaccesstoken",
"DROP TABLE oauthconsumer",
"DROP TABLE person",
"DROP TABLE pillarname",
"DROP TABLE projectseries",
"DROP TABLE project",
"DROP TABLE distroseries",
"DROP TABLE distribution",
"DROP FUNCTION valid_name(TEXT)",
"DROP LANGUAGE plpythonu",
]

DELETES = [
"DELETE FROM oauthrequesttoken",
"DELETE FROM oauthaccesstoken",
"DELETE FROM oauthconsumer",
"DELETE FROM person",
"DELETE FROM pillarname",
"DELETE FROM projectseries",
"DELETE FROM project",
"DELETE FROM distroseries",
"DELETE FROM distribution",
]
