# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestFactory",
    ]

from zope.component import getUtility

from lpresults.database.store import get_store
from lpresults.database.testing import factories as database_factories

from lpresults.registry.interfaces.distribution import IDistributionSet
from lpresults.registry.interfaces.person import IPersonSet
from lpresults.registry.interfaces.project import IProjectSet


class DatabaseFactory(database_factories.DatabaseFactory):

    def getRegistryStore(self):
        return get_store("registry")


class TestFactory(DatabaseFactory):

    def makePerson(self, name=None, identity=None):
        if name is None:
            name = self.getUniqueString(u"person-name")

        return getUtility(IPersonSet).create(name, identity)

    def makeDistribution(self, name=None, series_name=None):
        if name is None:
            name = self.getUniqueString(u"distribution-name")
        if series_name is None:
            series_name = self.getUniqueString(u"series-name")

        return getUtility(IDistributionSet).create(name, series_name)

    def makeDistroSeries(self, name=None, distribution_name=None):
        distribution = self.makeDistribution(distribution_name, name)
        return distribution.current_series

    def makeProject(self, name=None, series_name=None):
        if name is None:
            name = self.getUniqueString(u"project-name")
        if series_name is None:
            series_name = self.getUniqueString(u"series-name")

        return getUtility(IProjectSet).create(name, series_name)

    def makeProjectSeries(self, name=None, project_name=None):
        project = self.makeProject(project_name, name)
        return project.development_focus
