# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "FakeApplicationRequest",
    "FakeHTTPConnection",
    "FakeLaunchpad",
    "FakeSocket",
    ]

from cStringIO import StringIO
from httplib import HTTPConnection

from django.conf import settings

from zope.interface import implements
from zope.publisher.interfaces.http import IHTTPApplicationRequest

from lazr.restful.interfaces import IWebServiceLayer

from lpresults.registry.interfaces.launchpad import ILaunchpad


class FakeApplicationRequest:
    """Simple request object for testing purpose."""

    implements(IHTTPApplicationRequest, IWebServiceLayer)


class FakeHTTPConnection(HTTPConnection):

    def __init__(self, sock):
        super(FakeHTTPConnection, self).__init__("localhost")
        # Set the socket to bypass the connection
        self.sock = sock

    def send(self, str):
        """Ignores the writes on the socket."""
        pass


class FakeLaunchpad:
    """Simple Launchpad object for testing purpose."""

    implements(ILaunchpad)

    service_root = settings.LAUNCHPAD_SERVICE_ROOT
    service_version = settings.LAUNCHPAD_SERVICE_VERSION

    def getPillar(self, pillar_name):
        pillar = {}
        service_url = self.service_root + self.service_version

        if pillar_name.startswith("distr"):
            resource_type = "distribution"
            pillar["current_series_link"] = "/".join([
                service_url, pillar_name, "dapper"])
        elif pillar_name.startswith("project"):
            resource_type = "project"
            pillar["development_focus_link"] = "/".join([
                service_url, pillar_name, "trunk"])
        else:
            return None

        pillar["resource_type_link"] = "/".join([
            service_url, "#%s" % resource_type])
        return pillar

    def getPillarSeries(self, pillar_name, series_name):

        if pillar_name.startswith("distr"):
            resource_type = "distro_series"
        elif pillar_name.startswith("project"):
            resource_type = "project_series"
        else:
            return None

        service_url = self.service_root + self.service_version
        return {
            "resource_type_link": "/".join([
                service_url, "#%s" % resource_type]),
            }


class FakeSocket:
    """A socket-like object that can be given a predefined content."""

    def __init__(self, data):
        self.readfile = StringIO(data)

    def makefile(self, mode='r', bufsize=None):
        return self.readfile
