# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Validator functions."""

__metaclass__ = type

__all__ = [
    "name_validator",
    "valid_name",
    ]

import re

from zope.schema.interfaces import ValidationError


VALID_DATETIME_PATTERN = r"\d{8}T\d{6}\.\d+"
VALID_DATETIME_RE = re.compile(r"^%s$" % VALID_DATETIME_PATTERN)

VALID_NAME_PATTERN = r"[a-z0-9][a-z0-9\+\.\-]+"
VALID_NAME_RE = re.compile(r"^%s$" % VALID_NAME_PATTERN)


def name_validator(name):
    """Return True if the name is valid, or raise a ValidationError.

    :param name: The name attribute.
    """
    if not valid_name(name):
        raise ValidationError("Invalid name '%s'" % name)

    return True


def valid_name(name):
    """Return True if the name is valid, otherwise False.

    :param name: The name attribute.
    """
    if VALID_NAME_RE.match(name):
        return True

    return False
