# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Context",
    ]

import re

from lpresults.repository.state import State


ROOT_STATE_RE = re.compile(r"([A-Z]+) (.*)")
CHILD_STATE_RE = re.compile(r"(\t*)(.*)")
LINE_SPLIT_RE = re.compile(r"(?:  |\t)")


class Context:

    state_factory = State

    def __init__(self, source, result, state_factory=None):
        self.source = source
        self.result = result

        if state_factory is not None:
            self.state_factory = state_factory

    def parseData(self, data):
        self.result.start()

        # Get default child
        root_state = self.state_factory()
        state = root_state.getChild()

        for line in data.split("\n"):
            # Skip comments
            if line.startswith("#") or line.startswith(";"):
                continue

            # Strip trailing white space
            line = line.rstrip()
            if not line:
                continue

            # Match root states
            match = ROOT_STATE_RE.match(line)
            if match:
                name = match.group(1)
                line = match.group(2)
                state = root_state.getChild(name)

            # Match child, parent or sibling states
            (spaces, line) = CHILD_STATE_RE.match(line).groups()
            depth = len(spaces) + 1
            if depth > state.depth:
                state = state.getChild()
            else:
                while depth < state.depth:
                    state = state.parent
                state = state.getSibling()

            # Split line as args
            try:
                (ids, name) = LINE_SPLIT_RE.split(line, 1)
            except ValueError:
                # Workaround for a badly formatted line
                (ids, name) = line.split(" ", 1)

            args = ids.split(" ") + [name]
            state.handleEntry(self.result, *args)

        self.result.stop()
