# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "create_from_url",
    ]

from lpresults.repository.result import Result
from lpresults.repository.source import Source
from lpresults.repository.state import State


class PciResult(Result):

    def addVendor(self, vendor):
        pass

    def addDevice(self, device):
        pass

    def addSubdevice(self, subdevice):
        pass

    def addClass(self, cls):
        pass

    def addSubclass(self, subcls):
        pass

    def addProgif(self, progif):
        pass


class PciState(State):

    __slots__ = ("id", "name",)

    def handleEntry(self, result, id, name):
        self.id = int(id, 16)
        self.name = name


class PciVendorState(PciState):

    __state_parent__ = PciState

    def handleEntry(self, result, *args):
        super(PciVendorState, self).handleEntry(result, *args)
        result.addVendor(self)


class PciDeviceState(PciState):

    __state_parent__ = PciVendorState

    @property
    def vendor(self):
        return self.parent

    def handleEntry(self, result, *args):
        super(PciDeviceState, self).handleEntry(result, *args)
        result.addDevice(self)


class PciSubdeviceState(PciState):

    __state_parent__ = PciDeviceState

    @property
    def device(self):
        return self.parent

    def handleEntry(self, result, vendor_id, device_id, name):
        self.vendor_id = int(vendor_id, 16)
        self.device_id = int(device_id, 16)
        self.name = name.decode("utf-8")

        result.addSubdevice(self)


class PciClassState(PciState):

    __state_name__ = "C"
    __state_parent__ = PciState

    def handleEntry(self, result, *args):
        super(PciClassState, self).handleEntry(result, *args)
        result.addClass(self)


class PciSubclassState(PciState):

    __state_parent__ = PciClassState

    @property
    def cls(self):
        return self.parent

    def handleEntry(self, result, *args):
        super(PciSubclassState, self).handleEntry(result, *args)
        result.addSubclass(self)


class PciProgifState(PciState):

    __state_parent__ = PciSubclassState

    @property
    def subcls(self):
        return self.parent

    def handleEntry(self, result, *args):
        super(PciProgifState, self).handleEntry(result, *args)
        result.addProgif(self)


class PciSource(Source):

    def getContext(self, result):
        return self.context_factory(self, result, PciState)


create_from_url = PciSource
