# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "create_from_url",
    ]

from lpresults.repository.result import Result
from lpresults.repository.source import Source
from lpresults.repository.state import State


class PnpResult(Result):

    def addVendor(self, vendor):
        pass

    def addDevice(self, device):
        pass


class PnpState(State):

    __slots__ = ("id", "name",)


class PnpVendorState(PnpState):

    __state_parent__ = PnpState

    def handleEntry(self, result, id, name):
        self.name = id[:3]
        self.id = pnp_vendor_to_int(self.name)
        result.addVendor(self)

        device = self.getChild()
        device.handleEntry(result, id, name)


class PnpDeviceState(PnpState):

    __state_parent__ = PnpVendorState

    @property
    def vendor(self):
        return self.parent

    def handleEntry(self, result, id, name):
        self.id = int(id[3:], 16)
        self.name = name

        result.addDevice(self)


class PnpSource(Source):

    def getContext(self, result):
        return self.context_factory(self, result, PnpState)


def pnp_vendor_to_int(vendor):
    i = 0
    for c in vendor:
        i <<= 5
        i += ord(c.upper()) - ord("A") + 1

    return i


def pnp_int_to_vendor(i):
    vendor = []
    while i:
        c = i & 0x1f
        vendor[:0] = [chr(c + ord("A") - 1)]
        i >>= 5

    return "".join(vendor)


create_from_url = PnpSource
