# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.repository.formats.pci import (
    PciResult,
    PciSource,
    )
from lpresults.repository.tests.test_source import TestSourceMixin


class FakePciResult(PciResult):

    def __init__(self):
        self.vendors = []
        self.devices = []
        self.subdevices = []
        self.classes = []
        self.subclasses = []
        self.progifs = []

    def addVendor(self, vendor):
        self.vendors.append(vendor.name)

    def addDevice(self, device):
        self.devices.append(device.name)

    def addSubdevice(self, subdevice):
        self.subdevices.append(subdevice.name)

    def addClass(self, cls):
        self.classes.append(cls.name)

    def addSubclass(self, subcls):
        self.subclasses.append(subcls.name)

    def addProgif(self, progif):
        self.progifs.append(progif.name)


class TestPciResult(TestCase):

    def setUp(self):
        source = PciSource("file:///path")
        self.result = FakePciResult()
        self.context = source.getContext(self.result)

    def test_vendor(self):
        self.context.parseData("8086  Intel Corporation")
        self.assertEquals(len(self.result.vendors), 1)
        self.assertEquals(self.result.vendors[0], "Intel Corporation")

        self.context.parseData("1166  Broadcom")
        self.assertEquals(len(self.result.vendors), 2)
        self.assertEquals(self.result.vendors[1], "Broadcom")

    def test_device(self):
        self.context.parseData("""
8086  Intel Corporation
\t0007  82379AB
""")
        self.assertEquals(len(self.result.devices), 1)
        self.assertEquals(self.result.devices[0], "82379AB")

    def test_subdevice(self):
        self.context.parseData("""
8086  Intel Corporation
\t0044  Core Processor DRAM Controller
\t\t1025 0347  Aspire 7740G
""")
        self.assertEquals(len(self.result.subdevices), 1)
        self.assertEquals(self.result.subdevices[0], "Aspire 7740G")

    def test_class(self):
        self.context.parseData("C 03  Display controller")
        self.assertEquals(len(self.result.classes), 1)
        self.assertEquals(self.result.classes[0], "Display controller")

    def test_subclass(self):
        self.context.parseData("""
C 03  Display controller
\t00  VGA compatible controller
""")
        self.assertEquals(len(self.result.subclasses), 1)
        self.assertEquals(
            self.result.subclasses[0], "VGA compatible controller")

    def test_progif(self):
        self.context.parseData("""
C 03  Display controller
\t00  VGA compatible controller
\t\t00  VGA controller
""")
        self.assertEquals(len(self.result.progifs), 1)
        self.assertEquals(self.result.progifs[0], "VGA controller")


class TestPciSource(TestCase, TestSourceMixin):

    def getSource(self):
        return PciSource("file:///path")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
