# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.repository.formats.pnp import (
    PnpResult,
    PnpSource,
    )
from lpresults.repository.tests.test_source import TestSourceMixin


class FakePnpResult(PnpResult):

    def __init__(self):
        self.vendors = []
        self.devices = []

    def addVendor(self, vendor):
        self.vendors.append(vendor.name)

    def addDevice(self, device):
        self.devices.append(device.name)


class TestPnpResult(TestCase):

    def setUp(self):
        source = PnpSource("file:///path")
        self.result = FakePnpResult()
        self.context = source.getContext(self.result)

    def test_vendor(self):
        self.context.parseData("ABP0174 AdvanSys ABP742 SCSI Controller")
        self.assertEquals(len(self.result.vendors), 1)
        self.assertEquals(self.result.vendors[0], "ABP")

    def test_device(self):
        self.context.parseData("ABP0174 AdvanSys ABP742 SCSI Controller")
        self.assertEquals(len(self.result.devices), 1)
        self.assertEquals(
            self.result.devices[0], "AdvanSys ABP742 SCSI Controller")


class TestPnpSource(TestCase, TestSourceMixin):

    def getSource(self):
        return PnpSource("file:///path")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
