# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.repository.formats.usb import (
    UsbResult,
    UsbSource,
    )
from lpresults.repository.tests.test_source import TestSourceMixin


class FakeUsbResult(UsbResult):

    def __init__(self):
        self.vendors = []
        self.devices = []
        self.classes = []
        self.subclasses = []
        self.progifs = []

    def addVendor(self, vendor):
        self.vendors.append(vendor.name)

    def addDevice(self, device):
        self.devices.append(device.name)

    def addClass(self, cls):
        self.classes.append(cls.name)

    def addSubclass(self, subcls):
        self.subclasses.append(subcls.name)

    def addProgif(self, progif):
        self.progifs.append(progif.name)


class TestUsbResult(TestCase):

    def setUp(self):
        source = UsbSource("file:///path")
        self.result = FakeUsbResult()
        self.context = source.getContext(self.result)

    def test_vendor(self):
        self.context.parseData("03f0  Hewlett-Packard")
        self.assertEquals(len(self.result.vendors), 1)
        self.assertEquals(self.result.vendors[0], "Hewlett-Packard")

    def test_device(self):
        self.context.parseData("""
03f0  Hewlett-Packard
\t0004  DeskJet 895c
""")
        self.assertEquals(len(self.result.devices), 1)
        self.assertEquals(self.result.devices[0], "DeskJet 895c")

    def test_class(self):
        self.context.parseData("C 02  Communications")
        self.assertEquals(len(self.result.classes), 1)
        self.assertEquals(self.result.classes[0], "Communications")

    def test_subclass(self):
        self.context.parseData("""
C 02  Communications
\t02  Abstract (modem)
""")
        self.assertEquals(len(self.result.subclasses), 1)
        self.assertEquals(self.result.subclasses[0], "Abstract (modem)")

    def test_progif(self):
        self.context.parseData("""
C 02  Communications
\t02  Abstract (modem)
\t\t00  None
""")
        self.assertEquals(len(self.result.progifs), 1)
        self.assertEquals(self.result.progifs[0], "None")


class TestUsbSource(TestCase, TestSourceMixin):

    def getSource(self):
        return UsbSource("file:///path")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
