# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "create_from_url",
    ]

from lpresults.repository.result import Result
from lpresults.repository.source import Source
from lpresults.repository.state import State


class UsbResult(Result):

    def addVendor(self, vendor):
        pass

    def addDevice(self, device):
        pass

    def addClass(self, cls):
        pass

    def addSubclass(self, subcls):
        pass

    def addProgif(self, progif):
        pass


class UsbState(State):

    __slots__ = ("id", "name",)

    def handleEntry(self, result, id, name):
        self.id = int(id, 16)
        self.name = name


class UsbVendorState(UsbState):

    __state_parent__ = UsbState

    def handleEntry(self, result, *args):
        super(UsbVendorState, self).handleEntry(result, *args)
        result.addVendor(self)


class UsbDeviceState(UsbState):

    __state_parent__ = UsbVendorState

    @property
    def vendor(self):
        return self.parent

    def handleEntry(self, result, *args):
        super(UsbDeviceState, self).handleEntry(result, *args)
        result.addDevice(self)


class UsbClassState(UsbState):

    __state_name__ = "C"
    __state_parent__ = UsbState

    def handleEntry(self, result, *args):
        super(UsbClassState, self).handleEntry(result, *args)
        result.addClass(self)


class UsbSubclassState(UsbState):

    __state_parent__ = UsbClassState

    @property
    def cls(self):
        return self.parent

    def handleEntry(self, result, *args):
        super(UsbSubclassState, self).handleEntry(result, *args)
        result.addSubclass(self)


class UsbProgifState(UsbState):

    __state_parent__ = UsbSubclassState

    @property
    def subcls(self):
        return self.parent

    def handleEntry(self, result, *args):
        super(UsbProgifState, self).handleEntry(result, *args)
        result.addProgif(self)


class UsbAudioTerminalState(UsbState):

    __state_name__ = "AT"
    __state_parent__ = UsbState


class UsbHidState(UsbState):

    __state_name__ = "HID"
    __state_parent__ = UsbState


class UsbHidItemState(UsbState):

    __state_name__ = "R"
    __state_parent__ = UsbState


class UsbBiasState(UsbState):

    __state_name__ = "BIAS"
    __state_parent__ = UsbState


class UsbPhysicalState(UsbState):

    __state_name__ = "PHY"
    __state_parent__ = UsbState


class UsbHidUsageTypeState(UsbState):

    __state_name__ = "HUT"
    __state_parent__ = UsbState


class UsbHidUsageSubtypeState(UsbState):

    __state_parent__ = UsbHidUsageTypeState


class UsbLanguageState(UsbState):

    __state_name__ = "L"
    __state_parent__ = UsbState


class UsbLanguageDialectState(UsbState):

    __state_parent__ = UsbLanguageState


class UsbHidCountryCodeState(UsbState):

    __state_name__ = "HCC"
    __state_parent__ = UsbState


class UsbVideoTerminalState(UsbState):

    __state_name__ = "VT"
    __state_parent__ = UsbState


class UsbSource(Source):

    def getContext(self, result):
        return self.context_factory(self, result, UsbState)


create_from_url = UsbSource
