# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Source",
    "create_source",
    "register_name",
    ]

from lpresults import repository
from lpresults.repository.context import Context
from lpresults.repository.exceptions import RepositoryError


class Source:
    """
    Base class for a source.
    """
    context_factory = Context

    def __init__(self, url):
        self.url = url

    def getContext(self, result):
        return self.context_factory(self, result)


_source_names = {}


def create_source(name, url):
    """Create a source instance.

    :param name: Source name.
    :param url: String describing the URL.
    """
    if name in _source_names:
        factory = _source_names[name]

    else:
        module_name = "%s.formats.%s" % (
            repository.__name__,
            name.replace("-", "_"))
        try:
            module = __import__(module_name, None, None, [""])
        except ImportError:
            raise RepositoryError("Format named '%s' not found" % name)
        factory = module.create_from_url

    return factory(url)


def register_name(name, factory):
    """Register a handler for a new source name.

    :param name: Source name
    :param factory: Function taking a URL string and returning a source.
    """
    _source_names[name] = factory
