# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestSourceMixin",
    ]

import new
import sys

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.repository.context import Context
from lpresults.repository.exceptions import RepositoryError
from lpresults.repository.source import (
    Source,
    create_source,
    register_name,
    _source_names,
    )


class FakeResult:

    pass


class TestCreateSource(TestCase):

    module_name = "lpresults.repository.formats.foo"

    def setUp(self):
        super(TestCreateSource, self).setUp()

        def create_from_url(url):
            return Source(url)

        format_module = new.module("format_module")
        format_module.create_from_url = create_from_url
        sys.modules[self.module_name] = format_module

    def tearDown(self):
        del sys.modules[self.module_name]
        super(TestCreateSource, self).tearDown()

    def test_create(self):
        source = create_source("foo", "file:///path")
        self.assertEquals(source.url, "file:///path")

    def test_notFound(self):
        self.assertRaises(
            RepositoryError, create_source, "bar", "file:///path")


class TestRegisterName(TestCase):

    def tearDown(self):
        if "factory" in _source_names:
            del _source_names["factory"]

        super(TestRegisterName, self).tearDown()

    def test_register(self):
        def factory(url):
            return Source(url)

        register_name("factory", factory)
        self.assertEqual(_source_names["factory"], factory)
        # Check that we can create sources that use this scheme ...
        source = create_source("factory", "file:///path")
        self.assertTrue(source)
        self.assertEqual(source.url, "file:///path")


class TestSourceMixin:

    def getSource(self):
        raise NotImplementedError()

    def test_init(self):
        source = self.getSource()
        self.assertTrue(isinstance(source, Source))

    def test_context(self):
        source = self.getSource()
        context = source.getContext(FakeResult())
        self.assertTrue(isinstance(context, Context))


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
