# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "OpenIDBackend",
    ]

from zope.component import getUtility

from django.core.handlers.wsgi import WSGIRequest as DjangoWSGIRequest

from lpresults.registry.interfaces.person import IPersonSet

from lpresults.session.interfaces import IOpenIDFactory


class OpenIDBackend:
    """A backend that authenticates a User based on an OpenID request."""

    def authenticate(self, **credentials):
        """Authenticate the user based on an OpenID request."""

        request = credentials.get("request")
        if request is None:
            return None

        if not isinstance(request, DjangoWSGIRequest):
            return None

        factory = getUtility(IOpenIDFactory)
        openid = factory.getOpenID(request)
        response = openid.complete()
        if response is not None:
            person_set = getUtility(IPersonSet)
            person = person_set.getByIdentity(response.identity_url)
            if person is None:
                person = person_set.create(
                    response.fields["nickname"], response.identity_url)

        else:
            person = None

        return person

    def get_user(self, user_id):
        return getUtility(IPersonSet).get(user_id)
