# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IOpenID",
    "IOpenIDFactory",
    ]

from zope.interface import (
    Interface,
    Attribute,
    )


class IOpenID(Interface):
    """
    An object that knows about the policy for doing OpenID authentication.
    """

    provider_url = Attribute("The provider URL when logging in")

    def begin(self, return_to_suffix):
        """
        Redirect the request to the OpenID provider so that he can
        authenticate.

        :param return_to_suffix: The suffix added to the root URL where
            to return.
        """

    def complete():
        """
        Given a request that contains a response from an OpenID provider (via
        the user agent), check if OpenID authentication succeeded.  If it has,
        an OpenID response will be returned.
        """


class IOpenIDFactory(Interface):
    """
    A factory for OpenID policies.
    """

    logout_url = Attribute("The URL to redirect to when logging out")

    def getOpenID(request, consumer_factory=None):
        """Return an object that implements L{IOpenID}."""
