# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = [
    "create",
    ]

from storm.zope.schema import ZSchema


def create():
    from lpresults.session import schema as patch_package

    return ZSchema(CREATES, DROPS, DELETES, patch_package)


CREATES = [
"""
GRANT SELECT
    ON TABLE patch
    TO lpresults
""",
"""
CREATE TABLE openidassociation (
    server_url BYTEA NOT NULL,
    handle BYTEA NOT NULL,
    secret BYTEA NOT NULL,
    issued INTEGER NOT NULL,
    lifetime INTEGER NOT NULL,
    association_type BYTEA NOT NULL,
    PRIMARY KEY (server_url, handle))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE openidassociation
    TO lpresults
""",
"""
CREATE TABLE openidnonce (
    server_url BYTEA NOT NULL,
    timestamp INTEGER NOT NULL,
    salt BYTEA NOT NULL,
    PRIMARY KEY (server_url, timestamp, salt))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE openidnonce
    TO lpresults
""",
"""
CREATE TABLE sessiondata (
    session_key BYTEA PRIMARY KEY,
    generation INTEGER NOT NULL DEFAULT 0,
    access_time TIMESTAMP WITHOUT TIME ZONE NOT NULL
            DEFAULT (CURRENT_TIMESTAMP AT TIME ZONE 'UTC'))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE sessiondata
    TO lpresults
""",
"""
CREATE TABLE sessionpkgdata (
    session_key BYTEA NOT NULL,
    generation INTEGER NOT NULL,
    key BYTEA NOT NULL,
    data BYTEA NOT NULL,
    PRIMARY KEY (session_key, generation, key))
""",
"""
GRANT SELECT, INSERT, UPDATE, DELETE
    ON TABLE sessionpkgdata
    TO lpresults
""",
"""
CREATE LANGUAGE plpgsql
""",
"""
CREATE FUNCTION get_generation(
    provided_session_key BYTEA, update_access_time INTERVAL,
    update_generation INTERVAL) RETURNS INTEGER
LANGUAGE plpgsql AS
$$
DECLARE
    session_record RECORD;
BEGIN
    -- Do we have a fresh generation which has a recent access time?
    SELECT INTO session_record generation FROM sessiondata
        WHERE sessiondata.session_key = provided_session_key AND
              sessiondata.access_time >
              now() AT TIME ZONE 'UTC' - update_access_time;

    IF FOUND THEN
        -- Fantastic!  That's the cheapest as no updating is necessary.
        RETURN session_record.generation;
    END IF;

    -- No luck.  Either we'll have to update an older one, or create it.
    -- For doing that we lock the row to prevent race conditions.
    SELECT INTO session_record generation, access_time FROM sessiondata
        WHERE sessiondata.session_key = provided_session_key FOR UPDATE;

    IF FOUND THEN

        -- Ok, there is an existing row.  Now we have to figure if the
        -- row has expired, or if it just needs to get a new access time.
        -- Note that for this test to work correctly, and thus the whole
        -- system, it's important that the isolation level be set to
        -- READ COMMITTED (or AUTOCOMMIT).  Otherwise, even though we've
        -- locked the row above, we're looking at old data, and given
        -- the right conditions, this could allow for two different
        -- generations to be used in parallel in a short interval of time.
        IF session_record.access_time <
           now() AT TIME ZONE 'UTC' - update_generation THEN

            -- Yeah, it's pretty old.  Let's bump the generation.
            UPDATE sessiondata SET generation=generation+1,
                                    access_time=now() AT TIME ZONE 'UTC'
                WHERE sessiondata.session_key = provided_session_key;

            RETURN session_record.generation+1;

        ELSE

            -- It's recent enough.  Just update the access time.
            UPDATE sessiondata SET access_time=now() AT TIME ZONE 'UTC'
                WHERE sessiondata.session_key = provided_session_key;

            RETURN session_record.generation;

        END IF;
    END IF;

    -- There's no row whatsoever.  We'll have to create it.
    BEGIN
        INSERT INTO sessiondata (session_key) VALUES (provided_session_key);
    EXCEPTION
        WHEN unique_violation THEN
            NULL; -- Someone else won the race.  We don't care.
    END;
    -- Whatever happened, it's the first generation.
    RETURN 0;
END;
$$
""",
"""
GRANT EXECUTE
    ON FUNCTION get_generation(BYTEA, INTERVAL, INTERVAL)
    TO lpresults
""",
]

DROPS = [
"DROP FUNCTION get_generation(BYTEA, INTERVAL, INTERVAL)",
"DROP LANGUAGE plpgsql",
"DROP TABLE sessionpkgdata",
"DROP TABLE sessiondata",
"DROP TABLE openidnonce",
"DROP TABLE openidassociation",
]

DELETES = [
"DELETE FROM sessionpkgdata",
"DELETE FROM sessiondata",
"DELETE FROM openidnonce",
"DELETE FROM openidassociation",
]
