# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""OpenID integration for Launchpad Results."""

from __future__ import absolute_import

__metaclass__ = type

__all__ = [
    "openid_factory",
    ]

from openid.consumer.consumer import (
    Consumer as DefaultConsumer,
    SUCCESS,
    )
from openid.extensions import pape, sreg

from django.conf import settings
from django.http import HttpResponseRedirect

from zope.interface import implements

from lpresults.session.interfaces import (
    IOpenID,
    IOpenIDFactory,
    )

from lpresults.session.openid import get_openid_store


class OpenIDResponse:

    def __init__(self, identity_url, fields={}):
        self.identity_url = identity_url
        self.fields = fields

    def __unicode__(self):
        return self.identity_url


class OpenID:

    implements(IOpenID)

    default_consumer_factory = DefaultConsumer

    provider_url = settings.OPENID_PROVIDER_URL

    def __init__(self, request, consumer_factory=None):
        if consumer_factory is None:
            consumer_factory = self.default_consumer_factory
        self.request = request
        session_data = request.session
        store = get_openid_store()
        self.consumer = consumer_factory(session_data, store)

    def begin(self, return_to_suffix):
        """See `IOpenID`."""
        auth = self.consumer.begin(self.provider_url)
        auth.addExtension(
            pape.Request(max_auth_age=30 * 60))
        auth.addExtension(
            sreg.SRegRequest(required=["nickname"]))

        root_url = self.request.build_absolute_uri("/").strip("/")
        url = auth.redirectURL(root_url, root_url + return_to_suffix)
        return HttpResponseRedirect(url)

    def complete(self):
        """See `IOpenID`."""
        current_url = self.request.build_absolute_uri()
        response = self.consumer.complete(
            dict(self.request.REQUEST.items()), current_url)

        if response.status == SUCCESS:
            identity_url = unicode(response.identity_url)
            sreg_response = sreg.SRegResponse.fromSuccessResponse(response)
            return OpenIDResponse(identity_url, sreg_response)
        else:
            return None


class OpenIDFactory:

    implements(IOpenIDFactory)

    logout_url = settings.OPENID_LOGOUT_URL

    getOpenID = OpenID


openid_factory = OpenIDFactory()
