# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from lpresults.storage.bucket import Bucket
from lpresults.storage.uri import URI

from lpresults.storage.backends.file import (
    FileBackend,
    FileConnection,
    )

from lpresults.storage.testing.cases import TestCaseWithFactory

from lpresults.storage.tests.test_backend import TestBackendMixin
from lpresults.storage.tests.test_connection import TestConnectionMixin
from lpresults.storage.tests.test_directory import TestDirectoryMixin


class TestFileBackend(TestCaseWithFactory, TestBackendMixin):

    def setUp(self):
        super(TestFileBackend, self).setUp()

        path = self.makeFile()
        self._uri = URI("file:%s" % path)

    def getBackend(self):
        return FileBackend(self._uri)


class TestFileConnection(TestCaseWithFactory, TestConnectionMixin):

    def setUp(self):
        super(TestFileConnection, self).setUp()

        self._path = self.makeFile()

    def getConnection(self):
        return FileConnection(self._path)


class TestFileDirectory(TestCaseWithFactory, TestDirectoryMixin):

    def setUp(self):
        super(TestFileDirectory, self).setUp()

        path = self.makeFile()
        uri = URI(path)
        backend = FileBackend(uri)
        self._bucket = Bucket(backend)

    def getDirectory(self, name):
        return self._bucket[name]


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
