# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Bucket",
    ]

from lpresults.storage.directory import Directory
from lpresults.storage.exceptions import ClientError
from lpresults.storage.validators import valid_name


class Bucket:
    """The Storage Bucket.

    This is the highest-level interface to a backend in order to manage
    directories.

    Note that Bucket objects are not threadsafe. You should create one
    Bucket per thread in your application, passing them the same backend
    object.
    """

    directory_factory = Directory

    def __init__(self, backend):
        self._backend = backend
        self._connection = backend.connect()

    def __getitem__(self, name):
        """Get a Directory from this Bucket."""
        if not valid_name(name):
            raise ClientError("Invalid directory name: '%s'" % name)

        return self.directory_factory(self, name)

    def __delitem__(self, name):
        """Delete a Directory from this Bucket."""
        if not valid_name(name):
            raise ClientError("Invalid directory name: '%s'" % name)

        return self._connection.deleteDirectory(name)

    def getBackend(self):
        """Return this Bucket's Backend object."""
        return self._backend
