# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "LRUCache",
    ]


class LRUCache(dict):

    def __init__(self, size=10):
        self._size = size
        self._order = []

    def __iter__(self):
        for key in self._order:
            yield key

    def __delitem__(self, key):
        super(LRUCache, self).__delitem__(key)
        self._order.remove(key)

    def __getitem__(self, key):
        value = super(LRUCache, self).__getitem__(key)
        self._order.remove(key)
        self._order.insert(0, key)

        return value

    def __setitem__(self, key, value):
        if self._size != 0:
            if key in self:
                self._order.remove(key)
            else:
                super(LRUCache, self).__setitem__(key, value)

            self._order.insert(0, key)
            if len(self) > self._size:
                del self[self._order[-1]]
