# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Connection",
    ]


class Connection:
    """
    Base class for a connection.
    """

    def __init__(self, backend):
        """Initialize a connection.

        :param backend: Backend instance to use.
        """
        self._backend = backend  # Ensures deallocation order.

    def deleteDirectory(self, directory_name):
        """Delete a Directory by name.

        :param directory_name: Name of the directory to delete.
        """
        for file_name in self.listDirectory(directory_name):
            self.deleteFile(directory_name, file_name)

    def listDirectory(self, directory_name):
        """List the file names in the given directory name.

        :param directory_name: Name of the directory to list.
        """
        raise NotImplementedError()

    def getInfo(self, directory_name, file_name):
        """Get information about a file from the given directory name.

        :param directory_name: Directory name where to get the information.
        :param file_name: Name of the file for the information.
        """
        raise NotImplementedError()

    def getContent(self, directory_name, file_name):
        """Get content by name from the given directory name.

        :param directory_name: Directory name where to get the file.
        :param file_name: Name of the file to get content.
        """
        raise NotImplementedError()

    def putContent(self, directory_name, file_name, file):
        """Put content by name into the given directory name.

        :param directory_name: Directory name where to put the file.
        :param file_name: Name of the file to put.
        :param file: File-like object to read content.
        """
        raise NotImplementedError()

    def deleteFile(self, directory_name, file_name):
        """Delete a file by name from the given directory name.

        :param directory_name: Directory name where to delete the file.
        :param file_name: Name of the file to delete.
        """
        raise NotImplementedError()
