# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

import settings

import martian

from zope.interface import Interface
from zope.schema import getFieldsInOrder

import grokcore.component
# Without this trickery, this module (called "django") will mask the
# real Django modules. We could use "from __future__ import
# absolute_import", but that would not work with Python 2.4.
ObjectDoesNotExist = __import__(
    "django.core.exceptions", {}).core.exceptions.ObjectDoesNotExist
ImproperlyConfigured = __import__(
    "django.core.exceptions", {}).core.exceptions.ImproperlyConfigured

from lpresults.storage.zope.interfaces import (
    IStorageConfiguration,
    IStorageRequest,
    )
from lpresults.storage.zope.views import NotFoundView
from lpresults.storage.zope.zstorage import (
    ZStorage,
    global_zstorage,
    )

from lpresults.storage.django.configuration import DjangoStorageConfiguration


class DjangoConfigurationGrokker(martian.ClassGrokker):
    """Pull storage configuration from the Django 'settings' module.

    Subclass DjangoStorageConfiguration to get a configuration class
    that can be overridden from your Django 'settings' module.
    """
    martian.component(DjangoStorageConfiguration)

    def execute(self, cls, config, *kw):
        for name, field in getFieldsInOrder(IStorageConfiguration):
            settings_name = "LPRESULTS_STORAGE_" + name.upper()
            value = getattr(settings, settings_name, None)
            if value is not None:
                setattr(cls, name, value)

        return True


class DjangoZStorageGrokker(martian.ClassGrokker):
    """Pull storage bucket information from the Django 'settings' module."""
    martian.component(ZStorage)

    def execute(self, cls, config, *kw):
        if not hasattr(settings, "LPRESULTS_STORAGE_BUCKETS"):
            raise ImproperlyConfigured(
                "You need to specify LPRESULTS_STORAGE_BUCKETS"
                "in your Django settings file.")

        for name, uri in settings.LPRESULTS_STORAGE_BUCKETS.iteritems():
            global_zstorage.setDefaultURI(name, uri)

        return True

# We want a raised django.core.exceptions.ObjectDoesNotExist
# object to result in a 404 status code. But we don't
# control that class definition, so we can't annotate it.
# Instead, we define an adapter from that exception class to the
# NotFoundView
grokcore.component.global_adapter(
    NotFoundView, (ObjectDoesNotExist, IStorageRequest),
    Interface, name="index.html")
