# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Info",
    ]

from mimetypes import guess_type


class Info:
    """
    Information about storage files.
    """
    __slots__ = ("name", "size", "date_modified", "content_type",)

    default_content_type = "binary/octet-stream"

    def __init__(self, name, size, date_modified, content_type=None):
        """Initialize a directory.

        :param name: Name of the file.
        :param size: Size of the file.
        :param date_modified: Date the file was last modified.
        :param content_type: Optional content type.
        """
        self.name = name
        self.size = size
        self.date_modified = date_modified

        if content_type is None:
            self.content_type = guess_type(name)[0] \
                or self.default_content_type

        else:
            self.content_type = content_type
