# Copyright 2009 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Byte storage for LAZR integration."""

__metaclass__ = type

__all__ = [
    "ByteStorage",
    ]

from lazr.restful.interfaces import IByteStorage

from zope.interface import implements
from zope.traversing.browser import absoluteURL

from lpresults.storage.zope.utils import get_current_browser_request


class ByteStorage:
    """See `IByteStorage`."""
    implements(IByteStorage)

    def __init__(self, entry, field):
        """Initialize as the backing storage for one entry's field."""
        self.entry = entry
        self.field = field
        self.file = getattr(self.entry, self.field.__name__)

    @property
    def alias_url(self):
        """See `IByteStorage`."""
        request = get_current_browser_request()
        return absoluteURL(self.file, request)

    @property
    def filename(self):
        """See `IByteStorage`."""
        if self.is_stored:
            return self.file.name
        return self.field.__name__

    @property
    def is_stored(self):
        """See `IByteStorage`."""
        return self.file is not None

    def createStored(self, mediaType, representation, filename=None):
        """See `IByteStorage`."""
        if filename is None:
            filename = self.filename

    def deleteStored(self):
        """See `IByteStorage`."""
        setattr(self.entry, self.field.__name__, None)
