# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestCase",
    "TestCaseWithFactory",
    ]

import os
import shutil
import tempfile
import unittest

from lpresults.storage.testing.factories import TestFactory


class TempfileTestCase(unittest.TestCase):

    def __init__(self, *args, **kwargs):
        super(TempfileTestCase, self).__init__(*args, **kwargs)

        self._tempfiles = []

    def makeFile(
        self, prefix="tmp", suffix="", basename=None,
        dirname=None, path=None, content=None):
        """Create a temporary file and return the path to it.

        The file is removed after the test runs.
        """
        if path is not None:
            self._tempfiles.append(path)
        elif basename is not None:
            if dirname is None:
                dirname = tempfile.mkdtemp()
                self._tempfiles.append(dirname)
            path = os.path.join(dirname, basename)
        else:
            fd, path = tempfile.mkstemp(suffix, prefix, dirname)
            self._tempfiles.append(path)
            os.close(fd)
            if content is None:
                os.unlink(path)

        if content is not None:
            file = open(path, "w")
            file.write(content)
            file.close()

        return path

    def makeDir(self, prefix="tmp", suffix="", dirname=None, path=None):
        """Create a temporary directory and return the path to it.

        The directory is removed after the test runs.
        """
        if path is not None:
            os.makedirs(path)
        else:
            path = tempfile.mkdtemp(suffix, prefix, dirname)

        self._tempfiles.append(path)
        return path

    def tearDown(self):
        super(TempfileTestCase, self).tearDown()

        for path in self._tempfiles:
            if os.path.isfile(path):
                os.unlink(path)
            elif os.path.isdir(path):
                shutil.rmtree(path)


class TestCase(TempfileTestCase):

    pass


class TestCaseWithFactory(TestCase):

    factory_class = TestFactory

    def setUp(self):
        super(TestCaseWithFactory, self).setUp()

        self.factory = self.factory_class()
