# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestFactory",
    ]

from cStringIO import StringIO

from zope.component import getUtility

from lpresults.testing.factories import ObjectFactory

from lpresults.storage.zope.interfaces import (
    IStorageConfiguration,
    IZStorage,
    )


class ConfigurationFactory(ObjectFactory):

    def updateConfiguration(self, **kwargs):
        configuration = getUtility(IStorageConfiguration)
        for key, value in kwargs.iteritems():
            setattr(configuration, key, value)


class TestFactory(ConfigurationFactory):

    default_bucket_name = "attachments"

    def getBucket(self, name=None):
        if name is None:
            name = self.default_bucket_name

        zstorage = getUtility(IZStorage)
        return zstorage.get(name)

    def getDirectory(self, name=None, bucket_name=None):
        if name is None:
            name = self.getUniqueString("directory-name")

        bucket = self.getBucket(bucket_name)
        return bucket[name]

    def getFile(self, name=None, directory_name=None, bucket_name=None):
        if name is None:
            name = self.getUniqueString("file-name")

        directory = self.getDirectory(directory_name, bucket_name)
        return directory.getFile(name)

    def putFile(
        self, name=None, content=None, directory_name=None, bucket_name=None):

        if name is None:
            name = self.getUniqueString("file-name")
        if content is None:
            content = self.getUniqueString("file-content")

        directory = self.getDirectory(directory_name, bucket_name)
        file = StringIO(content)
        return directory.putFile(name, file)
