# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestBackendMixin",
    ]

import new
import sys

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.storage.backend import (
    Backend,
    create_backend,
    register_scheme,
    _backend_schemes,
    )
from lpresults.storage.connection import Connection
from lpresults.storage.exceptions import StorageError
from lpresults.storage.uri import URI


class TestCreateBackend(TestCase):

    module_name = "lpresults.storage.backends.backend_module"

    def setUp(self):
        super(TestCreateBackend, self).setUp()

        def create_from_uri(uri):
            return uri

        self.backend_module = new.module("backend_module")
        self.backend_module.create_from_uri = create_from_uri
        sys.modules[self.module_name] = self.backend_module

    def tearDown(self):
        del sys.modules[self.module_name]
        super(TestCreateBackend, self).tearDown()

    def test_str(self):
        backend = create_backend("backend-module:/path")
        self.assertTrue(backend)
        self.assertEquals(backend.scheme, "backend-module")
        self.assertEquals(backend.path, "/path")

    def test_unicode(self):
        backend = create_backend(u"backend-module:/path")
        self.assertTrue(backend)
        self.assertEquals(backend.scheme, "backend-module")
        self.assertEquals(backend.path, "/path")

    def test_uri(self):
        uri = URI("backend-module:/path")
        backend = create_backend(uri)
        self.assertTrue(backend is uri)

    def test_notFound(self):
        self.assertRaises(
            StorageError, create_backend, "notfound-module:/path")


class TestRegisterScheme(TestCase):

    def tearDown(self):
        if "factory" in _backend_schemes:
            del _backend_schemes["factory"]

        super(TestRegisterScheme, self).tearDown()

    def test_register(self):
        def factory(uri):
            return uri

        register_scheme("factory", factory)
        self.assertEqual(_backend_schemes["factory"], factory)
        # Check that we can create backends that use this scheme ...
        backend = create_backend("factory:/path")
        self.assertTrue(backend)
        self.assertEqual(backend.scheme, "factory")
        self.assertEqual(backend.path, "/path")


class TestBackendMixin:

    def getBackend(self):
        raise NotImplementedError()

    def test_init(self):
        backend = self.getBackend()
        self.assertTrue(isinstance(backend, Backend))

    def test_connect(self):
        backend = self.getBackend()
        connection = backend.connect()
        self.assertTrue(isinstance(connection, Connection))


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
