# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.storage.cache import LRUCache


class TestLRUCache(TestCase):

    def test_del(self):
        cache = LRUCache()
        cache["a"] = 1
        self.assertEquals(len(cache), 1)
        self.assertEquals(list(cache), ["a"])

        del cache["a"]
        self.assertEquals(len(cache), 0)
        self.assertEquals(list(cache), [])

    def test_item(self):
        cache = LRUCache()
        cache["a"] = 1
        self.assertEquals(cache["a"], 1)

        cache["a"] = 1
        self.assertEquals(cache["a"], 1)

        cache["b"] = 2
        self.assertEquals(cache["b"], 2)
        self.assertEquals(cache["a"], 1)

    def test_iter(self):
        cache = LRUCache()
        self.assertEquals(list(cache), [])

        cache["a"] = 1
        self.assertEquals(list(cache), ["a"])

        cache["b"] = 2
        self.assertEquals(list(cache), ["b", "a"])

        cache["a"]
        self.assertEquals(list(cache), ["a", "b"])

    def test_len(self):
        cache = LRUCache()
        self.assertEquals(len(cache), 0)

        cache["a"] = 1
        self.assertEquals(len(cache), 1)

        cache["a"] = 1
        self.assertEquals(len(cache), 1)

        cache["b"] = 2
        self.assertEquals(len(cache), 2)

    def test_sizeZero(self):
        cache = LRUCache(0)
        cache["a"] = 1
        self.assertTrue("a" not in cache)

    def test_sizeOne(self):
        cache = LRUCache(1)
        cache["a"] = 1
        self.assertTrue("a" in cache)

        cache["b"] = 2
        self.assertTrue("b" in cache)
        self.assertTrue("a" not in cache)

        cache["a"] = 1
        self.assertTrue("a" in cache)
        self.assertTrue("b" not in cache)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
