# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestConnectionMixin",
    ]

from cStringIO import StringIO

from unittest import defaultTestLoader

from lpresults.storage.connection import Connection
from lpresults.storage.exceptions import ClientError

from lpresults.storage.testing.layers import FunctionalLayer


class TestConnectionMixin:

    layer = FunctionalLayer

    def getConnection(self):
        raise NotImplementedError()

    def test_init(self):
        connection = self.getConnection()
        self.assertTrue(isinstance(connection, Connection))

    def test_deleteDirectory(self):
        connection = self.getConnection()
        directory_name = self.factory.getUniqueString("directory-name")

        # Put file
        file_name = self.factory.getUniqueString("file-name")
        connection.putContent(directory_name, file_name, StringIO())
        directory_list = list(connection.listDirectory(directory_name))
        self.assertEquals(len(directory_list), 1)

        # Delete directory
        connection.deleteDirectory(directory_name)

        # List empty directory
        directory_list = list(connection.listDirectory(directory_name))
        self.assertEquals(directory_list, [])

    def test_listDirectory(self):
        connection = self.getConnection()
        directory_name = self.factory.getUniqueString("directory-name")

        # List empty directory
        directory_list = list(connection.listDirectory(directory_name))
        self.assertEquals(directory_list, [])

        # List directory with one file
        file_name = self.factory.getUniqueString("file-name")
        connection.putContent(directory_name, file_name, StringIO())
        directory_list = list(connection.listDirectory(directory_name))
        self.assertEquals(len(directory_list), 1)
        self.assertEquals(directory_list[0], file_name)

    def test_getNoFile(self):
        connection = self.getConnection()
        directory_name = self.factory.getUniqueString("directory-name")

        file_name = self.factory.getUniqueString("file-name")
        chunks = connection.getContent(directory_name, file_name)
        self.assertRaises(ClientError, list, chunks)

    def test_putAndGetFile(self):
        connection = self.getConnection()
        directory_name = self.factory.getUniqueString("directory-name")

        # Put file
        file_name = self.factory.getUniqueString("file-name")
        in_file = StringIO(self.factory.getUniqueString("file-content"))
        connection.putContent(directory_name, file_name, in_file)
        in_file.seek(0)

        # Get file
        out_file = StringIO()
        out_file.writelines(connection.getContent(directory_name, file_name))
        out_file.seek(0)

        # Check data
        self.assertEquals(in_file.read(), out_file.read())

    def test_deleteFile(self):
        connection = self.getConnection()
        directory_name = self.factory.getUniqueString("directory-name")

        # Put file
        file_name = self.factory.getUniqueString("file-name")
        connection.putContent(directory_name, file_name, StringIO())

        # List directory with one file
        directory_list = list(connection.listDirectory(directory_name))
        self.assertEquals(directory_list, [file_name])

        # Delete file
        connection.deleteFile(directory_name, file_name)

        # List empty directory
        directory_list = list(connection.listDirectory(directory_name))
        self.assertEquals(directory_list, [])


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
