# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestDirectoryMixin",
    ]

from cStringIO import StringIO

from types import GeneratorType

from unittest import defaultTestLoader

from lpresults.storage.directory import Directory
from lpresults.storage.exceptions import ClientError

from lpresults.storage.testing.layers import FunctionalLayer


class TestDirectoryMixin:

    layer = FunctionalLayer

    def getDirectory(self, name):
        raise NotImplementedError()

    def test_init(self):
        directory_name = self.factory.getUniqueString("directory-name")
        directory = self.getDirectory(directory_name)
        self.assertTrue(isinstance(directory, Directory))

    def test_iter(self):
        directory_name = self.factory.getUniqueString("directory-name")
        directory = self.getDirectory(directory_name)
        directory_iter = iter(directory)
        self.assertTrue(isinstance(directory_iter, GeneratorType))

    def test_name(self):
        directory_name = "foo"
        directory = self.getDirectory(directory_name)
        self.assertEquals(directory.name, directory_name)

    def test_invalidName(self):
        directory_name = "foo/bar"
        self.assertRaises(ClientError,
            self.getDirectory,
            directory_name)

    def test_putAndGetFile(self):
        directory_name = self.factory.getUniqueString("directory-name")
        directory = self.getDirectory(directory_name)

        # Put file
        file_name = self.factory.getUniqueString("file-name")
        in_file = StringIO(self.factory.getUniqueString("file-content"))
        directory.putFile(file_name, in_file)
        in_file.seek(0)

        # Get file
        out_file = directory.getFile(file_name)

        # Check data
        self.assertEquals(in_file.read(), out_file.read())

    def test_getNoFile(self):
        directory_name = self.factory.getUniqueString("directory-name")
        directory = self.getDirectory(directory_name)

        file = directory.getFile("foo")
        self.assertRaises(ClientError,
            file.read)

    def test_getInvalidFile(self):
        directory_name = self.factory.getUniqueString("directory-name")
        directory = self.getDirectory(directory_name)

        self.assertRaises(ClientError,
            directory.getFile,
            "foo/bar")

    def test_putInvalidFile(self):
        directory_name = self.factory.getUniqueString("directory-name")
        directory = self.getDirectory(directory_name)

        self.assertRaises(ClientError,
            directory.putFile,
            "foo/bar",
            StringIO())

    def test_deleteFile(self):
        directory_name = self.factory.getUniqueString("directory-name")
        directory = self.getDirectory(directory_name)

        # Put file
        file_name = self.factory.getUniqueString("file-name")
        directory.putFile(file_name, StringIO())

        # List directory with one file
        self.assertEquals([file.name for file in directory], [file_name])

        # Delete file
        directory.deleteFile(file_name)

        # List empty directory
        self.assertEquals(list(directory), [])


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
