# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.storage.uri import URI


class TestURI(TestCase):

    def test_initEmpty(self):
        uri = URI()
        self.assertEquals(uri.scheme, None)

    def test_initScheme(self):
        uri = URI("scheme:")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, None)
        self.assertEquals(uri.password, None)
        self.assertEquals(uri.host, None)
        self.assertEquals(uri.port, None)
        self.assertEquals(uri.path, None)
        self.assertEquals(uri.options, {})

    def test_initNoScheme(self):
        uri = URI("p%61th")
        self.assertEquals(uri.scheme, "file")
        self.assertEquals(uri.path, "path")

    def test_initPath(self):
        uri = URI("scheme:/p%61th")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.path, "/path")

    def test_initHost(self):
        uri = URI("scheme://ho%73t")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.host, "host")

    def test_initUsername(self):
        uri = URI("scheme://user%6eame@")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.host, None)

    def test_initUsernamePassword(self):
        uri = URI("scheme://user%6eame:pass%77ord@")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.password, "password")
        self.assertEquals(uri.host, None)

    def test_initUsernameHost(self):
        uri = URI("scheme://user%6eame@ho%73t")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.host, "host")

    def test_initUsernamePasswordHost(self):
        uri = URI("scheme://user%6eame:pass%77ord@ho%73t")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.password, "password")
        self.assertEquals(uri.host, "host")

    def test_initUsernamePasswordHostPort(self):
        uri = URI("scheme://user%6eame:pass%77ord@ho%73t:1234")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.password, "password")
        self.assertEquals(uri.host, "host")
        self.assertEquals(uri.port, 1234)

    def test_initUsernamePasswordHostNoPort(self):
        uri = URI("scheme://user%6eame:pass%77ord@ho%73t:")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.password, "password")
        self.assertEquals(uri.host, "host")
        self.assertEquals(uri.port, None)

    def test_initUsernamePasswordHostPortPath(self):
        uri = URI("scheme://user%6eame:pass%77ord@ho%73t:1234/p%61th")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.password, "password")
        self.assertEquals(uri.host, "host")
        self.assertEquals(uri.port, 1234)
        self.assertEquals(uri.path, "/path")

    def test_initUsernamePasswordPath(self):
        uri = URI("scheme://user%6eame:pass%77ord@/p%61th")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.username, "username")
        self.assertEquals(uri.password, "password")
        self.assertEquals(uri.host, None)
        self.assertEquals(uri.port, None)
        self.assertEquals(uri.path, "/path")

    def test_initOptions(self):
        uri = URI("scheme:?a%62c=d%65f&ghi=jkl")
        self.assertEquals(uri.scheme, "scheme")
        self.assertEquals(uri.host, None)
        self.assertEquals(uri.path, None)
        self.assertEquals(uri.options, {"abc": "def", "ghi": "jkl"})

    def test_copy(self):
        uri = URI("scheme:/path?opt=value")
        uri_copy = uri.copy()
        self.assertTrue(uri_copy is not uri)
        self.assertTrue(uri_copy.options is not uri.options)

    def assertStr(self, uri_str):
        self.assertEquals(str(URI(uri_str)), uri_str)

    def test_strScheme(self):
        self.assertStr("scheme:")

    def test_strUsername(self):
        self.assertStr("scheme://username@/")

    def test_strPassword(self):
        self.assertStr("scheme://:password@/")

    def test_strPort(self):
        self.assertStr("scheme://:0/")

    def test_strHost(self):
        self.assertStr("scheme://host/")

    def test_strPath(self):
        self.assertStr("scheme:/path")

    def test_strOptions(self):
        self.assertStr("scheme:?a=b")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
