# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "URI",
    ]

from urllib import (
    quote,
    unquote,
    )


class URI:

    scheme = None
    username = None
    password = None
    host = None
    port = None
    path = None

    def __init__(self, uri_str=""):
        rest = uri_str
        self.options = {}

        # Parse options
        if "?" in rest:
            rest, options = rest.split("?", 1)
            for pair in options.split("&"):
                key, value = pair.split("=", 1)
                self.options[unquote(key)] = unquote(value)

        # Parse scheme
        if ":" in rest:
            self.scheme, rest = rest.split(":", 1)
            if rest.startswith("//"):
                rest = rest[2:]
                # Parse path
                if "/" in rest:
                    index = rest.index("/")
                    self.path = unquote(rest[index:])
                    rest = rest[:index]

                # Parse username and password
                if "@" in rest:
                    userpass, hostport = rest.rsplit("@", 1)
                    if ":" in userpass:
                        username, password = userpass.split(":", 1)
                        self.username = unquote(username)
                        self.password = unquote(password)
                    else:
                        self.username = unquote(userpass)
                else:
                    hostport = rest

                # Parse port
                if ":" in hostport:
                    host, port = hostport.split(":", 1)
                    self.host = unquote(host)
                    if port:
                        self.port = int(port)
                elif hostport:
                    self.host = unquote(hostport)

            elif rest:
                self.path = unquote(rest)

        elif rest:
            self.scheme = "file"
            self.path = unquote(rest)

    def __str__(self):
        tokens = []
        append = tokens.append
        if self.scheme is None or self.scheme == "file":
            if self.path is not None:
                append(self.path)
        else:
            append(escape(self.scheme))
            append(":")
            if (self.username is not None or self.password is not None
                or self.host is not None or self.port is not None):
                append("//")
                if self.username is not None or self.password is not None:
                    if self.username is not None:
                        append(escape(self.username))
                    if self.password is not None:
                        append(":")
                        append(escape(self.password))
                    append("@")

                if self.host is not None:
                    append(escape(self.host))

                if self.port is not None:
                    append(":")
                    append(str(self.port))

            if self.path is not None:
                if not self.path.startswith("/"):
                    append("/")
                append(escape(self.path, "/+"))

            if self.options:
                options = ["%s=%s" % (escape(key), escape(value))
                           for key, value in sorted(self.options.iteritems())]
                append("?")
                append("&".join(options))

        return "".join(tokens)

    def copy(self):
        uri = object.__new__(self.__class__)
        uri.__dict__.update(self.__dict__)
        uri.options = self.options.copy()
        return uri


def escape(s, safe=""):
    return quote(s, safe)
