# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "BucketLocation",
    "DirectoryLocation",
    "FileLocation",
    "StorageURL",
    ]

from zope.component import getUtility
from zope.interface import implements
from zope.location.interfaces import ILocation
from zope.traversing.browser import AbsoluteURL
from zope.traversing.browser.interfaces import IAbsoluteURL

from lpresults.storage.zope.interfaces import (
    IStorageRootPage,
    IZStorage,
    )


class StorageURL(AbsoluteURL):
    """An IAbsoluteURL adapter for storage objects."""
    implements(IAbsoluteURL)

    def __init__(self, context, request):
        self.context = context
        self.request = request


class LocationMixin:
    """An ILocation mixin for storage objects."""
    implements(ILocation)

    def __init__(self, context):
        self.context = context

    @property
    def __name__(self):
        return self.context.name


class BucketLocation(LocationMixin):
    """An ILocation adapter for Bucket objects."""

    @property
    def __name__(self):
        return getUtility(IZStorage).getName(self.context)

    @property
    def __parent__(self):
        return getUtility(IStorageRootPage)


class DirectoryLocation(LocationMixin):
    """An ILocation adapter for Directory objects."""

    @property
    def __parent__(self):
        return self.context.bucket


class FileLocation(LocationMixin):
    """An ILocation adapter for File objects."""

    @property
    def __parent__(self):
        return self.context.directory
