# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import (
    getMultiAdapter,
    getUtility,
    )

from lpresults.testing.fakes import make_fake_request

from lpresults.storage.zope.interfaces import (
    IStoragePage,
    IStorageResult,
    IStorageRootPage,
    )

from lpresults.storage.testing.cases import TestCaseWithFactory
from lpresults.storage.testing.layers import FunctionalStorageLayer


class TestRootPage(TestCaseWithFactory):

    layer = FunctionalStorageLayer

    def test_call(self):
        make_fake_request()
        page = getUtility(IStorageRootPage)
        self.assertTrue("Launchpad Storage" in page())


class TestFilePage(TestCaseWithFactory):

    layer = FunctionalStorageLayer

    def test_call(self):
        context = self.factory.putFile()
        request = make_fake_request()
        page = getMultiAdapter((context, request), IStoragePage)
        self.assertEquals(page().name, context.name)


class TestResultPage(TestCaseWithFactory):

    layer = FunctionalStorageLayer

    def test_call(self):
        content = self.factory.getUniqueString("file-content")
        context = self.factory.putFile(content=content)
        request = make_fake_request()
        result = getMultiAdapter((context, request), IStorageResult)
        self.assertEquals(iter(result).next(), content)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
