# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import (
    provideUtility,
    queryUtility,
    )

from lpresults.storage.zope.interfaces import IStorageConfiguration
from lpresults.storage.zope.simple import StorageRootAbsoluteURL

from lpresults.storage.testing.cases import TestCaseWithFactory
from lpresults.storage.testing.fakes import FakeConfiguration


class TestStorageRootAbsoluteURL(TestCaseWithFactory):

    def setUp(self):
        super(TestStorageRootAbsoluteURL, self).setUp()

        self.old_configuration = queryUtility(IStorageConfiguration)

        configuration = FakeConfiguration()
        provideUtility(configuration)

    def tearDown(self):
        super(TestStorageRootAbsoluteURL, self).tearDown()

        if self.old_configuration:
            provideUtility(self.old_configuration)

    def test_hostname(self):
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "https://localhost")

        self.factory.updateConfiguration(hostname="foo.com")
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "https://foo.com")

    def test_port(self):
        self.factory.updateConfiguration(port=443)
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "https://localhost")

        self.factory.updateConfiguration(port=444)
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "https://localhost:444")

    def test_useHttps(self):
        self.factory.updateConfiguration(use_https=False)
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "http://localhost")

        self.factory.updateConfiguration(use_https=True)
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "https://localhost")

    def test_rootURIPrefix(self):
        self.factory.updateConfiguration(root_uri_prefix=None)
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "https://localhost")

        self.factory.updateConfiguration(root_uri_prefix="foo")
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(str(absolute_url), "https://localhost/foo")

    def test_call(self):
        absolute_url = StorageRootAbsoluteURL(None, None)
        self.assertEquals(absolute_url(), "https://localhost/")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
