# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "BucketTraverser",
    "DirectoryTraverser",
    "RootTraverser",
    ]

import urllib

from zope.component import getUtility
from zope.interface import implements
from zope.publisher.interfaces import NotFound

from lpresults.storage.zope.interfaces import (
    IStorageTraverser,
    IZStorage,
    )


class TraverserMixin:
    """See `IStorageTraverser`."""
    implements(IStorageTraverser)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        """See `IPublishTraverse`."""
        name = urllib.unquote(name)
        value = self.get(request, name)
        if value is None:
            raise NotFound(self, name)
        return value

    def get(self, request, name):
        """See `IStorageTraverser`."""
        raise NotImplementedError()


class RootTraverser(TraverserMixin):

    def get(self, request, name):
        return getUtility(IZStorage).get(name)


class BucketTraverser(TraverserMixin):

    def get(self, request, name):
        return self.context[name]


class DirectoryTraverser(TraverserMixin):

    def get(self, request, name):
        return self.context.getFile(name)
