# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "WSGIApplication",
    "WSGIStorageConfiguration",
    ]

from pkg_resources import resource_string

from zope.component import getUtility
from zope.configuration import xmlconfig
from zope.publisher.publish import publish

from lpresults.storage.zope.interfaces import (
    IStorageConfiguration,
    IStorageRootPage,
    )
from lpresults.storage.zope.simple import (
    StorageConfiguration,
    StoragePublication,
    StorageRequest,
    )


class WSGIApplication:

    request_class = StorageRequest
    publication_class = StoragePublication

    def __init__(self, environ, start_response):
        self.environ = environ
        self.start_response = start_response

    def __iter__(self):
        environ = self.environ
        service_root = getUtility(IStorageRootPage)
        request = self.request_class(environ["wsgi.input"], environ)
        request.setPublication(self.publication_class(service_root))

        # Support post-mortem debugging.
        handle_errors = environ.get("wsgi.handleErrors", True)
        # The request returned by the publisher may in fact be different than
        # the one passed in.
        request = publish(request, handle_errors=handle_errors)
        # Start the WSGI server response.
        response = request.response
        self.start_response(response.getStatusString(), response.getHeaders())
        # Return the result body iterable.
        return iter(response.consumeBodyIter())

    @classmethod
    def configure_server(cls, host, port, config_package,
                         config_file="site.zcml"):
        """Configure lpresults.storage for a particular storage service."""
        zcml = resource_string(config_package, config_file)
        xmlconfig.string(zcml)
        config = getUtility(IStorageConfiguration)
        config.hostname = host
        config.port = port


class WSGIStorageConfiguration(StorageConfiguration):
    """A basic storage configuration optimized for WSGI apps.

    There is no difference between this and the default
    StorageConfiguration. It's only here for convenience.
    """
