# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ObjectFactory",
    ]

from itertools import count
from random import randint
from threading import local


class ObjectFactory:

    def __init__(self):
        # Initialise the unique identifier.
        self._local = local()

    def getUniqueEmailAddress(self):
        return "%s@example.com" % self.getUniqueString("email")

    def getUniqueInteger(self):
        """Return an integer unique to this factory instance.

        For each thread, this will be a series of increasing numbers,
        but the starting point will be unique per thread.
        """
        counter = getattr(self._local, "integer", None)
        if counter is None:
            counter = count(randint(0, 1000000))
            self._local.integer = counter
        return counter.next()

    def getUniqueString(self, prefix=None):
        """Return a string unique to this factory instance.

        :param prefix: Used as a prefix for the unique string. If
            unspecified, defaults to "generic-string".
        """
        if prefix is None:
            prefix = "generic-string"
        string = "%s%s" % (prefix, self.getUniqueInteger())
        return string.replace("_", "-").lower()
