# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

# Enable debugging by default
DEBUG = True

# Database settings
DATABASE_ENGINE = "storm.django.backend"
DATABASE_NAME = "tracker"
DATABASE_SUPPORTS_TRANSACTIONS = True

STORM_SCHEMAS = {
    "session": "lpresults.session.schema",
    "test": "lpresults.tracker.schema.test",
    "tracker": "lpresults.tracker.schema.tracker",
    }

STORM_STORES = {
    "session": "postgres://lpresults:lpresults@localhost/lpresults-session",
    "session-autocommit":
        "postgres://lpresults:lpresults@localhost/lpresults-session"
        "?isolation=autocommit",
    "test-1": "postgres://lpresults:lpresults@localhost/lpresults-test-1",
    "tracker": "postgres://lpresults:lpresults@localhost/lpresults-tracker",
    }

STORM_ALIASES = {
    "hardware": "tracker",
    "registry": "tracker",
    }

# Authentication settings
AUTHENTICATION_BACKENDS = (
    "lpresults.session.auth.OpenIDBackend",
    "lpresults.tracker.auth.OAuthBackend",
    )

# Session settings
SESSION_ENGINE = "lpresults.session.backend"
SESSION_COOKIE_AGE = 365 * 24 * 60 * 60
SESSION_COOKIE_NAME = "lpresults"

SECRET_KEY = "No cookie forging please!"  # Shhh.. not public.

# Launchpad settings
LAUNCHPAD_HELP_ROOT = "https://help.launchpad.net/"
LAUNCHPAD_SERVICE_ROOT = "https://api.launchpad.net/"
LAUNCHPAD_SERVICE_VERSION = "1.0"
LAUNCHPAD_WEB_ROOT = "https://launchpad.net/"

# LAZR settings
LAZR_RESTFUL_CODE_REVISION = "1"
LAZR_RESTFUL_HOSTNAME = ""
LAZR_RESTFUL_PATH_OVERRIDE = "+api"
LAZR_RESTFUL_PORT = 0
LAZR_RESTFUL_SERVICE_ROOT_URI_PREFIX = "+api/"
LAZR_RESTFUL_SERVICE_VERSION_URI_PREFIX = "1.0"
LAZR_RESTFUL_USE_HTTPS = False

# GPG settings
LPRESULTS_GPG_HOST = "keyserver.ubuntu.com"
LPRESULTS_GPG_PORT = 11371

# Repository settings
LPRESULTS_REPOSITORY_SOURCES = {
    "pci": "http://pciids.sourceforge.net/pci.ids.bz2",
    "pnp":
        "http://www-pc.uni-regensburg.de/hardware/TECHNIK/PCI_PNP/pnpid.txt",
    "usb": "http://www.linux-usb.org/usb.ids",
    }

# Storage settings
LPRESULTS_STORAGE_BUCKETS = {
    "attachments": "attachments",
    }

LPRESULTS_STORAGE_HOSTNAME = ""
LPRESULTS_STORAGE_PATH_OVERRIDE = "+download"
LPRESULTS_STORAGE_PORT = 0
LPRESULTS_STORAGE_ROOT_URI_PREFIX = "+download/"
LPRESULTS_STORAGE_USE_HTTPS = False

# Tracker settings
LPRESULTS_TRACKER_LOGGING = ""
LPRESULTS_TRACKER_USE_HTTPS = False

# OpenID settings
OPENID_PROVIDER_URL = "https://login.launchpad.net/"
OPENID_LOGOUT_URL = "https://login.launchpad.net/+logout"

LOGIN_URL = "/+login"
COMPLETE_URL = "/+complete"
LOGOUT_URL = "/+logout"

# Application settings
INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "lpresults.tracker",
    )

MIDDLEWARE_CLASSES = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "storm.django.middleware.ZopeTransactionMiddleware",
    "lpresults.tracker.middleware.error.ErrorMiddleware",
    "lpresults.tracker.middleware.exception.LogExceptionMiddleware",
    )

APPEND_SLASH = False

ROOT_URLCONF = "lpresults.tracker.urls"

SETTINGS_MODULE = None

# Template and media settings
TEMPLATE_CONTEXT_PROCESSORS = (
    "lpresults.tracker.context_processors.auth",
    "lpresults.tracker.context_processors.settings",
    "lpresults.tracker.context_processors.target",
    "django.core.context_processors.i18n",
    )

TEMPLATE_LOADERS = (
    "django.template.loaders.filesystem.load_template_source",
    )

TEMPLATE_DIRS = (
    "templates",
    )

MEDIA_URL = "/+static/"

MEDIA_ROOT = "static"
