# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ITestCase",
    "IHasTestCases",
    "ITestCaseTarget",
    "ITestCaseSet",
    "TestCaseGetParams",
    "TestCaseSearchParams",
    ]

from zope.interface import Interface
from zope.schema import TextLine

from lazr.restful.declarations import (
    collection_default_content,
    export_as_webservice_collection,
    export_as_webservice_entry,
    exported,
    export_read_operation,
    operation_returns_collection_of,
    )
from lazr.restful.frameworks.django import IDjangoLocation
from lazr.restful.simple import IMultiplePathPartLocation

from lpresults.registry.interfaces.search import (
    ISearchSet,
    SearchParams,
    )

from lpresults.tracker.interfaces.testtarget import (
    IHasTestTarget,
    TestTargetParamsMixin,
    )


class ITestCase(IMultiplePathPartLocation, IHasTestTarget):

    export_as_webservice_entry()

    name = exported(
        TextLine(
            title=u"Name",
            description=u"The name of the test.",
            required=True,
            readonly=True))


class IHasTestCases(Interface):

    def searchTestCases(name=None):
        """Search the ITestCases reported on this entity."""


class ITestCaseTarget(IHasTestCases):

    def getTestCaseId(name):
        """Get a test case ID for this target."""


class ITestCaseSet(IDjangoLocation, ISearchSet):

    export_as_webservice_collection(ITestCase)

    def getId(get_params):
        """Get the ID for a TestCase within a target with the given name.

        If a TestCase with the given name does not exist within the
        target, it will be created and a new ID will be returned.

        :param get_params: A TestCaseGetParams object.
        """

    @collection_default_content()
    @operation_returns_collection_of(ITestCase)
    @export_read_operation()
    def getAll():
        """Return the default set of test cases without filtering."""


class TestCaseGetParams(TestTargetParamsMixin):
    """Encapsulates search parameters for ITestCaseSet.getId()"""

    def __init__(self, name):
        self.name = name


class TestCaseSearchParams(SearchParams, TestTargetParamsMixin):
    """Encapsulates search parameters for ITestCaseSet.search()"""

    def __init__(self, name=None, **kwargs):
        super(TestCaseSearchParams, self).__init__(**kwargs)
        self.name = name
