# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ITestDistribution",
    "ITestDistributionSet",
    "TestDistributionSearchParams",
    ]

from lazr.restful.declarations import (
    collection_default_content,
    export_as_webservice_collection,
    export_as_webservice_entry,
    export_read_operation,
    operation_returns_collection_of,
    )

from lpresults.registry.interfaces.person import PersonParamsMixin
from lpresults.registry.interfaces.distribution import (
    IDistribution,
    IDistributionSet,
    DistributionSearchParams,
    )

from lpresults.tracker.interfaces.testcase import ITestCaseTarget
from lpresults.tracker.interfaces.testdistroseries import ITestDistroSeries
from lpresults.tracker.interfaces.testrank import ITestRankTarget
from lpresults.tracker.interfaces.testrun import ITestRunTarget
from lpresults.tracker.interfaces.teststat import (
    IHasTestStatDistributions,
    ITestStatTarget,
    )
from lpresults.tracker.interfaces.testtarget import TestTargetParamsMixin
from lpresults.tracker.services.patchers import (
    patch_collection_return_type,
    patch_entry_return_type,
    )


class ITestDistribution(
    IDistribution, ITestCaseTarget, ITestRunTarget, ITestRankTarget,
    ITestStatTarget):
    """A test distribution.

    This extends the concept of a distribution with test information.
    """
    export_as_webservice_entry(singular_name="distribution")


class ITestDistributionSet(IDistributionSet, IHasTestStatDistributions):

    export_as_webservice_collection(ITestDistribution)

    @collection_default_content()
    @operation_returns_collection_of(ITestDistribution)
    @export_read_operation()
    def search():
        """Return the default set of distributions without filtering."""


class TestDistributionSearchParams(
    DistributionSearchParams, PersonParamsMixin, TestTargetParamsMixin):
    """Encapsulates search parameters for ITestDistributionSet.search()"""

    def __init__(self, id=None, date_modified=None, **kwargs):
        super(TestDistributionSearchParams, self).__init__(**kwargs)
        self.id = id
        self.date_modified = date_modified


ITestDistribution["current_series"].schema = ITestDistroSeries
ITestDistribution["series"].value_type.schema = ITestDistroSeries
ITestDistroSeries["distribution"].schema = ITestDistribution

patch_entry_return_type(
    ITestDistribution, "retrieveSeries", ITestDistroSeries)
patch_collection_return_type(
    IHasTestStatDistributions, "getLatestTestDistributions",
    ITestDistribution)
patch_collection_return_type(
    IHasTestStatDistributions, "getTopTestDistributions", ITestDistribution)
