# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ITestPerson",
    "ITestPersonSet",
    "TestPersonSearchParams",
    ]

from lazr.restful.declarations import (
    collection_default_content,
    export_as_webservice_collection,
    export_as_webservice_entry,
    export_read_operation,
    operation_returns_collection_of,
    )

from lpresults.registry.interfaces.person import (
    IPerson,
    IPersonSet,
    PersonSearchParams,
    )

from lpresults.tracker.interfaces.teststat import (
    IHasTestStatPeople,
    ITestStatPerson,
    )
from lpresults.tracker.interfaces.testtarget import TestTargetParamsMixin
from lpresults.tracker.services.patchers import patch_collection_return_type


class ITestPerson(IPerson, ITestStatPerson):
    """A test person.

    This extends the concept of a person with test information.
    """
    export_as_webservice_entry(singular_name="person", plural_name="people")


class ITestPersonSet(IPersonSet, IHasTestStatPeople):

    export_as_webservice_collection(ITestPerson)

    @collection_default_content()
    @operation_returns_collection_of(ITestPerson)
    @export_read_operation()
    def search():
        """Return the default set of people without filtering."""


class TestPersonSearchParams(PersonSearchParams, TestTargetParamsMixin):
    """Encapsulates search parameters for ITestPersonSet.search()"""

    def __init__(self, id=None, date_modified=None, **kwargs):
        super(TestPersonSearchParams, self).__init__(**kwargs)
        self.id = id
        self.date_modified = date_modified


patch_collection_return_type(
    IHasTestStatPeople, "getLatestTesters", ITestPerson)
patch_collection_return_type(
    IHasTestStatPeople, "getTopTesters", ITestPerson)
