# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ITestPillar",
    "ITestPillarSet",
    "TestPillarSearchParams",
    ]

from zope.schema import TextLine

from lazr.restful.declarations import (
    export_as_webservice_entry,
    export_operation_as,
    export_read_operation,
    operation_parameters,
    operation_returns_collection_of,
    )

from lpresults.registry.interfaces.person import PersonParamsMixin
from lpresults.registry.interfaces.pillar import (
    IPillar,
    IPillarSet,
    PillarSearchParams,
    )

from lpresults.tracker.interfaces.teststat import ITestStatTarget
from lpresults.tracker.interfaces.testtarget import TestTargetParamsMixin


class ITestPillar(IPillar, ITestStatTarget):
    """A test pillar.

    This extends the concept of a pillar with test information.
    """
    export_as_webservice_entry(singular_name="pillar")


class ITestPillarSet(IPillarSet):

    export_as_webservice_entry("pillars")

    @operation_parameters(
        text=TextLine(
            title=u"Search text"),
        )
    @operation_returns_collection_of(ITestPillar)
    @export_operation_as("search")
    @export_read_operation()
    def searchPillars(text):
        """Return Projects/Distributions matching :text:."""


class TestPillarSearchParams(
    PillarSearchParams, PersonParamsMixin, TestTargetParamsMixin):
    """Encapsulates search parameters for ITestPillarSet.search()"""

    def __init__(self, id=None, date_modified=None, **kwargs):
        super(TestPillarSearchParams, self).__init__(**kwargs)
        self.id = id
        self.date_modified = date_modified
