# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ITestProject",
    "ITestProjectSet",
    "TestProjectSearchParams",
    ]

from lazr.restful.declarations import (
    collection_default_content,
    export_as_webservice_collection,
    export_as_webservice_entry,
    export_read_operation,
    operation_returns_collection_of,
    )

from lpresults.registry.interfaces.person import PersonParamsMixin
from lpresults.registry.interfaces.project import (
    IProject,
    IProjectSet,
    ProjectSearchParams,
    )

from lpresults.tracker.interfaces.testcase import ITestCaseTarget
from lpresults.tracker.interfaces.testprojectseries import ITestProjectSeries
from lpresults.tracker.interfaces.testrank import ITestRankTarget
from lpresults.tracker.interfaces.testrun import ITestRunTarget
from lpresults.tracker.interfaces.teststat import (
    IHasTestStatProjects,
    ITestStatTarget,
    )
from lpresults.tracker.interfaces.testtarget import TestTargetParamsMixin
from lpresults.tracker.services.patchers import (
    patch_collection_return_type,
    patch_entry_return_type,
    )


class ITestProject(
    IProject, ITestCaseTarget, ITestRunTarget, ITestRankTarget,
    ITestStatTarget):
    """A test project.

    This extends the concept of a project with test information.
    """
    export_as_webservice_entry(singular_name="project")


class ITestProjectSet(IProjectSet, IHasTestStatProjects):

    export_as_webservice_collection(ITestProject)

    @collection_default_content()
    @operation_returns_collection_of(ITestProject)
    @export_read_operation()
    def search():
        """Return the default set of projects without filtering."""


class TestProjectSearchParams(
    ProjectSearchParams, PersonParamsMixin, TestTargetParamsMixin):
    """Encapsulates search parameters for ITestProjectSet.search()"""

    def __init__(self, id=None, date_modified=None, **kwargs):
        super(TestProjectSearchParams, self).__init__(**kwargs)
        self.id = id
        self.date_modified = date_modified


ITestProject["development_focus"].schema = ITestProjectSeries
ITestProject["series"].value_type.schema = ITestProjectSeries
ITestProjectSeries["project"].schema = ITestProject

patch_entry_return_type(
    ITestProject, "retrieveSeries", ITestProjectSeries)
patch_collection_return_type(
    IHasTestStatProjects, "getLatestTestProjects", ITestProject)
patch_collection_return_type(
    IHasTestStatProjects, "getTopTestProjects", ITestProject)
