# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "IHasTestRanks",
    "IHasTestRankSystems",
    "ITestRank",
    "ITestRankSystem",
    "TestRankSetParams",
    ]

from zope.interface import Interface
from zope.schema import Int

from lazr.restful.declarations import (
    call_with,
    export_read_operation,
    exported,
    operation_returns_collection_of,
    )

from lpresults.tracker.interfaces.teststat import (
    IHasTestStats,
    IHasTestStatPeople,
    ITestStat,
    )
from lpresults.tracker.interfaces.testtarget import TestTargetParamsMixin


class ITestRank(ITestStat):
    """Test rank."""

    rank = exported(
        Int(
            title=u"Rank",
            required=True,
            readonly=True,
            description=u"The rank for the testing that happened"))


class IHasTestRanks(IHasTestStats):
    """An entity which has general test ranks."""


class IHasTestRankSystems(Interface):
    """An entity which has test ranks on systems."""

    @operation_returns_collection_of(Interface)  # ITestSystem
    @call_with(limit=5)
    @export_read_operation()
    def getLatestTestSystems(limit=None):
        """Get the latest systems tested.

        :param limit: Optional maximum number of items to return.
        """

    @operation_returns_collection_of(Interface)  # ITestSystem
    @call_with(limit=5)
    @export_read_operation()
    def getTopTestSystems(limit=None):
        """Get the top systems which have been tested the most.

        :param limit: Optional maximum number of items to return.
        """


class ITestRankSystem(IHasTestRanks, IHasTestStatPeople):
    """A system for which ranking can be determined."""


class ITestRankTarget(IHasTestRanks):
    """A target on which test ranks can be set.

    Examples include an ITestDistroSeries and ITestProjectSeries for now.
    """
    def setTestRank(person, system_unit, rank):
        """Set the test rank on this target.

        :param person: An `IPerson` for whom to set the rank.
        :param system_unit: An `ISystemUnit` for whom to set the rank.
        :param rank: The rank value.
        """


class TestRankSetParams(TestTargetParamsMixin):
    """Encapsulates parameters for ITestRankTarget.setTestRank()"""

    def __init__(self, person, system_unit, rank):
        self.person = person
        self.system_unit = system_unit
        self.rank = rank
