# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ITestSystem",
    "ITestSystemSet",
    "TestSystemSearchParams",
    ]

from zope.schema import TextLine

from lazr.restful.declarations import (
    export_as_webservice_entry,
    export_operation_as,
    export_read_operation,
    operation_parameters,
    operation_returns_collection_of,
    )

from lpresults.hardware.interfaces.system import (
    ISystem,
    ISystemSet,
    SystemSearchParams,
    )

from lpresults.tracker.interfaces.testrank import (
    IHasTestRankSystems,
    ITestRankSystem,
    )
from lpresults.tracker.interfaces.testsystemunit import ITestSystemUnit
from lpresults.tracker.interfaces.testtarget import TestTargetParamsMixin
from lpresults.tracker.services.patchers import patch_collection_return_type


class ITestSystem(ISystem, ITestRankSystem):
    """A test system.

    This extends the concept of a system with test information.
    """
    export_as_webservice_entry(singular_name="system")


class ITestSystemSet(ISystemSet, IHasTestRankSystems):

    export_as_webservice_entry("systems")

    @operation_parameters(
        text=TextLine(
        required=False,
        title=u"Search text"),
        )
    @operation_returns_collection_of(ITestSystem)
    @export_operation_as("search")
    @export_read_operation()
    def searchSystems(text=None):
        """Return the default set of systems without filtering."""


class TestSystemSearchParams(SystemSearchParams, TestTargetParamsMixin):
    """Encapsulates search parameters for ITestSystemSet.search()"""

    def __init__(self, id=None, date_modified=None, **kwargs):
        super(TestSystemSearchParams, self).__init__(**kwargs)
        self.id = id
        self.date_modified = date_modified


ITestSystem["units"].value_type.schema = ITestSystemUnit
ITestSystemUnit["system"].schema = ITestSystem

patch_collection_return_type(
    IHasTestRankSystems, "getLatestTestSystems", ITestSystem)
patch_collection_return_type(
    IHasTestRankSystems, "getTopTestSystems", ITestSystem)
