# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "ITestTarget",
    "IHasTestTarget",
    "TestTargetParamsMixin",
    ]

from zope.interface import Attribute
from zope.schema import Object

from lpresults.registry.interfaces.distroseries import IHasDistroSeries
from lpresults.registry.interfaces.projectseries import IHasProjectSeries

from lpresults.tracker.interfaces.teststore import IHasTestStore


class ITestTarget(IHasTestStore):
    """An entity on which a tests can be performed.

    Examples include an ITestDistroSeries and ITestProjectSeries for now.
    """
    name = Attribute("Name given to the target")


class IHasTestTarget(IHasDistroSeries, IHasProjectSeries):

    target = Object(
        schema=ITestTarget,
        title=u"Target",
        required=True,
        readonly=True)


class TestTargetParamsMixin:
    """Encapsulates test target parameters."""

    distro_series = None
    project_series = None

    @property
    def target(self):
        if self.distro_series:
            return self.distro_series
        elif self.project_series:
            return self.project_series
        else:
            raise AssertionError("Unable to determine test params target.")

    def setTarget(self, distro_series=None, project_series=None):
        if distro_series:
            self.distro_series = distro_series
        elif project_series:
            self.project_series = project_series
