# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "run",
    ]

from django.core import management
from django.core.management import find_management_module \
    as django_find_management_module

from lpresults.tracker import settings


def find_management_module(app_name):
    try:
        return django_find_management_module(app_name)
    except ImportError:
        pass

    location = ".".join([app_name, "management"])
    # Import, swallowing all errors as any element of location may be
    # a class or some such thing.
    try:
        __import__(location, {}, {})
    except ImportError:
        pass

    components = location.split('.')
    current = __import__(components[0], {}, {})
    for component in components[1:]:
        try:
            current = getattr(current, component)
        except AttributeError:
            raise ImportError("No module named %s" % component)

    return current.__path__[0]


def run(argv=None):
    management.find_management_module = find_management_module
    management.execute_manager(settings, argv)


if __name__ == "__main__":
    run()
