# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Attachment",
    "HasAttachmentsMixin",
    "AttachmentTargetMixin",
    "attachment_set",
    ]

from zope.component import getUtility
from zope.interface import implements

from lazr.restful.simple import TraverseWithGet
from lazr.restful.utils import get_current_browser_request

from lpresults.tracker.interfaces.attachment import (
    IAttachment,
    IAttachmentSet,
    IAttachmentTarget,
    IHasAttachments,
    )
from lpresults.tracker.models.bucket import get_bucket
from lpresults.tracker.models.file import (
    File,
    FileSet,
    )


class Attachment(File):

    implements(IAttachment)

    @property
    def __parent__(self):
        return self.target

    @property
    def __path_parts__(self):
        return ("attachments", self.data.name,)


class AttachmentSet(FileSet, TraverseWithGet):

    implements(IAttachmentSet)

    __url_path__ = "attachments"

    @property
    def __parent__(self):
        request = get_current_browser_request()
        return request.target

    def get(self, request, name):
        """See `ITraverseWithGet`."""
        return super(AttachmentSet, self).get(request.target, name)

    def getAll(self):
        """See `ITestCaseSet`."""
        request = get_current_browser_request()
        return request.target.attachments

    def getFileResultDecorator(self, result):
        """Get the decorator for a list."""
        return Attachment(*result)

    def getFileBucket(self, target):
        """See `IAttachmentSet`."""
        return get_attachment_bucket()


class HasAttachmentsMixin(TraverseWithGet):

    implements(IHasAttachments)

    @property
    def attachments(self):
        """See `IHasAttachments`."""
        return getUtility(IAttachmentSet).list(self)

    def get(self, request, name):
        """See `ITraverseWithGet`."""
        request.target = self
        if name == "attachments":
            return getUtility(IAttachmentSet)

        return None


class AttachmentTargetMixin(HasAttachmentsMixin):

    implements(IAttachmentTarget)

    def addAttachment(self, person, filename, data):
        """See `IAttachmentTarget`."""
        return getUtility(IAttachmentSet).put(self, filename, data)


def get_attachment_bucket():
    return get_bucket("attachments")


attachment_set = AttachmentSet()
