# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "File",
    "FileSet",
    ]

from cStringIO import StringIO


class File:

    def __init__(self, data, target):
        self.data = data
        self.target = target


class FileSet:

    def get(self, target, name):
        """See `IFileSet`."""
        directory = self.getFileDirectory(target)
        data = directory.getFile(name)
        return self.getFileResultDecorator((data, target))

    def put(self, target, name, data):
        """See `IFileSet`."""
        if isinstance(data, basestring):
            data = StringIO(data)

        directory = self.getFileDirectory(target)
        directory.putFile(name, data)

    def list(self, target):
        """See `IFileSet`."""
        directory = self.getFileDirectory(target)
        results = [(file, target)
            for file in sorted(directory, key=lambda f: f.date_modified)]
        return self.buildFileResultDecorator(results)

    def buildFileResultDecorator(self, results):
        """Build and return the decorator for a list."""
        if results:
            decorator = self.getFileResultDecorator
            results = [decorator(result) for result in results]

        return results

    def getFileResultDecorator(self, result):
        """Get the decorator for a list."""
        return File(*result)

    def getFileDirectory(self, target):
        bucket = self.getFileBucket(target)
        return bucket[target.name]

    def getFileBucket(self, target):
        """Get the file bucket."""
        raise NotImplementedError()
