# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestPerson",
    "test_person_set",
    ]

from storm.expr import (
    Alias,
    LeftJoin,
    Max,
    Sum,
    )

from zope.interface import implements

from lazr.delegates import delegates

from lpresults.database.datetime import UTCDateTimeAlias
from lpresults.database.search import search_to_expression

from lpresults.registry.formatters import item_to_tuple
from lpresults.registry.interfaces.person import IPerson
from lpresults.registry.models.person import (
    Person,
    PersonSet,
    )
from lpresults.registry.models.personhelper import PersonTwin

from lpresults.tracker.interfaces.testperson import (
    ITestPerson,
    ITestPersonSet,
    TestPersonSearchParams,
    )
from lpresults.tracker.models.teststat import (
    HasTestStatPeopleMixin,
    TestStat,
    TestStatPerson,
    )
from lpresults.tracker.models.teststore import get_test_stores


class TestPerson(TestStatPerson):
    """See `ITestPerson`."""
    implements(ITestPerson)
    delegates(IPerson)


class TestPersonSet(PersonSet, HasTestStatPeopleMixin):
    """See `ITestPersonSet`."""
    implements(ITestPersonSet)

    search_params_factory = TestPersonSearchParams

    def get(self, id):
        """See `IPersonSet`."""
        search_params = self.search_params_factory(id=id)
        return self.search(search_params).one()

    def buildSearchExpressions(self, search_params):
        """See SearchSet."""
        expressions = super(TestPersonSet, self).buildSearchExpressions(
            search_params)

        if search_params.id is not None:
            expressions.append(
                search_to_expression(Person.id, search_params.id))

        if search_params.date_modified:
            expressions.append(
                search_to_expression(
                    TestStat.date_modified,
                    search_params.date_modified))

        if search_params.distro_series:
            expressions.append(
                search_to_expression(
                    TestStat.distro_series_id,
                    search_params.distro_series,
                    modifier=lambda x: x.id))

        if search_params.project_series:
            expressions.append(
                search_to_expression(
                    TestStat.project_series_id,
                    search_params.project_series,
                    modifier=lambda x: x.id))

        return expressions

    def buildSearchResult(self, search_params):
        """See SearchSet."""
        result = super(TestPersonSet, self).buildSearchResult(search_params)

        return result.group_by(Person)

    def buildSearchTables(self, search_params):
        """See SearchSet."""
        tables = super(TestPersonSet, self).buildSearchTables(search_params)

        tables.append(
            LeftJoin(TestStat,
                TestStat.person_id == Person.id))

        return tables

    def getSearchResultDecorator(self, result):
        """See SearchSet."""
        return TestPerson(*result)

    def getSearchResultTables(self):
        """See SearchSet."""
        result_tables = super(TestPersonSet, self).getSearchResultTables()

        return item_to_tuple(result_tables) + (
            Alias(Sum(TestStat.counter), name="test_run_count"),
            UTCDateTimeAlias(
                Max(TestStat.date_modified), name="last_test_run_date"),
            )


def person_created(person, event):
    # Flush each store to ensure correct ordering of
    # SQL statements emitted by storm.
    for name, test_store in get_test_stores():
        person_twin = PersonTwin(person)
        test_store.add(person_twin)
        test_store.flush()

    return person


test_person_set = TestPersonSet()
