# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "HasTestRankSystemsMixin",
    "TestRank",
    ]

from storm.locals import (
    AutoReload,
    Int,
    )

from zope.component import getUtility
from zope.interface import implements

from lazr.delegates import delegates

from lpresults.database.search import SearchNotEqual

from lpresults.hardware.models.systemunithelper import SystemUnitHelper

from lpresults.tracker.interfaces.testrank import (
    ITestRank,
    TestRankSetParams,
    )
from lpresults.tracker.interfaces.teststat import ITestStat
from lpresults.tracker.interfaces.testsystem import (
    ITestSystemSet,
    TestSystemSearchParams,
    )
from lpresults.tracker.models.store import get_tracker_store


class TestRank(SystemUnitHelper):

    implements(ITestRank)
    delegates(ITestStat, context="test_stat")

    __storm_table__ = "testrank"

    id = Int(primary=True, allow_none=False, default=AutoReload)
    test_stat_id = Int(name="test_stat")
    system_unit_id = Int(name="system_unit")
    rank = Int(allow_none=False)


class HasTestRankSystemsMixin:

    def getLatestTestSystems(self, limit=None):
        """See `IHasTestStatPeople`."""
        return self.getTestSystems("-last_test_run_date", limit)

    def getTopTestSystems(self, limit=None):
        """See `IHasTestStatPeople`."""
        return self.getTestSystems("-test_run_count", limit)

    def getTestSystems(self, order_by=None, limit=None):
        search_params = TestSystemSearchParams(
            date_modified=SearchNotEqual(None),
            order_by=order_by,
            )
        self.updateTestRankParams(search_params)

        test_systems = getUtility(ITestSystemSet).search(search_params)
        return test_systems.config(limit=limit)

    def updateTestRankParams(self, params):
        # Do nothing by default
        pass


class TestRankTarget:

    def setTestRank(self, *args, **kwargs):
        """See `ITestRankTarget`."""
        set_params = TestRankSetParams(*args, **kwargs)
        self.updateTestRankParams(set_params)

        # This makes sure the target is defined
        set_params.target

        person_id = getattr(set_params.person, "id", None)
        distro_series_id = getattr(set_params.distro_series, "id", None)
        project_series_id = getattr(set_params.project_series, "id", None)
        system_unit_id = getattr(set_params.system_unit, "id", None)

        get_tracker_store().execute(
            "SELECT set_testrank(?, ?, ?, ?, ?)",
            (person_id, distro_series_id, project_series_id,
                system_unit_id, set_params.rank), noresult=True)

    def updateTestRankParams(self, params):
        self.updateTestTargetParams(params)
