# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestHasAttachmentsMixin",
    "TestAttachmentTargetMixin",
    ]

from unittest import defaultTestLoader

from lpresults.tracker.testing.layers import FunctionalStorageLayer


class TestHasAttachmentsMixin:

    layer = FunctionalStorageLayer

    def getAttachmentTarget(self):
        raise NotImplementedError()

    def test_attachments(self):
        target = self.getAttachmentTarget()
        self.assertEquals(len(target.attachments), 0)


class TestAttachmentTargetMixin(TestHasAttachmentsMixin):

    def test_add(self):
        target = self.getAttachmentTarget()
        self.assertEquals(len(target.attachments), 0)

        person = self.factory.makeTestPerson()
        filename = self.factory.getUniqueString("filename")
        data = self.factory.getUniqueString("data")
        target.addAttachment(person, filename, data)
        self.assertEquals(len(target.attachments), 1)

        attachment = target.attachments[0]
        self.assertEquals(attachment.data.read(), data)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
