# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from lpresults.tracker.interfaces.testcase import (
    TestCaseGetParams,
    TestCaseSearchParams,
    )
from lpresults.tracker.models.testcase import test_case_set
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestCaseSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def setUp(self):
        super(TestTestCaseSet, self).setUp()
        self.project = self.factory.makeTestProject()

    def getTestCaseId(self, **kwargs):
        get_params = TestCaseGetParams(**kwargs)
        self.project.updateTestTargetParams(get_params)
        return test_case_set.getId(get_params)

    def searchTestCases(self, **kwargs):
        search_params = TestCaseSearchParams(**kwargs)
        self.project.updateTestTargetParams(search_params)
        return test_case_set.search(search_params)

    def test_getId(self):
        kwargs = {
            "name": self.factory.getUniqueString(u"test-name"),
            }

        id = self.getTestCaseId(**kwargs)
        self.assertNotEquals(id, None)

        case = self.searchTestCases(**kwargs).one()
        self.assertNotEquals(case, None)
        self.assertEquals(case.name, kwargs["name"])

    def test_getIdTwice(self):
        kwargs = {
            "name": self.factory.getUniqueString(u"test-name"),
            }

        id1 = self.getTestCaseId(**kwargs)
        id2 = self.getTestCaseId(**kwargs)
        self.assertEquals(id1, id2)

    def test_searchCaseSensitive(self):
        kwargs = {
            "name": self.factory.getUniqueString(u"test-name"),
            }

        self.getTestCaseId(**kwargs)
        case = self.searchTestCases(**kwargs).one()
        self.assertNotEquals(case, None)

    def test_searchCaseInsensitive(self):
        kwargs = {
            "name": self.factory.getUniqueString(u"test-name"),
            }

        self.getTestCaseId(**kwargs)

        kwargs["name"] = kwargs["name"].lower()
        case = self.searchTestCases(**kwargs).one()
        self.assertNotEquals(case, None)

        kwargs["name"] = kwargs["name"].upper()
        case = self.searchTestCases(**kwargs).one()
        self.assertNotEquals(case, None)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
