# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.tracker.interfaces.testdistribution import (
    ITestDistribution,
    ITestDistributionSet,
    TestDistributionSearchParams,
    )
from lpresults.tracker.models.tests.test_teststat import (
    TestHasTestStatDistributionsMixin,
    TestTestStatTargetMixin,
    )
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestDistribution(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_init(self):
        test_distribution = self.factory.makeTestDistribution()
        self.assertTrue(ITestDistribution.providedBy(test_distribution))

    def test_cmp(self):
        test_distribution = self.factory.makeTestDistribution()
        self.assertEquals(test_distribution, test_distribution)
        self.assertEquals(test_distribution, test_distribution.context)


class TestTestDistributionSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_get(self):
        test_distribution = self.factory.makeTestDistribution()
        other = getUtility(ITestDistributionSet).get(test_distribution.id)
        self.assertNotEquals(other, None)
        self.assertTrue(ITestDistribution.providedBy(other))
        self.assertEquals(other, test_distribution)

    def test_searchName(self):
        name = self.factory.getUniqueString(u"distribution-name")
        params = TestDistributionSearchParams(name=name)
        other = getUtility(ITestDistributionSet).search(params).one()
        self.assertEquals(other, None)

        distribution = self.factory.makeDistribution(name=name)
        test_distribution = self.factory.makeTestDistribution(
            distribution=distribution)
        other = getUtility(ITestDistributionSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertTrue(ITestDistribution.providedBy(other))
        self.assertEquals(other, test_distribution)

    def test_searchByPerson(self):
        person = self.factory.makeTestPerson()
        params = TestDistributionSearchParams()
        params.setPerson(person=person)
        distributions = getUtility(ITestDistributionSet).search(params)
        self.assertEquals(distributions.count(), 0)

        test_distribution = self.factory.makeTestDistribution()
        test_distribution.incrementTestStat(person)
        distributions = getUtility(ITestDistributionSet).search(params)
        self.assertEquals(distributions.count(), 1)


class TestHasTestStatDistributions(
    TestCaseWithFactory, TestHasTestStatDistributionsMixin):

    def getContainer(self):
        return getUtility(ITestDistributionSet)


class TestTestStatTarget(
    TestCaseWithFactory, TestTestStatTargetMixin):

    def setUp(self):
        super(TestTestStatTarget, self).setUp()

        self.test_distribution = self.factory.makeTestDistribution()

    def getTestTarget(self):
        return self.test_distribution


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
