# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader


from lpresults.registry.models.distroserieshelper import DistroSeriesTwin
from lpresults.registry.models.tests.test_series import TestSeriesSetMixin

from lpresults.tracker.interfaces.testdistroseries import ITestDistroSeries
from lpresults.tracker.models.tests.test_teststat import (
    TestTestStatTargetMixin,
    )
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestDistroSeries(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_init(self):
        test_distro_series = self.factory.makeTestDistroSeries()
        self.assertTrue(ITestDistroSeries.providedBy(test_distro_series))

    def test_cmp(self):
        test_distro_series = self.factory.makeTestDistroSeries()
        self.assertEquals(test_distro_series, test_distro_series)
        self.assertEquals(test_distro_series, test_distro_series.context)


class TestTestDistroSeriesSet(TestCaseWithFactory, TestSeriesSetMixin):

    layer = FunctionalDatabaseLayer

    def getSeriesContext(self):
        return self.factory.makeTestDistribution()


class TestTestDistroSeriesTwin(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_get(self):
        distro_series = self.factory.makeTestDistroSeries()
        # DistroSeriesTwin only needs to appear in its store
        distro_series_twin = distro_series.test_store.get(
            DistroSeriesTwin, distro_series.id)
        self.assertTrue(distro_series_twin)
        self.assertEquals(distro_series_twin.id, distro_series.id)

    def test_distroSeriesAttribute(self):
        distro_series = self.factory.makeTestDistroSeries()
        # DistroSeriesTwin only needs to appear in its store
        distro_series_twin = distro_series.test_store.get(
            DistroSeriesTwin, distro_series.id)
        self.assertEquals(distro_series_twin.distro_series, distro_series)


class TestTestStatTarget(
    TestCaseWithFactory, TestTestStatTargetMixin):

    def setUp(self):
        super(TestTestStatTarget, self).setUp()

        self.test_distro_series = self.factory.makeTestDistroSeries()

    def getTestTarget(self):
        return self.test_distro_series


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
