# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.registry.models.personhelper import PersonTwin

from lpresults.tracker.interfaces.testperson import (
    ITestPerson,
    ITestPersonSet,
    TestPersonSearchParams,
    )
from lpresults.tracker.models.teststore import get_test_stores
from lpresults.tracker.models.tests.test_teststat import (
    TestHasTestStatPeopleMixin,
    TestTestStatPersonMixin,
    )
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestPerson(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_init(self):
        test_person = self.factory.makeTestPerson()
        self.assertTrue(ITestPerson.providedBy(test_person))

    def test_cmp(self):
        test_person = self.factory.makeTestPerson()
        self.assertEquals(test_person, test_person)
        self.assertEquals(test_person, test_person.context)


class TestTestPersonSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_get(self):
        test_person = self.factory.makeTestPerson()
        other = getUtility(ITestPersonSet).get(test_person.id)
        self.assertNotEquals(other, None)
        self.assertTrue(ITestPerson.providedBy(other))
        self.assertEquals(other, test_person)

    def test_searchName(self):
        name = self.factory.getUniqueString(u"test_person-name")
        params = TestPersonSearchParams(name=name)
        other = getUtility(ITestPersonSet).search(params).one()
        self.assertEquals(other, None)

        person = self.factory.makePerson(name=name)
        test_person = self.factory.makeTestPerson(person=person)
        other = getUtility(ITestPersonSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertTrue(ITestPerson.providedBy(other))
        self.assertEquals(other, test_person)

    def test_searchByProject(self):
        project = self.factory.makeTestProject()
        params = TestPersonSearchParams()
        project.updateTestTargetParams(params)
        people = getUtility(ITestPersonSet).search(params)
        self.assertEquals(people.count(), 0)

        test_person = self.factory.makeTestPerson()
        project.incrementTestStat(test_person)
        people = getUtility(ITestPersonSet).search(params)
        self.assertEquals(people.count(), 1)


class TestTestPersonTwin(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_get(self):
        person = self.factory.makePerson()
        # PersonTwin should appear in each test store
        for name, test_store in get_test_stores():
            person_twin = test_store.get(PersonTwin, person.id)
            self.assertTrue(person_twin)
            self.assertEquals(person_twin.id, person.id)

    def test_personAttribute(self):
        person = self.factory.makePerson()
        # PersonTwin should appear in each test store
        for name, test_store in get_test_stores():
            person_twin = test_store.get(PersonTwin, person.id)
            self.assertEquals(person_twin.person, person)


class TestHasTestStatPeople(
    TestCaseWithFactory, TestHasTestStatPeopleMixin):

    def getContainer(self):
        return getUtility(ITestPersonSet)

    def getTestTarget(self):
        return self.factory.makeTestProject()


class TestTestStatPerson(TestCaseWithFactory, TestTestStatPersonMixin):

    def setUp(self):
        super(TestTestStatPerson, self).setUp()

        self.test_person = self.factory.makeTestPerson()

    def getTestPerson(self):
        return self.test_person


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
