# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.tracker.interfaces.testdistribution import ITestDistribution
from lpresults.tracker.interfaces.testpillar import (
    ITestPillarSet,
    TestPillarSearchParams,
    )
from lpresults.tracker.interfaces.testproject import ITestProject
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestPillarSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_createDistribution(self):
        name = self.factory.getUniqueString(u"distribution-name")
        series_name = self.factory.getUniqueString(u"series-name")
        distribution = getUtility(ITestPillarSet).createDistribution(
            name, series_name)
        self.assertNotEquals(distribution, None)
        self.assertTrue(ITestDistribution.providedBy(distribution))
        self.assertEquals(distribution.name, name)

    def test_createProject(self):
        name = self.factory.getUniqueString(u"project-name")
        series_name = self.factory.getUniqueString(u"series-name")
        project = getUtility(ITestPillarSet).createProject(
            name, series_name)
        self.assertNotEquals(project, None)
        self.assertTrue(ITestProject.providedBy(project))
        self.assertEquals(project.name, name)

    def test_searchProjectName(self):
        name = self.factory.getUniqueString(u"project-name")
        params = TestPillarSearchParams(name=name)
        other = getUtility(ITestPillarSet).search(params).one()
        self.assertEquals(other, None)

        project = self.factory.makeProject(name=name)
        test_project = self.factory.makeTestProject(project=project)
        other = getUtility(ITestPillarSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, test_project)

    def test_searchDistributionName(self):
        name = self.factory.getUniqueString(u"distribution-name")
        params = TestPillarSearchParams(name=name)
        other = getUtility(ITestPillarSet).search(params).one()
        self.assertEquals(other, None)

        distribution = self.factory.makeDistribution(name=name)
        test_distribution = self.factory.makeTestDistribution(
            distribution=distribution)
        other = getUtility(ITestPillarSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertEquals(other, test_distribution)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
