# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from zope.component import getUtility

from lpresults.tracker.interfaces.testproject import (
    ITestProject,
    ITestProjectSet,
    TestProjectSearchParams,
    )
from lpresults.tracker.models.tests.test_teststat import (
    TestHasTestStatProjectsMixin,
    TestTestStatTargetMixin,
    )
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestProject(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_init(self):
        test_project = self.factory.makeTestProject()
        self.assertTrue(ITestProject.providedBy(test_project))

    def test_cmp(self):
        test_project = self.factory.makeTestProject()
        self.assertEquals(test_project, test_project)
        self.assertEquals(test_project, test_project.context)


class TestTestProjectSet(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_get(self):
        test_project = self.factory.makeTestProject()
        other = getUtility(ITestProjectSet).get(test_project.id)
        self.assertNotEquals(other, None)
        self.assertTrue(ITestProject.providedBy(other))
        self.assertEquals(other, test_project)

    def test_searchName(self):
        name = self.factory.getUniqueString(u"project-name")
        params = TestProjectSearchParams(name=name)
        other = getUtility(ITestProjectSet).search(params).one()
        self.assertEquals(other, None)

        project = self.factory.makeProject(name=name)
        test_project = self.factory.makeTestProject(project=project)
        other = getUtility(ITestProjectSet).search(params).one()
        self.assertNotEquals(other, None)
        self.assertTrue(ITestProject.providedBy(other))
        self.assertEquals(other, test_project)

    def test_searchByPerson(self):
        person = self.factory.makeTestPerson()
        params = TestProjectSearchParams()
        params.setPerson(person=person)
        projects = getUtility(ITestProjectSet).search(params)
        self.assertEquals(projects.count(), 0)

        test_project = self.factory.makeTestProject()
        test_project.incrementTestStat(person)
        projects = getUtility(ITestProjectSet).search(params)
        self.assertEquals(projects.count(), 1)


class TestHasTestStatProjects(
    TestCaseWithFactory, TestHasTestStatProjectsMixin):

    def getContainer(self):
        return getUtility(ITestProjectSet)


class TestTestStatTarget(
    TestCaseWithFactory, TestTestStatTargetMixin):

    def setUp(self):
        super(TestTestStatTarget, self).setUp()

        self.test_project = self.factory.makeTestProject()

    def getTestTarget(self):
        return self.test_project


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
