# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from lpresults.registry.models.projectserieshelper import ProjectSeriesTwin
from lpresults.registry.models.tests.test_series import TestSeriesSetMixin

from lpresults.tracker.interfaces.testprojectseries import ITestProjectSeries
from lpresults.tracker.models.tests.test_teststat import (
    TestTestStatTargetMixin,
    )
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestProjectSeries(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_init(self):
        test_project_series = self.factory.makeTestProjectSeries()
        self.assertTrue(ITestProjectSeries.providedBy(test_project_series))

    def test_cmp(self):
        test_project_series = self.factory.makeTestProjectSeries()
        self.assertEquals(test_project_series, test_project_series)
        self.assertEquals(test_project_series, test_project_series.context)


class TestTestProjectSeriesSet(TestCaseWithFactory, TestSeriesSetMixin):

    layer = FunctionalDatabaseLayer

    def getSeriesContext(self):
        return self.factory.makeTestProject()


class TestTestProjectSeriesTwin(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_get(self):
        project_series = self.factory.makeTestProjectSeries()
        # ProjectSeriesTwin only needs to appear in its store
        project_series_twin = project_series.test_store.get(
            ProjectSeriesTwin, project_series.id)
        self.assertTrue(project_series_twin)
        self.assertEquals(project_series_twin.id, project_series.id)

    def test_projectSeriesAttribute(self):
        project_series = self.factory.makeTestProjectSeries()
        # ProjectSeriesTwin only needs to appear in its store
        project_series_twin = project_series.test_store.get(
            ProjectSeriesTwin, project_series.id)
        self.assertEquals(project_series_twin.project_series, project_series)


class TestTestStatTarget(
    TestCaseWithFactory, TestTestStatTargetMixin):

    def setUp(self):
        super(TestTestStatTarget, self).setUp()

        self.test_project_series = self.factory.makeTestProjectSeries()

    def getTestTarget(self):
        return self.test_project_series


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
