# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestHasTestRankSystemsMixin",
    ]

import pytz

from datetime import (
    datetime,
    timedelta,
    )

from unittest import defaultTestLoader

from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


UTC = pytz.timezone("UTC")


class TestHasTestRankSystemsMixin:

    layer = FunctionalDatabaseLayer

    def getContainer(self):
        raise NotImplementedError()

    def getTestTarget(self):
        raise NotImplementedError()

    def test_getLatestTestSystems(self):
        container = self.getContainer()
        target = self.getTestTarget()

        # Create older target
        tester1 = self.factory.makePerson()
        date = datetime(2010, 1, 1, tzinfo=UTC)
        target.incrementTestStat(tester1, date_modified=date)
        unit1 = self.factory.makeSystemUnit()
        target.setTestRank(tester1, unit1, 0)

        # Create newer target
        tester2 = self.factory.makePerson()
        date += timedelta(days=1)
        target.incrementTestStat(tester2, date_modified=date)
        unit2 = self.factory.makeSystemUnit()
        target.setTestRank(tester2, unit2, 0)

        systems = container.getLatestTestSystems()
        self.assertNotEquals(systems, None)
        self.assertEquals(systems.count(), 2)
        self.assertEquals(systems[0], unit2.system)
        self.assertEquals(systems[1], unit1.system)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
