# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from storm.zope.zstorm import ZStorm
from storm.locals import (
    Store,
    Int,
    )

from zope.component import provideUtility

from lpresults.tracker.models.teststore import (
    LazyTestStore,
    HasTestStore,
    )


class TestHasTestStore(TestCase):

    def setUp(self):

        class StubHasTestStore(HasTestStore):

            @property
            def test_store_name(self):
                return "stub-has-test-store"

        self.has_test_store = StubHasTestStore()
        self.zstorm = ZStorm()
        provideUtility(self.zstorm)

        self.test_store = self.zstorm.get("stub-has-test-store", "sqlite:")
        self.test_store.execute("CREATE TABLE test (id INTEGER PRIMARY KEY)")

    def test_lazyStoreProperty(self):
        self.assertTrue("test_store" not in self.has_test_store.__dict__)
        self.assertEquals(type(self.has_test_store.test_store), Store)
        self.assertTrue("test_store" in self.has_test_store.__dict__)

    def test_storeHasTheRightName(self):
        name = self.zstorm.get_name(self.has_test_store.test_store)
        self.assertEquals(name, "stub-has-test-store")

    def test_testStoreName(self):
        name = "another-store-name"
        store = self.zstorm.create(name, "sqlite:")

        class StubHasTestStore(HasTestStore):

            @property
            def test_store_name(self):
                return name

        has_test_store = StubHasTestStore()
        self.assertEquals(has_test_store.test_store, store)

    def test_storeAccessOnClass(self):
        self.assertEquals(type(HasTestStore.test_store), LazyTestStore)

    def test_invalidatingObjectResetsStore(self):

        class StormObject(HasTestStore):
            __storm_table__ = "test"

            id = Int(primary=True)

            @property
            def test_store_name(self):
                return "stub-has-test-store"

        object = StormObject()
        self.test_store.add(object)
        self.test_store.flush()
        self.assertEquals(type(object.test_store), Store)
        self.test_store.invalidate()
        self.assertTrue("store" not in object.__dict__)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
