# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from lpresults.hardware.models.systemunithelper import SystemUnitTwin
from lpresults.hardware.models.tests.test_unit import TestUnitSetMixin

from lpresults.tracker.interfaces.testsystemunit import (
    ITestSystemUnit,
    ITestSystemUnitTarget,
    )
from lpresults.tracker.models.teststore import get_test_stores
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import FunctionalDatabaseLayer


class TestTestSystemUnit(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_init(self):
        test_system_unit = self.factory.makeTestSystemUnit()
        self.assertTrue(ITestSystemUnit.providedBy(test_system_unit))

    def test_cmp(self):
        test_system_unit = self.factory.makeTestSystemUnit()
        self.assertEquals(test_system_unit, test_system_unit)
        self.assertEquals(test_system_unit, test_system_unit.context)


class TestTestSystemUnitSet(TestCaseWithFactory, TestUnitSetMixin):

    layer = FunctionalDatabaseLayer

    def getUnitContext(self):
        return self.factory.makeTestSystem()


class TestTestSystemUnitTwin(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def test_get(self):
        system_unit = self.factory.makeSystemUnit()
        # SystemUnitTwin should appear in each test store
        for name, test_store in get_test_stores():
            system_unit_twin = test_store.get(SystemUnitTwin, system_unit.id)
            self.assertTrue(system_unit_twin)
            self.assertEquals(system_unit_twin.id, system_unit.id)

    def test_systemUnitAttribute(self):
        system_unit = self.factory.makeSystemUnit()
        # SystemUnitTwin should appear in each test store
        for name, test_store in get_test_stores():
            system_unit_twin = test_store.get(SystemUnitTwin, system_unit.id)
            self.assertEquals(system_unit_twin.system_unit, system_unit)


class TestTestSystemUnitTarget(TestCaseWithFactory):

    layer = FunctionalDatabaseLayer

    def setUp(self):
        super(TestTestSystemUnitTarget, self).setUp()

        self.person = self.factory.makePerson()
        self.project = self.factory.makeTestProject()

    def createSystemUnitTarget(self, **kwargs):
        """Make an arbitrary test run target with no runs on it."""
        test_run = self.project.createTestRun(self.person, **kwargs)
        return ITestSystemUnitTarget(test_run)

    def test_setSystemUnit(self):
        target = self.createSystemUnitTarget()
        self.assertEquals(target.system_unit, None)

        make = self.factory.getUniqueString(u"test-make")
        model = self.factory.getUniqueString(u"test-model")
        identifiers = [self.factory.getUniqueString(u"test-identifiers")]
        target.setSystemUnit(make, model, identifiers)
        self.assertNotEquals(target.system_unit, None)
        self.assertEquals(target.system_unit.system.make, make)
        self.assertEquals(target.system_unit.system.model, model)
        self.assertEquals(target.system_unit.identifiers, identifiers)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
