# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "HasTestStore",
    "get_test_stores",
    ]

from lpresults.database.store import (
    get_store,
    get_stores,
    )


class LazyTestStore:

    def __get__(self, obj, cls=None):
        if obj is not None:
            obj.test_store = get_store(obj.test_store_name)
            return obj.test_store

        return self


class HasTestStore:

    test_store = LazyTestStore()

    @property
    def test_store_key(self):
        """See `IHasTestStore`."""
        raise NotImplementedError()

    @property
    def test_store_name(self):
        """See `IHasTestStore`."""
        return "test-%d" % self.test_store_key

    def __storm_invalidated__(self):
        """Reset the test store."""
        self.__dict__.pop("test_store", None)


def get_test_stores():
    """Return stores that begin with 'test-'.

    Given that the test_store_key is a numeric value that is appended
    onto the string 'test-' this means that 'test-' should not be used
    to name any non-test databases.
    """
    return get_stores("test")
