# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestTargetHelper",
    "TestTargetMixin",
    ]

from zope.component import getUtility
from zope.interface import implements

from lazr.restful.simple import TraverseWithGet

from lpresults.registry.models.distroserieshelper import DistroSeriesHelper
from lpresults.registry.models.projectserieshelper import ProjectSeriesHelper

from lpresults.tracker.interfaces.attachment import IAttachmentSet
from lpresults.tracker.interfaces.testcase import ITestCaseSet
from lpresults.tracker.interfaces.testrun import ITestRunSet
from lpresults.tracker.interfaces.testtarget import IHasTestTarget


class TestTargetHelper(DistroSeriesHelper, ProjectSeriesHelper):

    implements(IHasTestTarget)

    def __init__(self, distro_series=None, project_series=None):
        DistroSeriesHelper.__init__(self, distro_series)
        ProjectSeriesHelper.__init__(self, project_series)

    @property
    def target(self):
        """See `IHasTestTarget`."""
        if self.distro_series_id:
            return self.distro_series
        elif self.project_series_id:
            return self.project_series
        else:
            raise AssertionError("Unable to determine test run target.")


class TestTargetMixin(TraverseWithGet):

    def get(self, request, name):
        """See `ITraverseWithGet`."""
        # XXX: These statements are not necessarily true for all test
        # targets, not all of them have attachments for example.
        request.target = self
        if name == "attachments":
            return getUtility(IAttachmentSet)
        elif name == "testcases":
            return getUtility(ITestCaseSet)
        elif name == "+testruns":
            return getUtility(ITestRunSet)

        return None

    def updateTestTargetParams(self, target_params):
        raise NotImplementedError()
