# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import defaultTestLoader

from lpresults.database.store import get_store

from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import DatabaseLayer


class TestTestCaseMixin:

    layer = DatabaseLayer

    def assertCountTestCases(self, name, count):
        test_case_count = self.store.execute(
            "SELECT COUNT(*) FROM testcase "
            "WHERE distro_series IS NOT DISTINCT FROM ? "
            "AND project_series IS NOT DISTINCT FROM ? "
            "AND name = ?",
            (self.distro_series, self.project_series, name)).get_one()[0]
        self.assertEquals(test_case_count, count)

    def test_getTestCaseId(self):
        name = self.factory.getUniqueString(u"test-name")

        self.assertCountTestCases(name, 0)

        first_test_case = self.store.execute(
            "SELECT get_test_case(?, ?, ?)",
            (self.distro_series, self.project_series, name)).get_one()[0]
        self.assertNotEquals(first_test_case, None)
        self.assertCountTestCases(name, 1)

        second_test_case = self.store.execute(
            "SELECT get_test_case(?, ?, ?)",
            (self.distro_series, self.project_series, name)).get_one()[0]
        self.assertNotEquals(second_test_case, None)
        self.assertEquals(first_test_case, second_test_case)
        self.assertCountTestCases(name, 1)


class TestTestCaseDistroSeries(TestCaseWithFactory, TestTestCaseMixin):

    def setUp(self):
        super(TestTestCaseDistroSeries, self).setUp()

        self.store = get_store("test-1")
        self.distro_series = self.factory.getUniqueInteger()
        self.distribution = self.factory.getUniqueInteger()
        self.project_series = None
        self.store.execute(
            "INSERT INTO distroseriestwin VALUES (?, ?)",
            (self.distro_series, self.distribution,))

    def tearDown(self):
        super(TestTestCaseDistroSeries, self).tearDown()

        self.store.execute(
            "DELETE FROM distroseriestwin WHERE id = ?",
            (self.distro_series,))


class TestTestCaseProjectSeries(TestCaseWithFactory, TestTestCaseMixin):

    def setUp(self):
        super(TestTestCaseProjectSeries, self).setUp()

        self.store = get_store("test-1")
        self.distro_series = None
        self.project_series = self.factory.getUniqueInteger()
        self.project = self.factory.getUniqueInteger()
        self.store.execute(
            "INSERT INTO projectseriestwin VALUES (?, ?)",
            (self.project_series, self.project,))

    def tearDown(self):
        super(TestTestCaseProjectSeries, self).tearDown()

        self.store.execute(
            "DELETE FROM projectseriestwin WHERE id = ?",
            (self.project_series,))


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
