# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from datetime import datetime

from unittest import defaultTestLoader

from storm.exceptions import InternalError

from lpresults.tracker.models.store import get_tracker_store
from lpresults.tracker.testing.cases import TestCaseWithFactory
from lpresults.tracker.testing.layers import DatabaseLayer


class TestTestStat(TestCaseWithFactory):

    layer = DatabaseLayer

    def setUp(self):
        super(TestTestStat, self).setUp()

        self.store = get_tracker_store()

        person_name = self.factory.getUniqueString(u"person-name")
        person_identity = self.factory.getUniqueString(u"person-identity")
        self.person = self.store.execute(
            "INSERT INTO person "
            "(name, identity) "
            "VALUES (?, ?) RETURNING id",
            (person_name, person_identity,),
            ).get_one()[0]

        project_name = self.factory.getUniqueString(u"project-name")
        project = self.store.execute(
            "INSERT INTO project "
            "(name) "
            "VALUES (?) RETURNING id",
            (project_name,),
            ).get_one()[0]

        project_series_name = self.factory.getUniqueString(
            u"project-series-name")
        self.project_series = self.store.execute(
            "INSERT INTO projectseries "
            "(project, name) "
            "VALUES (?, ?) RETURNING id",
            (project, project_series_name,),
            ).get_one()[0]

        system_make = self.factory.getUniqueString(u"system-make")
        system_model = self.factory.getUniqueString(u"system-model")
        system = self.store.execute(
            "INSERT INTO system "
            "(make, model) "
            "VALUES (?, ?) RETURNING id",
            (system_make, system_model,),
            ).get_one()[0]

        system_identifiers = [
            self.factory.getUniqueString(u"system-identifier")]
        self.system_unit = self.store.execute(
            "INSERT INTO systemunit "
            "(system, identifiers) "
            "VALUES (?, ?) RETURNING id",
            (system, system_identifiers,),
            ).get_one()[0]

    def test_setTestRankError(self):
        self.assertRaises(InternalError,
            self.store.execute,
            "SELECT set_testrank(?, ?, ?, ?, ?)",
            (self.person, None, self.project_series, self.system_unit, 3,))

    def test_setTestStat(self):
        # Check that the rank can be set
        date_modified = datetime(2010, 1, 1)
        self.store.execute(
            "SELECT increment_teststat(?, ?, ?, ?)",
            (self.person, None, self.project_series, date_modified,),
            )
        self.store.execute(
            "SELECT set_testrank(?, ?, ?, ?, ?)",
            (self.person, None, self.project_series, self.system_unit, 3,))

        # Check that the rank has been set
        rank = self.store.execute(
            "SELECT rank FROM testrank WHERE system_unit = ?",
            (self.system_unit,)).get_one()[0]
        self.assertEquals(rank, 3)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
