# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "APIApplication",
    ]

from pkg_resources import resource_string

from django.contrib.auth import authenticate

from zope.configuration import xmlconfig

from storm.django.middleware import ZopeTransactionMiddleware

from lazr.restful.simple import Publication
from lazr.restful.wsgi import WSGIApplication

from lpresults.tracker import settings


class APIPublication(Publication):

    def __init__(self, *args, **kwargs):
        super(APIPublication, self).__init__(*args, **kwargs)
        self.middleware = ZopeTransactionMiddleware()

    def beforeTraversal(self, request):
        """Begin a transaction before traversal."""
        self.middleware.process_request(request)

        # Set the user which might be Anonymous.
        request.user = authenticate(request=request)

        super(APIPublication, self).beforeTraversal(request)

    def afterCall(self, request, ob):
        """Commit or abort the transaction after the call.

        On successful completion of the request, the transaction
        will be committed.
        """
        self.middleware.process_response(request, ob)

        super(APIPublication, self).afterCall(request, ob)

    def handleException(self, object, request, exc_info, retry_allowed=1):
        """Abort the transaction on exceptions."""
        self.middleware.process_exception(request, exc_info)

        base = super(APIPublication, self)
        base.handleException(object, request, exc_info, retry_allowed)


class APIApplication(WSGIApplication):
    """API End-Point."""

    publication_class = APIPublication

    @classmethod
    def configure_server(
        cls, config_package=settings.__package__, config_file="api.zcml"):
        """Configure server for a particular API service."""
        zcml = resource_string(config_package, config_file)
        xmlconfig.string(zcml)
