# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Patcher functions."""

__metaclass__ = type

__all__ = [
    "patch_entry_return_type",
    "patch_collection_return_type",
    ]

from lazr.restful.declarations import LAZR_WEBSERVICE_EXPORTED


def patch_entry_return_type(exported_class, method_name, return_type):
    """Update return type for a webservice method that returns entries.

    :param exported_class: The class containing the method.
    :param method_name: The method name that you need to patch.
    :param return_type: The new return type for the method.
    """
    exported_class[method_name].queryTaggedValue(
        LAZR_WEBSERVICE_EXPORTED)["return_type"].schema = return_type


def patch_collection_return_type(exported_class, method_name, return_type):
    """Update return type for a webservice method that returns a collection.

    :param exported_class: The class containing the method.
    :param method_name: The method name that you need to patch.
    :param return_type: The new return type for the method.
    """
    collection = exported_class[method_name].queryTaggedValue(
        LAZR_WEBSERVICE_EXPORTED)
    collection["return_type"].value_type.schema = return_type
