# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "get_host_from_request",
    ]


def get_host_from_request(request):
    """Get the hostname and port using the request headers.

    :param request: HTTP request.
    """
    if "HTTP_X_FORWARDED_HOST" in request:
        host = request["HTTP_X_FORWARDED_HOST"]

    elif "HTTP_HOST" in request:
        host = request["HTTP_HOST"]

    else:
        # Reconstruct the host using the algorithm from PEP 333.
        host = request["SERVER_NAME"]
        port = request.get("SERVER_PORT", None)
        if port is not None:
            host = ":".join([host, port])

    # Split the given host string into a hostname and port.
    if ":" in host:
        hostname, port = host.split(":", 1)
        try:
            port = int(port)
        except ValueError:
            port = None

    else:
        hostname = host
        port = None

    return hostname, port
