# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "StorageServiceConfiguration",
    "StorageServiceRoot",
    "StorageServiceRootAbsoluteURL",
    ]

from lpresults.storage.django.configuration import DjangoStorageConfiguration
from lpresults.storage.zope.simple import (
    StorageRoot,
    StorageRootAbsoluteURL,
    )
from lpresults.storage.zope.wsgi import WSGIStorageConfiguration

from lpresults.tracker.services.request import get_host_from_request


class StorageServiceConfiguration(
    DjangoStorageConfiguration, WSGIStorageConfiguration):
    """
    Configuration information for the storage service.

    This class is defined here so that Grok can pick it up, but whole
    configuration is actually stored in settings.py
    """


class StorageServiceRoot(StorageRoot):
    """
    This class contains no code of its own. It's defined so that grok
    will pick it up.
    """


class StorageServiceRootAbsoluteURL(StorageRootAbsoluteURL):
    """
    This class is mostly defined so that grok will pick it up but also
    provides support for getting host information from the given request.
    """

    def __init__(self, context, request):
        super(StorageServiceRootAbsoluteURL, self).__init__(context, request)

        if not self.host:
            self.host, self.port = get_host_from_request(request)
