# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.tracker.services.request import get_host_from_request


class TestGetHostFromRequest(TestCase):

    def test_http_x_forwarded_host(self):
        host, port = get_host_from_request({
            "HTTP_X_FORWARDED_HOST": "localhost"})
        self.assertEquals(host, "localhost")
        self.assertEquals(port, None)

        host, port = get_host_from_request({
            "HTTP_X_FORWARDED_HOST": "localhost:80"})
        self.assertEquals(host, "localhost")
        self.assertEquals(port, 80)

    def test_http_host(self):
        host, port = get_host_from_request({
            "HTTP_HOST": "localhost"})
        self.assertEquals(host, "localhost")
        self.assertEquals(port, None)

        host, port = get_host_from_request({
            "HTTP_HOST": "localhost:80"})
        self.assertEquals(host, "localhost")
        self.assertEquals(port, 80)

    def test_server_name(self):
        host, port = get_host_from_request({
            "SERVER_NAME": "localhost"})
        self.assertEquals(host, "localhost")
        self.assertEquals(port, None)

        host, port = get_host_from_request({
            "SERVER_NAME": "localhost",
            "SERVER_PORT": "80"})
        self.assertEquals(host, "localhost")
        self.assertEquals(port, 80)

    def test_invalid_port(self):
        host, port = get_host_from_request({
            "HTTP_HOST": "localhost:foo"})
        self.assertEquals(host, "localhost")
        self.assertEquals(port, None)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
