# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from zope.component import (
    getUtility,
    queryUtility,
    )
from zope.traversing.browser import absoluteURL

from lazr.restful.interfaces import (
    IWebServiceConfiguration,
    IServiceRootResource,
    )

from lpresults.tracker.services.webservice import (
    WebServiceConfiguration,
    WebServiceRoot,
    )
from lpresults.tracker.testing.layers import FunctionalLayer


class TestWebServiceConfiguration(TestCase):

    layer = FunctionalLayer

    def test_activeVersions(self):
        active_versions = WebServiceConfiguration.active_versions
        self.assertTrue("1.0" in active_versions)

    def test_utility(self):
        utility = queryUtility(IWebServiceConfiguration)
        self.assertTrue(isinstance(utility, WebServiceConfiguration))


class TestWebServiceRoot(TestCase):

    layer = FunctionalLayer

    def test_utility(self):
        utility = queryUtility(IServiceRootResource)
        self.assertTrue(isinstance(utility, WebServiceRoot))


class TestWebServiceRootAbsoluteURL(TestCase):

    layer = FunctionalLayer

    def test_str(self):
        configuration = getUtility(IWebServiceConfiguration)
        request = configuration.createRequest("", {"HTTP_HOST": "localhost"})
        request.annotations[request.VERSION_ANNOTATION] = "1.0"

        context = getUtility(IServiceRootResource)
        url = absoluteURL(context, request)
        self.assertEquals(url, "http://localhost/+api/1.0/")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
