# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestFactory",
    ]

from zope.component import getUtility

from lpresults.hardware.testing import factories as hardware_factories
from lpresults.registry.testing import factories as registry_factories

from lpresults.tracker.interfaces.testdistribution import ITestDistributionSet
from lpresults.tracker.interfaces.testperson import ITestPersonSet
from lpresults.tracker.interfaces.testproject import ITestProjectSet
from lpresults.tracker.interfaces.testsystem import ITestSystemSet


class TestFactory(
    registry_factories.TestFactory, hardware_factories.TestFactory):

    def makeTestDistribution(self, distribution=None, **kwargs):
        if distribution is None:
            distribution = self.makeDistribution(**kwargs)

        return getUtility(ITestDistributionSet).get(distribution.id)

    def makeTestDistroSeries(self, name=None, distribution_name=None):
        distribution = self.makeTestDistribution(
            name=distribution_name, series_name=name)
        return distribution.current_series

    def makeTestPerson(self, person=None, **kwargs):
        if person is None:
            person = self.makePerson(**kwargs)

        return getUtility(ITestPersonSet).get(person.id)

    def makeTestProject(self, project=None, **kwargs):
        if project is None:
            project = self.makeProject(**kwargs)

        return getUtility(ITestProjectSet).get(project.id)

    def makeTestProjectSeries(self, name=None, project_name=None):
        project = self.makeTestProject(name=project_name, series_name=name)
        return project.development_focus

    def makeTestSystem(self, system=None, **kwargs):
        if system is None:
            system = self.makeSystem(**kwargs)

        return getUtility(ITestSystemSet).get(system.id)

    def makeTestSystemUnit(self, identifiers=None, **kwargs):
        system = self.makeTestSystem(**kwargs)
        if identifiers is None:
            identifiers = [self.getUniqueString(u"unit-identifer")]

        return system.newUnit(identifiers)
