# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "FunctionalLayer",
    "FunctionalDatabaseLayer",
    "FunctionalStorageLayer",
    ]

from lpresults.testing import layers

from lpresults.database.testing import layers as database_layers

from lpresults.storage.testing import layers as storage_layers


class DatabaseLayerFactory(database_layers.DatabaseLayerFactory):
    """Sets up PostgreSQL databases for use in tests."""

    stores = [
        "tracker",
        "test-1",
        "test-2",
        ]

    schema_package_template = "lpresults.tracker.schema.%s"


DatabaseLayer = DatabaseLayerFactory("DatabaseLayer")


class FunctionalLayerFactory(layers.FunctionalLayerFactory):

    zcml = "site.zcml"


FunctionalLayer = FunctionalLayerFactory("FunctionalLayer")


class FunctionalDatabaseLayerFactory:
    """Loads functional and database layers for use in tests."""

    __bases__ = ()

    def __init__(self, name, *args, **kwargs):
        self.__name__ = name
        self.functional = FunctionalLayerFactory(name)
        self.database = DatabaseLayerFactory(name, *args, **kwargs)

    def setUp(self):
        self.functional.setUp()
        self.database.setUp()

    def tearDown(self):
        self.functional.tearDown()
        self.database.tearDown()

    def testSetUp(self):
        self.functional.testSetUp()
        self.database.testSetUp()

    def testTearDown(self):
        self.functional.testTearDown()
        self.database.testTearDown()


FunctionalDatabaseLayer = FunctionalDatabaseLayerFactory(
    "FunctionalDatabaseLayer",
    database_id="functional-layer")


class FunctionalStorageLayerFactory:
    """Loads functional and storage layers for use in tests."""

    __bases__ = ()

    def __init__(self, name, *args, **kwargs):
        self.__name__ = name
        self.functional = FunctionalLayerFactory(name)
        self.storage = storage_layers.StorageLayerFactory(
            name, *args, **kwargs)
        self.database = DatabaseLayerFactory(name)

    def setUp(self):
        self.functional.setUp()
        self.storage.setUp()
        self.database.setUp()

    def tearDown(self):
        self.functional.tearDown()
        self.storage.tearDown()
        self.database.tearDown()

    def testSetUp(self):
        self.functional.testSetUp()
        self.storage.testSetUp()
        self.database.testSetUp()

    def testTearDown(self):
        self.functional.testTearDown()
        self.storage.testTearDown()
        self.database.testTearDown()


FunctionalStorageLayer = FunctionalStorageLayerFactory(
    "FunctionalStorageLayer",
    storage_id="functional-layer")
