# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "handler404",
    "handler500",
    "urlpatterns",
    ]

import os
import re

from django.conf import settings
from django.conf.urls.defaults import patterns

from lpresults.registry.validators import VALID_NAME_PATTERN


handler404 = "lpresults.tracker.views.errors.not_found"
handler500 = "lpresults.tracker.views.errors.server_error"


# Static files
urlpatterns = patterns("django.views.static",
    (r"^favicon.ico$", "serve", {
        "document_root": settings.MEDIA_ROOT,
        "path": os.path.join(
            "launchpad", "favicon.ico"),
        }),
    (r"^robots.txt$", "serve", {
        "document_root": settings.MEDIA_ROOT,
        "path": "robots.txt",
        }),
    (r"^@@/(?P<path>.*)$", "serve", {
        "document_root": os.path.join(
            settings.MEDIA_ROOT, "launchpad", "images"),
        }),
    (r"^%s(?P<path>.*)$" % re.escape(settings.MEDIA_URL[1:]), "serve", {
        "document_root": settings.MEDIA_ROOT,
        }),
    )

# Dynamic files
urlpatterns += patterns("lpresults.tracker.views",
    (r"^$", "root"),
    (r"^%s$" % re.escape(settings.LOGIN_URL[1:]), "login_begin"),
    (r"^%s$" % re.escape(settings.COMPLETE_URL[1:]), "login_complete"),
    (r"^%s$" % re.escape(settings.LOGOUT_URL[1:]), "logout"),
    (r"^people/?$", "people"),
    (r"^projects/?$", "projects"),
    (r"^systems/?$", "systems"),
    (r"^systems/(?P<system_id>%(id)s)/?$"
        % {"id": r"\d+"}, "system"),
    (r"^~(?P<person_name>%(name)s)/?$"
        % {"name": VALID_NAME_PATTERN}, "person"),
    (r"^(?P<pillar_name>%(name)s)/?$"
        % {"name": VALID_NAME_PATTERN}, "pillar"),
    (r"^(?P<pillar_name>%(name)s)/\+series/?$"
        % {"name": VALID_NAME_PATTERN}, "series"),
    (r"^(?P<pillar_name>%(name)s)/(?P<series_name>%(name)s)/?$"
        % {"name": VALID_NAME_PATTERN}, "series"),
    (r"^(?P<pillar_name>%(name)s)(/(?P<series_name>%(name)s))?/\+testers/?$"
        % {"name": VALID_NAME_PATTERN}, "people"),
    (r"^(?P<pillar_name>%(name)s)(/(?P<series_name>%(name)s))?/\+testruns/?$"
        % {"name": VALID_NAME_PATTERN}, "testruns"),
    (r"^(?P<pillar_name>%(name)s)/(?P<series_name>%(name)s)"
        "/\+testruns/(?P<sequence>%(sequence)s)/?$"
        % {"name": VALID_NAME_PATTERN, "sequence": r"\d+"}, "testrun"),
    (r"^\+apidoc/(?P<path>.*)$", "apidoc"),
    (r"^\+gm/(?P<path>.*)$", "gm"),
    )

# Redirection patterns
urlpatterns += patterns("django.views.generic.simple",
    (r"^\+api$", "redirect_to", {"url": "/+api/"}),
    (r"^\+apidoc$", "redirect_to", {"url": "/+apidoc/"}),
    )

# Launchpad patterns
urlpatterns += patterns("lpresults.tracker.views",
    (r"^\+access-token$", "oauth_access"),
    (r"^\+authorize-token$", "oauth_authorize"),
    (r"^\+request-token$", "oauth_request"),
    )
