# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "unauthorized",
    "forbidden",
    "not_found",
    "server_error",
    ]

from django.http import HttpResponse
from django.template import (
    RequestContext,
    loader,
    )


class ErrorPage:

    def __init__(self, status, errormsg=None):
        self.template_name = "errors/%s.html" % status
        self.status_code = status
        self.errormsg = errormsg

    def __call__(self, request):
        template = loader.get_template(self.template_name)
        items = {
            "errormsg": self.errormsg,
            }
        context = RequestContext(request, items)
        return HttpResponse(
            template.render(context),
            status=self.status_code)


unauthorized = ErrorPage(401)
forbidden = ErrorPage(403)
not_found = ErrorPage(404)
server_error = ErrorPage(500)
