# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "login_begin",
    "login_complete",
    "logout",
    ]

import urllib

from django.contrib.auth import (
    REDIRECT_FIELD_NAME,
    authenticate,
    login as auth_login,
    logout as auth_logout,
    )
from django.core.urlresolvers import reverse
from django.http import (
    HttpResponseForbidden,
    HttpResponseRedirect,
    )

from zope.component import getUtility

from lpresults.session.interfaces import IOpenIDFactory


def login_begin(request, redirect_field_name=REDIRECT_FIELD_NAME):
    redirect_url = request.REQUEST.get(redirect_field_name)
    return_to = reverse(login_complete)
    if redirect_url:
        redirect_url = redirect_url.encode("utf-8")
        # The urllib quoting isn't technically necessary, since it
        # seems python-openid will deal with improperly escaped URLs.
        # I'm just doing it to maintain my mental well-being.
        return_to += "?" + urllib.urlencode({
            redirect_field_name: redirect_url,
            })

    factory = getUtility(IOpenIDFactory)
    openid = factory.getOpenID(request)

    return openid.begin(return_to)


def login_complete(request, redirect_field_name=REDIRECT_FIELD_NAME):
    user = authenticate(request=request)
    if user is not None:
        auth_login(request, user)
        redirect_url = request.REQUEST.get(redirect_field_name, "/")
        return HttpResponseRedirect(redirect_url)

    else:
        return HttpResponseForbidden("Unknown user")


def logout(request):
    identity = None
    if request.user:
        identity = request.user.identity

    auth_logout(request)

    if identity:
        factory = getUtility(IOpenIDFactory)
        return_to = factory.logout_url

    else:
        return_to = reverse(login_begin)

    return HttpResponseRedirect(return_to)
