# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""OAuth integration for Launchpad Results."""

__metaclass__ = type

__all__ = [
    "oauth_request",
    "oauth_authorize",
    "oauth_access",
    ]

import simplejson

from urlparse import parse_qs

from django.contrib.auth.decorators import login_required
from django.http import HttpResponseForbidden

from zope.component import getUtility

from lpresults.tracker.http import HttpResponseUnauthorized

from lpresults.registry.interfaces.launchpad import ILaunchpad
from lpresults.registry.interfaces.oauth import (
    IOAuthDataStore,
    OAuthPermission,
    )


def oauth_request(request):
    response = getUtility(ILaunchpad).proxy(request)
    params = simplejson.loads(unicode(response.content))

    consumer_key = params["oauth_token_consumer"]
    data_store = getUtility(IOAuthDataStore)
    consumer = data_store.lookupConsumer(consumer_key)
    if not consumer:
        consumer = data_store.createConsumer(consumer_key)

    consumer.createRequestToken(
        params["oauth_token"],
        params["oauth_token_secret"])

    return response


@login_required
def oauth_authorize(request):
    params = request.GET
    token = getUtility(IOAuthDataStore).lookupRequest(
        params.get("oauth_token"))
    if token is not None:
        token.review(request.user, OAuthPermission.WRITE_PRIVATE)

    return getUtility(ILaunchpad).proxy(request)


def oauth_access(request):
    params = request.POST
    data_store = getUtility(IOAuthDataStore)
    consumer = data_store.lookupConsumer(params.get("oauth_consumer_key"))
    if consumer is None:
        return u""

    token = data_store.lookupRequest(params.get("oauth_token"))
    if token is None:
        return HttpResponseUnauthorized("No request token specified.")

    expected_signature = "&".join([consumer.secret, token.secret])
    if expected_signature != params.get("oauth_signature"):
        return HttpResponseUnauthorized("Invalid OAuth signature.")

    if not token.is_reviewed:
        return HttpResponseUnauthorized(
            "Request token has not yet been reviewed. Try again later.")

    response = getUtility(ILaunchpad).proxy(request)
    query = parse_qs(unicode(response.content))
    if not query:
        return HttpResponseForbidden(
            "End-user refused to authorize request token.")

    token.createAccessToken(
        query["oauth_token"][0],
        query["oauth_token_secret"][0])

    return response
