# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Person related views."""

__metaclass__ = type

__all__ = [
    "person",
    "people",
    ]

from django.http import HttpResponseNotFound
from django.shortcuts import render_to_response
from django.template import RequestContext

from zope.component import getUtility

from lpresults.tracker.interfaces.testperson import ITestPersonSet
from lpresults.tracker.interfaces.testpillar import ITestPillarSet


def person(request, person_name):
    person = getUtility(ITestPersonSet).getByName(person_name)
    if person is None:
        return HttpResponseNotFound("Invalid person name: %s" % person_name)

    dictionary = {
        "person": person,
        }

    return render_to_response(
        "person/index.html",
        dictionary,
        RequestContext(request))


def people(request, pillar_name=None, series_name=None):
    dictionary = {}
    if pillar_name is not None:
        pillar = getUtility(ITestPillarSet).getByName(pillar_name)
        if pillar is None:
            return HttpResponseNotFound(
                "Invalid pillar name: %s" % pillar_name)

        dictionary["pillar"] = pillar

        if series_name is not None:
            series = pillar.getSeries(series_name)
            if series is None:
                return HttpResponseNotFound(
                    "Invalid series name: %s" % series_name)

            dictionary["series"] = series

    return render_to_response(
        "people.html",
        dictionary,
        context_instance=RequestContext(request))
