# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Series related views."""

__metaclass__ = type

__all__ = [
    "series",
    ]

from django.http import HttpResponseNotFound
from django.shortcuts import render_to_response
from django.template import RequestContext

from zope.component import getUtility

from lpresults.tracker.interfaces.testpillar import ITestPillarSet


def series(request, pillar_name, series_name=None):
    dictionary = {}
    pillar = getUtility(ITestPillarSet).getByName(pillar_name)
    if pillar is None:
        return HttpResponseNotFound(
            "Invalid pillar name: %s" % pillar_name)

    dictionary["pillar"] = pillar

    # Plural and singular series use the same view method
    if series_name is None:
        template_name = "series.html"
    else:
        series = pillar.getSeries(series_name)
        if series is None:
            return HttpResponseNotFound(
                "Invalid series name: %s" % series_name)

        dictionary["series"] = series
        template_name = "series/index.html"

    return render_to_response(
        template_name,
        dictionary,
        RequestContext(request))
