# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""System related views."""

__metaclass__ = type

__all__ = [
    "system",
    "systems",
    ]

from django.http import HttpResponseNotFound
from django.shortcuts import render_to_response
from django.template import RequestContext

from zope.component import getUtility

from lpresults.tracker.interfaces.testsystem import ITestSystemSet
from lpresults.tracker.interfaces.testpillar import ITestPillarSet


def system(request, system_id):
    system = getUtility(ITestSystemSet).get(int(system_id))
    if system is None:
        return HttpResponseNotFound("Invalid system id: %s" % system_id)

    dictionary = {
        "system": system,
        }

    return render_to_response(
        "system/index.html",
        dictionary,
        RequestContext(request))


def systems(request, pillar_name=None, series_name=None):
    dictionary = {}
    if pillar_name is not None:
        pillar = getUtility(ITestPillarSet).getByName(pillar_name)
        if pillar is None:
            return HttpResponseNotFound(
                "Invalid pillar name: %s" % pillar_name)

        dictionary["pillar"] = pillar

        if series_name is not None:
            series = pillar.getSeries(series_name)
            if series is None:
                return HttpResponseNotFound(
                    "Invalid series name: %s" % series_name)

            dictionary["series"] = series

    return render_to_response(
        "systems.html",
        dictionary,
        context_instance=RequestContext(request))
