# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""Test run related views."""

__metaclass__ = type

__all__ = [
    "testrun",
    "testruns",
    ]

from django.http import HttpResponseNotFound
from django.shortcuts import render_to_response
from django.template import RequestContext

from zope.component import getUtility

from lpresults.tracker.interfaces.testpillar import ITestPillarSet


def testrun(request, pillar_name, series_name, sequence):
    pillar = getUtility(ITestPillarSet).getByName(pillar_name)
    if pillar is None:
        return HttpResponseNotFound(
            "Invalid pillar name: %s" % pillar_name)

    series = pillar.getSeries(series_name)
    if series is None:
        return HttpResponseNotFound(
            "Invalid series name: %s" % series_name)

    sequence = int(sequence)
    testrun = series.searchTestRuns(request.user, sequence=sequence).any()
    if testrun is None:
        return HttpResponseNotFound(
            "Invalid test run sequence: %d" % sequence)

    dictionary = {
        "pillar": pillar,
        "series": series,
        "test_run": testrun,
        }

    return render_to_response(
        "testrun/index.html",
        dictionary,
        RequestContext(request))


def testruns(request, pillar_name, series_name):
    dictionary = {}
    pillar = getUtility(ITestPillarSet).getByName(pillar_name)
    if pillar is None:
        return HttpResponseNotFound(
            "Invalid pillar name: %s" % pillar_name)

    dictionary["pillar"] = pillar

    if series_name:
        series = pillar.getSeries(series_name)
        if series is None:
            return HttpResponseNotFound(
                "Invalid series name: %s" % series_name)

        dictionary["series"] = series

    return render_to_response(
        "testruns.html",
        dictionary,
        RequestContext(request))
