# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "get_logger",
    ]

import re
import sys
import time

import logging
from logging import (
    StreamHandler,
    FileHandler,
    )


class Formatter(logging.Formatter):
    """Formatter encoding our preferred output format."""

    def __init__(self, fmt=None, datefmt=None):
        if fmt is None:
            fmt = '%(asctime)s %(levelname)-7s %(message)s'
        if datefmt is None:
            datefmt = "%Y-%m-%d %H:%M:%S"
        logging.Formatter.__init__(self, fmt, datefmt)
        self.converter = time.gmtime  # Output should be UTC


def reset_logger(logger):
    """Reset the state of the given logger instance."""
    for handler in logger.handlers[:]:
        handler.flush()
        try:
            handler.close()
        except KeyError:
            pass
        logger.removeHandler(handler)


def get_logger(level, log_stream=None, log_file=None, name=None):
    root_logger = logging.getLogger()
    reset_logger(root_logger)

    # Log to the given file or the default stream
    if log_file:
        handler = FileHandler(log_file)
    else:
        handler = StreamHandler(log_stream)

    # Set the level on the handler rather than the logger
    level = logging.getLevelName(level.upper())
    handler.setLevel(level)
    root_logger.setLevel(0)

    # Do the same for the formatter
    formatter = Formatter()
    handler.setFormatter(formatter)
    root_logger.addHandler(handler)

    # Create our own logger
    if name is None:
        name = sys.argv[0]
        name = re.sub(r".py[oc]?$", "", name)

    return logging.getLogger(name)
