# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "DmiSysParser",
    ]

import os

from lpresults.xunit.parsers.dmi import DmiDevice


class DmiSysParser:

    _attribute_map = {
        "sys_vendor": "system_vendor",
        "product_name": "system_name",
        "product_serial": "system_serial",
        }

    def __init__(self, stream):
        self.stream = stream

    def run(self, result):
        attributes = {}
        last_category = None
        for line in self.stream:
            # Skip empty lines
            line = line.strip()
            if not line:
                continue

            path, value = line.split(":", 1)
            key = os.path.basename(path)
            key = self._attribute_map.get(key, key)

            # Skip anything other than board, bios or chassis
            category = key.upper().split("_")[0]
            if category not in ("BOARD", "BIOS", "CHASSIS", "SYSTEM",):
                continue

            # Add device when the category changes
            if category != last_category:
                if last_category is not None:
                    device = DmiDevice(attributes, last_category)
                    result.addDevice(device)
                    attributes = {}

                last_category = category

            attributes[key] = value

        # Add device for the last category
        if attributes:
            device = DmiDevice(attributes, last_category)
            result.addDevice(device)

        return result
