# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from cStringIO import StringIO

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.xunit.parsers.dmesg import DmesgParser


class DmesgResult:

    def __init__(self):
        self.addresses = []

    def addAddress(self, address):
        self.addresses.append(address)


class TestDmesgParser(TestCase):

    def getParser(self, string):
        stream = StringIO(string)
        return DmesgParser(stream)

    def getResult(self, string):
        parser = self.getParser(string)
        result = DmesgResult()
        parser.run(result)
        return result

    def test_noEth(self):
        result = self.getResult("00:00:00:00:00:00")
        self.assertEquals(len(result.addresses), 0)

    def test_noAddress(self):
        result = self.getResult("eth0")
        self.assertEquals(len(result.addresses), 0)

    def test_address(self):
        result = self.getResult("eth0 00:00:00:00:00:00")
        self.assertEquals(len(result.addresses), 1)
        self.assertEquals(result.addresses[0],  "00:00:00:00:00:00")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
