# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "TestDmiMixin",
    ]

from unittest import defaultTestLoader


class DmiResult:

    def __init__(self):
        self.devices = []

    def addDevice(self, device):
        self.devices.append(device)

    def getDevice(self, category):
        for device in self.devices:
            if device.category == category:
                return device

        return None


class TestDmiMixin:

    def getParser(self):
        raise NotImplementedError()

    def getResult(self):
        parser = self.getParser()
        result = DmiResult()
        parser.run(result)
        return result

    def test_devices(self):
        result = self.getResult()
        self.assertEquals(len(result.devices), 4)

    def test_bios(self):
        result = self.getResult()
        device = result.getDevice("BIOS")
        self.assertTrue(device)
        self.assertEquals(device.product, "BIOS PRODUCT")
        self.assertEquals(device.vendor, "BIOS VENDOR")
        self.assertEquals(device.serial, None)

    def test_board(self):
        result = self.getResult()
        device = result.getDevice("BOARD")
        self.assertTrue(device)
        self.assertEquals(device.product, None)
        self.assertEquals(device.vendor, None)
        self.assertEquals(device.serial, None)

    def test_chassis(self):
        result = self.getResult()
        device = result.getDevice("CHASSIS")
        self.assertTrue(device)
        self.assertEquals(device.product, "Notebook")
        self.assertEquals(device.vendor, "CHASSIS VENDOR")
        self.assertEquals(device.serial, None)

    def test_system(self):
        result = self.getResult()
        device = result.getDevice("SYSTEM")
        self.assertTrue(device)
        self.assertEquals(device.product, "SYSTEM PRODUCT")
        self.assertEquals(device.vendor, "SYSTEM VENDOR")
        self.assertEquals(device.serial, "SYSTEM SERIAL")


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
