# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "Protocol",
    "get_protocol",
    ]

import os
import re
import sys

from lpresults import xunit


class Protocol:
    """
    Base class for a protocol.
    """

    def __init__(self, file):
        """Initialize a protocol.

        :param file: File-like object to read content.
        """
        self.file = file

    def run(self, test_run_factory):
        """Process the file and feed its contents to a test run.

        :param test_run_factory: Factory function for creating a test run.
        """


def get_protocol(file):
    """Get the first protocol that matches the given file.

    :param file: File-like object.
    """
    pattern = re.compile(r"^([^_].+)\.py$")

    dirname = os.path.join(os.path.dirname(xunit.__file__), "protocols")
    filenames = os.listdir(dirname)
    matches = [pattern.match(filename) for filename in filenames]
    names = [match.group(1) for match in matches if match]

    protocol = None
    for name in names:
        module_name = "%s.protocols.%s" % (xunit.__name__, name)
        __import__(module_name, None, None, [""])
        module = sys.modules[module_name]
        protocol = module.create_from_file(file)
        if protocol:
            break

    return protocol
