# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import absolute_import

__metaclass__ = type

__all__ = [
    "create_from_file",
    ]

from subunit import ProtocolTestCase

from lpresults.xunit.status import Status


class SubunitResult:

    def __init__(self, test_run_factory, **kwargs):
        self.test_run = test_run_factory(**kwargs)

    def startTest(self, test):
        pass

    def stopTest(self, test):
        pass

    def addError(self, test, error=None, details=None):
        self.test_run.addTestResult(name=test.id(), status=Status.UNRESOLVED)

    def addExpectedFailure(self, test, error=None, details=None):
        self.test_run.addTestResult(name=test.id(), status=Status.XFAIL)

    def addFailure(self, test, details):
        self.test_run.addTestResult(name=test.id(), status=Status.FAIL)

    def addSkip(self, test, reason=None, details=None):
        self.test_run.addTestResult(name=test.id(), status=Status.UNTESTED)

    def addSuccess(self, test, details=None):
        self.test_run.addTestResult(name=test.id(), status=Status.PASS)


class SubunitProtocol(ProtocolTestCase):

    def run(self, test_run_factory, **kwargs):
        result = SubunitResult(test_run_factory, **kwargs)
        super(SubunitProtocol, self).run(result)


def create_from_file(file):
    header = """Tests started at approximately"""
    buffer = file.read(len(header))
    file.seek(0)

    protocol = None
    if buffer == header:
        protocol = SubunitProtocol(file)

    return protocol
