# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = [
    "retry_except",
    ]

import time


def retry_except(ExceptionToCheck, tries=5, delay=1, step=1):
    """Retry decorator.

    :param ExceptionToCheck: Exception to check for retrying.
    :param tries: Number of tries to attempt.
    :param delay: Initial delay in seconds.
    :param step: Increment in seconds for each try.
    """

    def decorator(function):

        def decorated(*args, **kwargs):
            for i in xrange(0, tries - 1):
                try:
                    return function(*args, **kwargs)
                except ExceptionToCheck:
                    time.sleep(delay * (i * step + 1))

            return function(*args, **kwargs)

        return decorated

    return decorator
