# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.xunit.functor import Functor


class TestFunctor(TestCase):

    def test_initCallable(self):

        def foo():
            pass

        functor = Functor(foo)
        self.assertTrue(callable(functor))

    def test_initNotCallable(self):
        self.assertRaises(AssertionError,
            Functor,
            "foo")

    def test_callWithArgs(self):

        def foo(a, b):
            return (a, b)

        functor = Functor(foo, 1, 2)
        self.assertEquals(functor(), (1, 2))

    def test_callWithKwargs(self):

        def foo(a=1, b=2):
            return (a, b)

        functor = Functor(foo, a=3, b=4)
        self.assertEquals(functor(), (3, 4))
        self.assertEquals(functor(a=4), (4, 4))
        self.assertEquals(functor(b=5), (3, 5))
        self.assertEquals(functor(a=4, b=5), (4, 5))


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
