# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

__all__ = []

from optparse import (
    OptionParser,
    OptionValueError,
    )
from unittest import (
    defaultTestLoader,
    TestCase,
    )

from lpresults.xunit.option import (
    check_date,
    DateOption,
    )
from lpresults.xunit.testing.cases import ApplicationTestCase


class TestCheckDate(TestCase):

    def test_valid(self):
        check_date(None, "-d", "2010/01/01")

    def test_invalid(self):
        self.assertRaises(
            OptionValueError, check_date, None, "-d", "2010-01-01")
        self.assertRaises(
            OptionValueError, check_date, None, "-d", "20100101")


class TestDateOption(ApplicationTestCase):

    def setUp(self):
        super(TestDateOption, self).setUp()
        self.parser = OptionParser(option_class=DateOption)
        self.parser.add_option("--date", type="date")

    def test_valid(self):
        self.parser.parse_args(["--date=2010/01/01"])

    def test_invalid(self):
        self.assertRaises(SystemExit,
            self.parser.parse_args,
            ["--date=20100101"])


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)
