#!/usr/bin/env python
#
# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import re

from glob import glob
from os import path

from distutils.command.install_data import install_data

import ez_setup
ez_setup.use_setuptools()

from setuptools import (
    setup,
    find_packages,
    )


LPRESULTS_CONFIG = os.environ.get("LPRESULTS_CONFIG", "default")


class custom_install_data(install_data, object):

    def finalize_options(self):
        """Add wildcard and recursion support for filenames."""
        super(custom_install_data, self).finalize_options()

        for data_file in self.data_files:
            if type(data_file) != str:
                files = data_file[1]
                i = 0
                while i < len(files):
                    if "*" in files[i]:
                        for e in glob(files[i]):
                            files.append(e)
                        files.pop(i)
                        i -= 1
                    if path.isdir(files[i]):
                        subdir = path.join(
                            data_file[0], path.basename(files[i]))
                        subfiles = [path.join(files[i], name)
                            for name in os.listdir(files[i])]
                        self.data_files.append((subdir, subfiles))
                        files.pop(i)
                        i -= 1
                    i += 1


def changelog_version(changelog="debian/changelog"):
    """
    Return the latest version taken from the debian changelog file
    for use in `version` in a distutils `setup.py` file.

    :param changelog: Path to changelog file.
    """
    version = "dev"
    if path.exists(changelog):
        file = open(changelog)
        try:
            head = file.readline()
        finally:
            file.close()

        match = re.compile(".*\((.*)\).*").match(head)
        if match:
            version = match.group(1)

    return version


SETUP_NAME = "lpresults"
SETUP_VERSION = changelog_version()

LPRESULTS_CONF_DIR = path.join("..", "etc", "lpresults")
LPRESULTS_DATA_DIR = path.join("share", "lpresults")


dist = setup(
    name=SETUP_NAME,
    version=SETUP_VERSION,
    namespace_packages=["lpresults"],
    include_package_data=True,
    packages=find_packages("lib"),
    package_dir={
        "": "lib",
    },
    scripts=[
        "tools/results2launchpad",
        "tools/results2local",
        "tools/launchpad2results",
    ],
    data_files=[
        (LPRESULTS_CONF_DIR, [
            path.join("configs", LPRESULTS_CONFIG, "*.conf"),
            ]),
        (path.join(LPRESULTS_DATA_DIR, "scripts"), [
            "scripts/*",
            ]),
        (path.join(LPRESULTS_DATA_DIR, "templates"), [
            "templates/*",
            ]),
        (path.join(LPRESULTS_DATA_DIR, "static"), [
            "static/*.txt",
            ]),
        (path.join(LPRESULTS_DATA_DIR, "static", "build"), [
            "static/build/*.css",
            "static/build/*.js",
            "static/build/icon-sprites*",
            ]),
        (path.join(LPRESULTS_DATA_DIR, "static", "launchpad"), [
            "static/launchpad/favicon.ico",
            ]),
        (path.join(LPRESULTS_DATA_DIR, "static", "launchpad", "images"), [
            "static/launchpad/images/*.png",
            ]),
    ],
    cmdclass={
        "install_data": custom_install_data,
    },

    # Setuptools metadata.
    zip_safe=False,
    install_requires=[
        "storm",
        "cssutils",
        "django",
        "httplib2",
        "python-openid",
        "python-subunit",
        "launchpadlib",
        "lazr.enum",
        "lazr.restful",
        "lazr.restfulclient",
        "lazr.uri",
        "lazr-js",
        "oauth",
        "RestrictedPython",
        "setuptools",
        "simplejson",
        "wadllib",
        "zope.component",
        "zope.interface",
        "zope.schema",
        "zope.security",
        "zope.testing",
    ],
    package_data={
        "": [
            "*.xsl",
            "*.zcml",
        ],
    },

    # PyPI metadta.
    author="Marc Tardif",
    author_email="marc.tardif@canonical.com",
    license="Affero GPL v3",
    url="https://launchpad.net/launchpad-results",
    download_url="https://launchpad.net/launchpad-results/+download",
    )
