/* Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 *
 * The Action module provides an abstraction layer to run client
 * requests in a deferred context.
 *
 * @module lpresults.command
 * @submodule action
 */
YUI.add('lpresults.command.action', function(Y) {

    var namespace = Y.namespace('lpresults.command');

    /**
     * The Action class provides the ability to run a function and get
     * a Deferred on which additional callbacks can be defined.
     *
     * @class Action
     * @extends Base
     * @constructor
     */
    var Action = function(config) {
        Action.superclass.constructor.apply(this, arguments);
    };

    Action.NAME = 'action';
    Action.ATTRS = {
        /**
         * The client function to use. If not defined, one will be created
         * during initialization.
         *
         * @attribute client
         */
        client: {
            value: null
        }
    };

    Y.extend(Action, Y.Base, {
        /**
         * Run the client with the given config. Return a Deferred firing
         * with the result of the action, or failing with an error if the
         * action failed.
         *
         * @method run
         * @public
         */
       run: function(config) {
            var deferred = new Y.lpresults.command.Deferred();

            var client = this.get('client');
            if (client !== null) {
                try {
                    result = client(config);
                    this.runSuccess.apply(this, [deferred, result]);
                } catch (error) {
                    this.runFailure.apply(this, [deferred, error]);
                }
            }
 
            return deferred;
        },

        /**
         * Handle a failed run action.
         *
         * @method runFailure
         * @protected
         */
        runFailure: function(deferred, result) {
            deferred.errback(result);
        },

        /**
         * Handle a successful run action.
         *
         * @method runSuccess
         * @protected
         */
        runSuccess: function(deferred, result) {
            deferred.callback(result);
        }
    });

    /*
     * Ensure that Action is available within the command namespace.
     */
    namespace.Action = Action;

}, '0.1', {'requires': ['base', 'lpresults.command.deferred']});
