/* Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 *
 * The LPBatching module provides a class for paginating the results
 * from running an action.
 *
 * @module lpresults.lp
 * @submodule batching
 */
YUI.add('lpresults.lp.batching', function(Y) {

    var namespace = Y.namespace('lpresults.lp');

    var DEFAULT_CONTENT_SIZE = 75;

    /**
     * The LPBatching class provides the ability to plugin functionality into
     * a DOM tree so that it can update itself when results are returned
     * from an action.
     *
     * @class LPBatching
     * @extends Widget
     * @constructor
     */
    var LPBatching = function() {
        LPBatching.superclass.constructor.apply(this, arguments);
    };

    Y.mix(LPBatching, {
        NAME: 'lpbatching',
        ATTRS: {
            /**
             * The element to initialize BatchingPlugin.
             *
             * @attribute controlBox
             * @type Node
             */
            controlBox: {
                value: null
            }
        }
    });

    Y.extend(LPBatching, Y.lpresults.lp.LPContent, {
        /**
         * Get the batching content to render.
         *
         * @method getContent
         * @protected
         */
        getContent: function(action) {
            var content = new Y.lpresults.ui.Batching({
                action: action,
                sizeParameter: 'ws.size',
                startParameter: 'ws.start',
                contentBox: this.get('contentBox'),
                contentSize: DEFAULT_CONTENT_SIZE
                });
            content.plug(Y.lpresults.ui.BatchingPlugin, {
                controlBox: this.get('controlBox')
                });

            return content;
        }
    });

    /*
     * Ensure that LPBatching is available within the lp namespace.
     */
    namespace.LPBatching = LPBatching;

}, '0.1', {'requires': ['lpresults.ui.batching', 'lpresults.lp.content']});
