/* Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 *
 * The LPContent module provides a class for rendering a list of results
 * in a DOM tree.
 *
 * @module lpresults.lp
 * @submodule content
 */
YUI.add('lpresults.lp.content', function(Y) {

    var namespace = Y.namespace('lpresults.lp');

    /**
     * The LPContent class provides the ability to plugin functionality into
     * a DOM tree so that it can update itself when results are returned
     * from an action.
     *
     * @class LPContent
     * @extends Widget
     * @constructor
     */
    var LPContent = function() {
        LPContent.superclass.constructor.apply(this, arguments);
    };

    Y.mix(LPContent, {
        NAME: 'lpcontent',
        ATTRS: {
            /**
             * The uri to initialize LPAction.
             *
             * @attribute uri
             * @type String
             */
            uri: {
                value: null
            },

            /**
             * The operation name to initialize LPAction.
             *
             * @attribute operationName
             * @type String
             */
            operationName: {
                value: null
            }
        }
    });

    Y.extend(LPContent, Y.Widget, {
        /**
         * Get the action to initialize the content.
         *
         * @method getAction
         * @protected
         */
        getAction: function() {
            return new Y.lpresults.lp.LPAction({
                uri: this.get('uri'),
                operationName: this.get('operationName')
                });
        },

        /**
         * Get the content to render.
         *
         * @method getContent
         * @protected
         */
        getContent: function(action) {
            return new Y.lpresults.ui.Content({
                action: action,
                contentBox: this.get('contentBox')
                });
        },

        /**
         * Render the content widget.
         *
         * @method renderUI
         * @protected
         */
        renderUI: function() {
            var action = this.getAction.apply(this);
            var content = this.getContent.apply(this, [action]);
            content.render();
        }
    });

    /*
     * Ensure that LPContent is available within the lp namespace.
     */
    namespace.LPContent = LPContent;

}, '0.1', {'requires': ['widget', 'lpresults.ui.content', 'lpresults.lp.action']});
