/* Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 * GNU Affero General Public License version 3 (see the file LICENSE).
 *
 * The LPListing module provides a class for listing the results from
 * running an action.
 *
 * @module lpresults.lp
 * @submodule listing
 */
YUI.add('lpresults.lp.listing', function(Y) {

    var namespace = Y.namespace('lpresults.lp');

    var DEFAULT_CONTENT_SIZE = 5;

    /**
     * The LPListing class provides the ability to plugin functionality into
     * a DOM tree so that it can update itself when results are returned
     * from an action.
     *
     * @class LPListing
     * @extends Widget
     * @constructor
     */
    var LPListing = function() {
        LPListing.superclass.constructor.apply(this, arguments);
    };

    Y.mix(LPListing, {
        NAME: 'lplisting',
        ATTRS: {
            /**
             * The all link to initialize Listing. Default is the action uri.
             *
             * @attribute allLink
             * @type String
             */
            allLink: {
                valueFn: function() {
                    return this.get('uri');
                }
            },

            /**
             * The all text to initialize ListingPlugin. Default is 'All'.
             *
             * @attribute allText
             * @type String
             */
            allText: {
                value: 'All'
            },

            /**
             * The element to initialize ListingPlugin.
             *
             * @attribute controlBox
             * @type Node
             */
            controlBox: {
                value: null
            }
        }
    });

    Y.extend(LPListing, Y.lpresults.lp.LPContent, {
        /**
         * Get the listing content to render.
         *
         * @method getContent
         * @protected
         */
        getContent: function(action) {
            var content = new Y.lpresults.ui.Listing({
                action: action,
                allLink: this.get('allLink'),
                contentBox: this.get('contentBox'),
                contentSize: DEFAULT_CONTENT_SIZE
                });
            content.plug(Y.lpresults.ui.ListingPlugin, {
                controlBox: this.get('controlBox'),
                allClass: 'see-all',
                allText: this.get('allText')
                });

            return content;
        }
    });

    /*
     * Ensure that LPListing is available within the lp namespace.
     */
    namespace.LPListing = LPListing;

}, '0.1', {'requires': ['lpresults.ui.listing', 'lpresults.lp.content']});
