/* Copyright (c) 2009-2010, Canonical Ltd. All rights reserved. */

YUI({
    base: '../../../build/yui/',
    filter: 'raw', combine: false, fetchCSS: false
    }).use('test', 'console', 'lpresults.ui.formatter', function(Y) {

    var fixedDate = new Date(2006, 0, 25, 12, 0, 0);

    // Extend the base DateFormatter to return a fixed date for now.
    var TestDateFormatter = function() {
        TestDateFormatter.superclass.constructor.apply(this, arguments);
    };

    TestDateFormatter.NAME = 'test_date_formatter';

    Y.extend(TestDateFormatter, Y.lpresults.ui.DateFormatter, {
        _now: function() {
            return fixedDate;
        }
    });

    var suite = new Y.Test.Suite('formatter Tests');

    // Test the time method.
    suite.add(new Y.Test.Case({
        name: 'time',

        test_time: function() {
            var formatter = new TestDateFormatter({date: fixedDate});
            Y.Assert.areSame(formatter.time(), '12:00:00');
        }
    }));

    // Test the date method.
    suite.add(new Y.Test.Case({
        name: 'date',

        test_date: function() {
            var formatter = new TestDateFormatter({date: fixedDate});
            Y.Assert.areSame(formatter.date(), '2006-01-25');
        }
    }));

    // Test the datetime method.
    suite.add(new Y.Test.Case({
        name: 'datetime',

        test_datetime: function() {
            var formatter = new TestDateFormatter({date: fixedDate});
            Y.Assert.areSame(formatter.datetime(), '2006-01-25 12:00:00');
        }
    }));

    // Test the displayDate method.
    suite.add(new Y.Test.Case({
        name: 'displayDate',

        test_inDay: function() {
            var date = new Date(fixedDate);
            date.setDate(date.getDate() + 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.displayDate(), 'in 1 day');
        },

        test_inDays: function() {
            var date = new Date(fixedDate);
            date.setDate(date.getDate() + 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.displayDate(), 'on 2006-01-30');
        },

        test_daysAgo: function() {
            var date = new Date(fixedDate);
            date.setDate(date.getDate() - 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.displayDate(), 'on 2006-01-20');
        }
    }));

    // Test the approximateDate method.
    suite.add(new Y.Test.Case({
        name: 'approximateDate',

        test_inSecond: function() {
            var date = new Date(fixedDate);
            date.setSeconds(date.getSeconds() + 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), 'in 1 second');
        },

        test_inSeconds: function() {
            var date = new Date(fixedDate);
            date.setSeconds(date.getSeconds() + 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), 'in 5 seconds');
        },

        test_secondAgo: function() {
            var date = new Date(fixedDate);
            date.setSeconds(date.getSeconds() - 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '1 second ago');
        },

        test_secondsAgo: function() {
            var date = new Date(fixedDate);
            date.setSeconds(date.getSeconds() - 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '5 seconds ago');
        },

        test_inMinute: function() {
            var date = new Date(fixedDate);
            date.setMinutes(date.getMinutes() + 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), 'in 1 minute');
        },

        test_inMinutes: function() {
            var date = new Date(fixedDate);
            date.setMinutes(date.getMinutes() + 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), 'in 5 minutes');
        },

        test_minuteAgo: function() {
            var date = new Date(fixedDate);
            date.setMinutes(date.getMinutes() - 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '1 minute ago');
        },

        test_minutesAgo: function() {
            var date = new Date(fixedDate);
            date.setMinutes(date.getMinutes() - 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '5 minutes ago');
        },

        test_inHour: function() {
            var date = new Date(fixedDate);
            date.setHours(date.getHours() + 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), 'in 1 hour');
        },

        test_inHours: function() {
            var date = new Date(fixedDate);
            date.setHours(date.getHours() + 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), 'in 5 hours');
        },

        test_hourAgo: function() {
            var date = new Date(fixedDate);
            date.setHours(date.getHours() - 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '1 hour ago');
        },

        test_hoursAgo: function() {
            var date = new Date(fixedDate);
            date.setHours(date.getHours() - 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '5 hours ago');
        },

        test_inDay: function() {
            var date = new Date(fixedDate);
            date.setDate(date.getDate() + 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), 'in 1 day');
        },

        test_inDays: function() {
            var date = new Date(fixedDate);
            date.setDate(date.getDate() + 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '2006-01-30');
        },

        test_dayAgo: function() {
            var date = new Date(fixedDate);
            date.setDate(date.getDate() - 1);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '1 day ago');
        },

        test_daysAgo: function() {
            var date = new Date(fixedDate);
            date.setDate(date.getDate() - 5);

            var formatter = new TestDateFormatter({date: date});
            Y.Assert.areSame(formatter.approximateDate(), '2006-01-20');
        }
    }));

    // Lock, stock, and two smoking barrels.
    var handle_complete = function(data) {
        window.status = '::::' + JSON.stringify(data);
        };
    Y.Test.Runner.on('complete', handle_complete);
    Y.Test.Runner.add(suite);

    var console = new Y.Console({newestOnTop: false});
    console.render('#log');

    Y.on('domready', function() {
        Y.Test.Runner.run();
        });
});
