// ==UserScript==
// @name           Launchpad Results
// @namespace      https://launchpad.net/launchpad-results
// @description    (Launchpad) Results
// @include        {{ LAUNCHPAD_RESULTS_WEB_ROOT }}*
// @include        {{ LAUNCHPAD_WEB_ROOT }}*
// @include        {{ LAUNCHPAD_WEB_ALL }}*
// ==/UserScript==

(function () {
    var SCRIPT = {
        name: "Launchpad Results",
        namespace: "https://launchpad.net/launchpad-results",
        description: "(Launchpad) Results",
        source: "https://code.launchpad.net/launchpad-results",
        version: "0.1",
        date: (new Date(2010, 10 - 1, 04))// update date
        .valueOf()
    };

    var launchpad_web_root = "{{ LAUNCHPAD_WEB_ROOT }}";
    var launchpad_results_service_root = "{{ LAUNCHPAD_RESULTS_SERVICE_ROOT }}";
    var launchpad_results_web_root = "{{ LAUNCHPAD_RESULTS_WEB_ROOT }}";

    /**
     * Evaluate an xpath which returns all links on a page.
     *
     * For example:
     *
     *     xpath("//*[@class='facetmenu']").snapshotItem(0);
     *
     * @function xpath
     */
    function xpath(query, context) {
        context = context ? context : document;
        return document.evaluate(
            query,
            context,
            null,
            XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
            null);
    }

    /**
     * Format a date into a readable string.
     *
     * For example:
     *
     *     formatDate("2011-04-22T19:14:28.262254");
     *
     * @function formatDate
     */
    function formatDate(date) {
        return date.replace("T", " ").replace(/\..*/, "");
    }

    /**
     * Get the pillar name from the current document location.
     *
     * For example:
     *
     *     getPillar();
     *
     * @function getPillar
     */
    function getPillar(location) {
        if (!location)
            location = document.location;

        var pathname = location.pathname;
        if (!pathname)
            return "";

        var pillar = pathname.replace("/", "").replace(/\/.*/, "");
        if (pillar.charAt(0) == "+")
            return "";

        return pillar;
    }

    /**
     * Get a URL and execute the given callback when ready.
     *
     * For example:
     *
     *     getURL(
     *         launchpad_results_service_root + "launchpad/date_last_tested",
     *         cb);
     *
     * @function getURL
     */
    function getURL(url, cb) {
      GM_xmlhttpRequest({
        method: "GET",
         url: url,
         headers: {
            "User-agent": "Mozilla/4.0 (compatible) Greasemonkey",
            "Accept": "application/json",
         },
         onload: function(xhr) {
            if (xhr.status != 200) {
                    alert("Failed to retrieve " + url + "\n" + xhr.statusText);
                    return -1;
            }
            var data = eval("(" + xhr.responseText + ")");
            cb(data);
         },
         onerror: function(xhr) {
            alert("Failed getURL " + url + "\n" + responseDetails.status +
                    " " + responseDetails.statusText + "\n\n");
         }
      });
    }

    /**
     * Create a factemenu list item.
     *
     * For example:
     *
     *     <li class="answers" title="Launchpad Answer Tracker"><a
     *     href="https://answers.launchpad.net/">Answers</a></li>
     *
     * @function createFacetmenuItem
     */
    function createFacetmenuItem(text, href, attrs) {
        var text_node = document.createTextNode(text);

        var a = document.createElement("a");
        a.appendChild(text_node);
        a.setAttribute("href", href);

        var li = document.createElement("li");
        li.appendChild(a);

        for (var key in attrs) {
            li.setAttribute(key, attrs[key]);
        }

        return li;
    }

    /**
     * Create a test run list item.
     *
     * For example:
     *
     * <li>2010-01-01 12:12:12</li>
     *
     * @function createTestRunItem
     */
    function createTestRunItem(test_run) {
        var date = formatDate(test_run.date_created);
        var text_node = document.createTextNode(date);

        var li = document.createElement("li");
        li.appendChild(text_node);

        return li;
    }

    /**
     * Create a latest test runs portlet.
     *
     * @function createLatestTestRunsPortlet
     */
    function createLatestTestRunsPortlet() {
        var text_node = document.createTextNode("Latest test runs");

        var h2 = document.createElement("h2");
        h2.appendChild(text_node);

        var div = document.createElement("div");
        div.setAttribute("class", "portlet");
        div.setAttribute("id", "portlet-latest-test-runs");
        div.appendChild(h2);

        return div;
    }

    /**
     * Update the Launchpad facetmenu to include the Results list item.
     *
     * @function updateLaunchpadFacetmenu
     */
    function updateLaunchpadFacetmenu(facetmenu) {
        var pillar = getPillar();
        var item = createFacetmenuItem("Results",
            launchpad_results_web_root + pillar, {
                "class": "bugs",
                "title": "Launchpad Results Tracker"
                });
        facetmenu.appendChild(item);
    }

    /**
     * Update the Launchpad Results facetmenu to include all the Launchpad
     * list items.
     *
     * @function updateLaunchpadFacetmenu
     */
    function updateLaunchpadresultsFacetmenu(facetmenu) {
        var pillar = getPillar();
        if (pillar) {
            var new_overview = createFacetmenuItem("Overview",
                launchpad_web_root + pillar, {
                    "class": "overview",
                    });
            var old_overview = facetmenu.querySelector(".overview");
            facetmenu.replaceChild(new_overview, old_overview);
        }

        var items = [];
        items.push(createFacetmenuItem("Code",
            launchpad_web_root.replace("//", "//code.") + pillar, {
                "class": "branches",
                "title": "The Code Bazaar"
                }));
        items.push(createFacetmenuItem("Bugs",
            launchpad_web_root.replace("//", "//bugs.") + pillar, {
                "class": "bugs"
                }));
        items.push(createFacetmenuItem("Blueprints",
            launchpad_web_root.replace("//", "//blueprints.") + pillar, {
                "class": "specifications",
                "title": "Launchpad feature specification tracker."
                }));
        items.push(createFacetmenuItem("Translations",
            launchpad_web_root.replace("//", "//translations.") + pillar, {
                "class": "translations"
                }));
        items.push(createFacetmenuItem("Answers",
            launchpad_web_root.replace("//", "//answers.") + pillar, {
                "class": "answers",
                "title": "Launchpad Answer Tracker"
                }));

        var results = facetmenu.querySelector(".results");
        for (var i in items) {
            facetmenu.insertBefore(items[i], results);
        }
    }

    /**
     * Update the facetmenu liste items depending on the location.
     *
     * @function updateFacetmenu
     */
    function updateFacetmenu() {
        var facetmenu = xpath("//*[@class='facetmenu']").snapshotItem(0);
        if (facetmenu !== null) {
            var location = document.location.toString();
            if (location.match(launchpad_results_web_root)) {
                updateLaunchpadresultsFacetmenu(facetmenu);
            }
            else {
                updateLaunchpadFacetmenu(facetmenu);
            }
        }
    }

    /**
     * Update project information in Launchpad
     *
     * @function updateLastestTestRuns
     */
    function updateLastestTestRuns() {
        var location = document.location.toString();
        if (location.match(launchpad_web_root)) {
            var latest_bugs = xpath("//*[@id='portlet-latest-bugs']").snapshotItem(0);
            if (latest_bugs) {
                var latest_test_runs_items = document.createElement("ul");
                var latest_test_runs_portlet = createLatestTestRunsPortlet();
                latest_test_runs_portlet.appendChild(latest_test_runs_items);

                var parent_div = latest_bugs.parentNode.parentNode;
                parent_div.appendChild(latest_test_runs_portlet);

                var pillar = getPillar();
                var url = launchpad_results_service_root + pillar + "?ws.op=getLatestTestRuns";
                getURL(url, function(data) {
                    for (var i in data.entries) {
                        var test_run_item = createTestRunItem(data.entries[i]);
                        latest_test_runs_items.appendChild(test_run_item);
                    }
                });
            }
        }
    }

    window.addEventListener("load", function(e) {
        updateFacetmenu();
        updateLastestTestRuns();
    }, false);

})();
